/*==================================================================
    Author : Dmitri Bandourin

    created: March 1999

    Purpose: 
    This subroutine defines new GEDE parameters
    (rot matrix, coordinates of GEDE elements) after applying
    global alignment constants for a subdetector:
    rotation matrix M[3][3], 3 translations, shearing.
====================================================================*/

#include <iostream.h>
#include <stdlib.h>
#include <math.h>
#include <stdio.h>

#define EXT1 extern
#define EXT2 
#include "alca/ReadGlAlDB.h"
#include "alca/FillGD.h"
	   

void acGlobAlAlign(float rmin[3],float rmax[3],float rsmn[2],float rsmx[2],
     float cos,float sin,float rot[3][3]) 
{
 
     float M[3][3],dR[3],T[3],FZ,dRmn[3],dRmx[3],Z0,R0[3];
     float XMIN =0.0,XMAX =0.0,YMIN =0.0,YMAX =0.0,ZMIN =0.0,ZMAX =0.0, 
     SXMN =0.0,SXMX =0.0,SYMN =0.0,SYMX =0.0;
     double sum, ML_[3][3], ML[3][3]; 
     int i,j,k;


// rotation matrix:
     for( i = 0; i <= 2; i++) {
// translation vector:
       dR[i] = ReadGlAlDB.dR[i];
       for( j = 0; j <= 2; j++) {
         M[i][j] =  ReadGlAlDB.M[i][j];  
       }
     } 
// Z ref. point:
     Z0 = ReadGlAlDB.Z0;
// rescaling factor:  
     FZ = ReadGlAlDB.FZ; 
//shearings:
     T[0] = ReadGlAlDB.TX;
     T[1] = ReadGlAlDB.TY;
     T[2] = 0.0;

     R0[0] = 0.; R0[1] = 0.; R0[2] = Z0;  
 
//... Combined rotation: new ROT[][] = Rph(cos,sin)*rot[][] (MS->LS->WS).
     float Rph[3][3],ROT[3][3];

     Rph[0][0] = Rph[1][1] = cos;
     Rph[0][1] = -sin; 
     Rph[1][0] = sin;
     Rph[0][2] = Rph[1][2] = Rph[2][0] = Rph[2][1] = 0.0;
     Rph[2][2] = 1.0;
 
     for( i = 0; i <= 2; i++) {
       for( j = 0; j <= 2; j++) {
         sum = 0.0;
         for( k = 0; k <= 2; k++) {
           sum += Rph[i][k]*rot[k][j];
         }
         ROT[i][j] = sum;
       }
     }

//... New presentation for M[][] in a wire system (WS): ML[][].
//... ML = ROT{-1}*M*ROT
     for( i = 0; i <= 2; i++) {
       for( j = 0; j <= 2; j++) {
         sum = 0.0;
         for( k = 0; k <= 2; k++) {
           sum += M[i][k]*ROT[k][j];
         }
         ML_[i][j] = sum;
       }
     }
     for( i = 0; i <= 2; i++) {
       for( j = 0; j <= 2; j++) {
         sum = 0.0;
         for( k = 0; k <= 2; k++) {
           sum += ROT[k][i]*ML_[k][j];
         }
         ML[i][j] = sum;
       }
     }

//... Define translations caused by shearing:
     for(int i = 0; i <= 2; i++) {   
       dRmn[i] = (rmin[3] - Z0)*T[i];
       dRmx[i] = (rmax[3] - Z0)*T[i];
     }
//...Define new GEDE parameters.
 
//... firstly rescaling, than - rotation (no commutation!)
//... new rmin[3], rmax[3] after rescaling. 
     rmin[3] = (rmin[3] - Z0)*FZ + Z0;
     rmax[3] = (rmax[3] - Z0)*FZ + Z0;

     for(int i = 0; i <= 2; i++) {   
        XMIN += ROT[0][i]*(dR[i] + dRmn[i]) + 
                ML[0][i]*(rmin[i] - R0[i]);
        XMAX += ROT[0][i]*(dR[i] + dRmn[i]) + 
                ML[0][i]*(rmax[i] - R0[i]);
        YMIN += ROT[1][i]*(dR[i] + dRmn[i]) + 
                ML[1][i]*(rmin[i] - R0[i]);
        YMAX += ROT[1][i]*(dR[i] + dRmn[i]) + 
                ML[1][i]*(rmax[i] - R0[i]);
        ZMIN += ROT[2][i]*(dR[i] + dRmn[i]) + 
                ML[2][i]*(rmin[i] - R0[i]) + R0[i];
        ZMAX += ROT[2][i]*(dR[i] + dRmn[i]) + 
                ML[2][i]*(rmax[i] - R0[i]) + R0[i];
     }
     for(int i = 0; i <= 1; i++) {   
        SXMN += ROT[0][i]*(dR[i] + dRmn[i]) + ML[0][i]*rsmn[i];
        SXMX += ROT[0][i]*(dR[i] + dRmn[i]) + ML[0][i]*rsmx[i];
        SYMN += ROT[1][i]*(dR[i] + dRmn[i]) + ML[1][i]*rsmn[i];
        SYMX += ROT[1][i]*(dR[i] + dRmn[i]) + ML[1][i]*rsmx[i];
     }

     FillGD.XMIN = XMIN; FillGD.XMAX = XMAX;
     FillGD.YMIN = YMIN; FillGD.YMAX = YMAX;
     FillGD.ZMIN = ZMIN; FillGD.ZMAX = ZMAX;
     FillGD.SXMN = SXMN; FillGD.SXMX = SXMX;
     FillGD.SYMN = SYMN; FillGD.SYMX = SYMX;

//...Define new rotation matrix for GEDE: rot -> Rot = rot*M.
     for( i = 0; i <= 2; i++) {
       for( j = 0; j <= 2; j++) {
         sum = 0.0;
         for( k = 0; k <= 2; k++) {
           sum += rot[i][k]*M[k][j];
         }
         FillGD.Rot[i][j] = sum;
       }
     }
}


