// ********************************************************************
//
//  Routine:      acReadGlobal
//  Author:       T.Oest
//
//  Description:  read global alignment constants from database
//                
//
//  Modification:
//
// ********************************************************************


#include <iostream.h>
#include <stdlib.h>
#include <math.h>
#include <algo.h>
#include <limits.h>
#include "arte/arte.hh"
#include "database/table.h"

#include "online/esp.h"

extern "C" {
 void kuexec_(char * Command, int Length);
}

void ackuexec(char * KuipCommand);


static int first        = 0;
static int nextMajor    = 0;
static int actualMajor  = 0;
static int maxMajor     = 0;
static int minMajor     = 0;

//*****************************************************************
// function
//*****************************************************************

extern int Db_dbr_last_major, Db_dbr_last_minor, Db_dbr_empty;

int acReadGlobal() {

//==================================================================
// no action for MC
//==================================================================

  if (ArteTable<MTRA>::size() > 0 ) return 0;

//==================================================================
// database server (default: /SETUP_ALIGN)
//                  can be changed by setting DB_GLOBAL_ALIGNMENT
//==================================================================

 char defaultserver[13];
 sprintf(defaultserver,"/SETUP_ALIGN");
 char * dbserver;

 if (getenv("DB_GLOBAL_ALIGNMENT") == NULL){
   dbserver = defaultserver;
 } else {
   dbserver = getenv("DB_GLOBAL_ALIGNMENT");
 };

//==================================================================
// event UNIX time
//==================================================================

  int EventUnixTime = 0;

  if (ArteTable<DDAQ>::size() > 0){ 
   ArtePointer<DDAQ> ddaq( 2 );

   EVT_DATA *p = (EVT_DATA *) &ddaq->bits;
   ESP_get_int( &p->event_time, &EventUnixTime, 1 );
  };

  if (EventUnixTime == 0) return -1;

  //  cout << " event time " << EventUnixTime << endl;
//==================================================================
// Initialisation: get first tabel
//==================================================================

  char LunName[5] = {'D','B','G','A',0};
  char * KuipCommand =  (char*) malloc(max(128,
                                       int(24+strlen(dbserver))));

  if (first == 0){
   first = 1;  

   ArteTable<ACGA>::clear();
   sprintf (KuipCommand,"IOFILE %s %s",LunName,dbserver);
   ackuexec(KuipCommand);
   sprintf (KuipCommand,"dbopen %s ro",LunName);
   ackuexec(KuipCommand);
   sprintf (KuipCommand,"dbread %s ACGA 0 0 gt",LunName);
   ackuexec(KuipCommand);

   if (ArteTable<ACGA>::size() != 0){
    nextMajor = Db_dbr_last_major;  } else {
    nextMajor = 0;
   };
    minMajor = nextMajor;

   ArteTable<ACGA>::clear();
   sprintf (KuipCommand,"dbread %s ACGA %i 0 le",LunName,INT_MAX);
   ackuexec(KuipCommand);

   if (ArteTable<ACGA>::size() != 0){
    maxMajor = Db_dbr_last_major;   } else {
    maxMajor = 0;
   };

   ArteTable<ACGA>::clear();
   sprintf (KuipCommand,"dbclose %s",LunName);
   ackuexec(KuipCommand);

   //   cout << " first major number " <<  nextMajor << endl;
   //   cout << " max   major number " <<  maxMajor << endl;
   //   cout << " min   major number " <<  minMajor << endl;

  };
   

//==================================================================
// read in new ACGA table if table not existing or update necessay
//==================================================================

  if (EventUnixTime < minMajor) {
   ArteTable<ACGA>::clear();
   return -2;
  };

  if ( ArteTable<ACGA>::size() == 0 || 
      (EventUnixTime >= nextMajor  && actualMajor != maxMajor  ) || 
       EventUnixTime < actualMajor){

   int major;

   major = EventUnixTime + 1;

   sprintf (KuipCommand,"dbopen %s ro",LunName);
   ackuexec(KuipCommand);

   if (EventUnixTime < maxMajor){
    ArteTable<ACGA>::clear();
    sprintf (KuipCommand,"dbread %s ACGA %i 0 ge",LunName,major);
    ackuexec(KuipCommand);

    if (ArteTable<ACGA>::size() != 0){
     nextMajor = Db_dbr_last_major;  } else {
     nextMajor = 0;
    };
   } else {
    nextMajor = maxMajor;
   };
   //   cout << " nextMajor: " <<  nextMajor << endl;

   ArteTable<ACGA>::clear();
   sprintf (KuipCommand,"dbread %s ACGA %i 0 lt",LunName,major);
   ackuexec(KuipCommand);

   actualMajor = Db_dbr_last_major;
   //   cout << " actualMajor: " <<  actualMajor << endl;

   sprintf (KuipCommand,"dbclose %s",LunName);
   ackuexec(KuipCommand);
   
  };

//==================================================================
// the end
//==================================================================

 if (ArteTable<ACGA>::size() == 0) return -3;

 return 0;
}
