//                              -*- Mode: C++ -*-
//: EAClusTrackInfo.C
//
//
// Author: Mauro Villa   
// ECAL group 
// HERA-B experiment, Desy
// e-mail: villa@bo.infn.it
//
// Created 2/5/97
// Last Update ........
//

#ifdef HIST
#  include "ntuple.hh"
#endif
#include "care/EMClusterTree.hh"
#include "care/EMHitCell.hh"
#include "care/EAClusTrackInfo.hh"
#include "arte/ArteMessage.hh"
#include "care/ECLastHit.hh"
#undef HIST
#ifdef HIST
#  include "MCtruthAnalyzer.hh"
#endif
#include "care/EMTrackOnEcal.hh"


//////////////////// ClusTrackInfo Class Definitions ////////////////

// No functions
EAClusTrackInfo::EAClusTrackInfo(const EMCluster* clus) : track(0) {
  clus->analyzeMe(*this);
}

EAClusTrackInfo::~EAClusTrackInfo(){}
//--
//  Base function to be called with the TopCluster List
//
void EAClusTrackInfo::doAnalysis(const EMClusterTree * tree){
  track = 0;
  for(const EMClusterMember* child = tree->begin(); child!=tree->end(); 
      child = child->next()){
      child->analyzeMe(*this); 
  }
}

//--
//  Base function to be called with the TopCluster List
//
void EAClusTrackInfo::analyzeTopList(const EMClusterTree & topList){
  doAnalysis(&topList);
}

//--
// Cluster Analyzer
void EAClusTrackInfo::doAnalysis(const EMCluster * clus){
  
  // Analyze Cluster that is not a neutral one
  // perform operation on all elements of the cluster
  for(const EMClusterMember* child = clus->begin(); 
      child!=clus->end() && track==0;  child = child->next())
    child->analyzeMe(*this);
}

//--
// HitCell Analyzer
void EAClusTrackInfo::doAnalysis(const EMHitCell *){}

//--
// ClusterMember Analyzer
void EAClusTrackInfo::doAnalysis(const EMClusterMember *){}


//--
// TrackOnEcal Analyzer
void EAClusTrackInfo::doAnalysis(const EMTrackOnEcal * tra){
  track = tra;
}

//--
//  return track informations stored
TrackInfo EAClusTrackInfo::getTrack() const{

#ifdef CMATCH
  if( track!=0){
    return *track->Infos();
  }
  TrackInfo trIn;
#else
  if( track!=0){
    return track->Infos();
  }
  TrackInfo trIn(0,0.0,0.0,0.0,0.0,0.0,0.0,0);
#endif
  return trIn;
}

 
