//                              -*- Mode: C++ -*-
//:ELNearClusters.C
//
// Author: Mauro Villa   
// ECAL group 
// HERA-B experiment, Desy
// e-mail: villa@bo.infn.it
//
// Created 27/5/97
// Last Update ........
//
//

#include "care/ELNearClustersRec.hh"
#include "care/ELNearHitCells.hh"
#undef WRITE
/*
CLASS 
  ELNearClustersRec
  Get the list of clusters nearby a given cell
  and provide access to the list


*/

///////////////// ELNearHitCells.C member function definitions ////////
//--
//  constructor
ELNearClustersRec::ELNearClustersRec() :  ELNearClusters() {} 


//--
// constructor
ELNearClustersRec::ELNearClustersRec(ELNearHitCells & nearT) :
  ELNearClusters(nearT){}

//--
//  destructor
ELNearClustersRec::~ELNearClustersRec(){}



//--
//  Minimize the cluster list
//  This means take the cluster list and reduce it (following upstream
//  parent links) to a list of clusters with the same parent cluster.
//  Reduce the list eliminating the clusters equals and sort them in 
//  decreasing order in energy of the biggest signal.
void ELNearClustersRec::minimize(){
 // Start evaluating the minimum depth of the cluster stored
  Level_t minimumDepth = MAXDEPTH; // a very huge value;
  Level_t nearminDepth = MAXDEPTH; // a very huge value;

  Level_t depth;
  unsigned int i;
  for( i=0; i<howMany(); i++){
    depth = neibClusters[i]->depthLevel();
    if( depth<minimumDepth){
      nearminDepth = minimumDepth;
      minimumDepth = depth;
    }
    if( depth<nearminDepth && depth>minimumDepth){
      nearminDepth = depth;
    }
  }
  // now in minimumDepth there is the minimum depth of the clusters in the 
  // array
  // Now for each cluster in the array follow the parent link until a 
  // parent of depth equal to minimumDepth is reached
  for( i=0; i<howMany(); i++){
    while(neibClusters[i]->depthLevel() > nearminDepth){
      neibClusters[i] = neibClusters[i]->getParent();
      if(  neibClusters[i]==0){
	cout<<" Error: reached Top level in minimize"<<endl;
      } 
    }
  }
  // for each cluster found test if there are more than one occurrence:
  // if it is so put to 0 all occurreces after the first
  oneOccurrence();
  //
  //  reduce the array
  reduce();
  //
  if(howMany()==2){
    // special case for 2 clusters
    unsigned int low = 0; // stores the cluster with lower depth
    unsigned int high = 1;
    if( neibClusters[low]->depthLevel()>neibClusters[high]->depthLevel() ){
      unsigned int tmp = low;
      low = high;
      high = tmp;
    }
    const EMCluster* cluster =  neibClusters[high];
    while(cluster->depthLevel() > minimumDepth)
      cluster = cluster->getParent();
    if( cluster == neibClusters[low]){
      neibClusters[low] = 0;
      reduce();
    }
  }
  // now test if only a cluster has been found.
  // if not follow up for a level until level 1 is found or numberNNC = 1
  while(howMany()>1 && minimumDepth>FIRSTLEVEL){
    for(i=0; i<howMany(); i++)
      neibClusters[i] = neibClusters[i]->getParent();
    minimumDepth = neibClusters[0]->depthLevel();
    oneOccurrence();
    reduce();
  }
}
