//                              -*- Mode: C++ -*-
//:ELNearNeuCellLUT.hh
//
//
// Author: Mauro Villa   
// ECAL group 
// HERA-B experiment, Desy
// e-mail: villa@bo.infn.it
//
// Created 8/4/97
/* $Log: ELNearNeiCellLUT.C,v $
 * Revision 1.4  1999/08/04 15:21:40  natasha
 * new care version from Mauro
 *
 * Revision 1.2  1999/06/01  16:07:42  villa
 * before HH
 *
 */

#include <iostream.h>
#include <string.h>
#include "care/ELNearNeiCellLUT.hh"
#include "care/ELCellGrid.hh"
#include "care/ECCellID.hh"

#ifdef _OLD_NAMES_
#  include <algo.h>   
#else
#  include <algorithm>
#endif


#undef DEBUG

/**
class  ELNearNeiCellLUT.
  This is a Look Up Table to transform a GCAL cell number to an integer
  index ranging from 0 to number of cells -1. It can also transform
  the three numbers Module, Ix, Iy (unpacked GCAL cell number) into the same
  integer index.

*/


// utility function
int sortneib(const CellIdx_t arg1, const CellIdx_t arg2);

// LookUp operations
// Get the number of nearest neibouring cells
int  ELNearNeiCellLUT::howManyNeibouring(CellIdx_t idx) {
  // remember  first neibour index of the last accessed cell by HowMany
  ELLUTsManager LUTs;
  const GCCE & info = LUTs.cellsDesc(idx);
  firstneibour = info.adrs;
  return (info.neib & 0xFF);
}


//--
//  Initialize this LUT and update the CellDesc LUT
//-- The initializing function is this
void ELNearNeiCellLUT::initialize(){
 
  // Define a temporary vector for neibours of a given cell
  CellIdx_t  neibours[20]; // 20 is a very high upper limit!
  //
  GNEI gnei = {-1};
  int  totNeibours = 1; // Total of neibouring cells found
  int  currNeib;    // number of neibourings of the cell under examination
  int ix,iy,ndim;
  Coord_t xc,yc,xwidth,ywidth;
  CellIdx_t testIdx = 1;
# ifdef DEBUG
  cout<<"Finding neibours of "<<ArtePointer<GCCE>::nrow()<<" cells"<<endl;
# endif
  ArteTable<GCCE>::iterator currCell= ArteTable<GCCE>::begin();
  for(; currCell!=ArteTable<GCCE>::end();
      ++currCell, testIdx = testIdx.index()+1){
    // Now CurrCell is the current cell under examination
    currNeib = 0;
    ECCellID(currCell->ceid).getModuleIndex(ix,iy,ndim);
    ArtePointer<GCAL> pMod = ECCellID(currCell->ceid).modulePointer();
    // width of the current cell
    xwidth = (pMod->xmax-pMod->xmin)/ndim;
    ywidth = (pMod->ymax-pMod->ymin)/ndim;
    // center of the current cell 
    xc = pMod->xmin+xwidth*(((float) ix)-0.5);
    yc = pMod->ymin+ywidth*(((float) iy)-0.5);
#   ifdef DEBUG
    cout<<" Neibours of cell "<<testIdx<<" mod="<<currCell->ceid
    	<<" (test:"<<ECCellID(currCell->ceid).cellIdx()<<","<<ix<<","<<iy
	<<") at ("<<xc<<","<<yc<<")"<<endl;
#   endif
    // Find neighbours list
    currNeib = crux(neibours,xc,yc,xwidth,ywidth);
#   ifdef DEBUG
    cout<<"Found "<<currNeib<<" neibours for cell "<<testIdx
	<<" mod="<<currCell->ceid
	<<" ("<<pMod.index()<<","<<ix<<","<<iy<<") in ("<<xc<<","<<yc<<")"
	<<endl;
#   endif
    // Store the pointer and neibours number in CS
    currCell->neib = currNeib;    // Store number of neibours
    currCell->adrs = totNeibours; // index of where next neib ID 
                                       // will be stored
    for(int j=0; j<currNeib; j++){
#     ifdef DEBUG
      cout<<neibours[j]<<" ";
#     endif
      gnei.ncod = neibours[j].index();
      ArtePointer<GNEI>::append(gnei);
    }
#   ifdef DEBUG
    cout<<endl;
#   endif
    // Define a temporary vector for neibours of a given cell
    CellIdx_t  neibours2[20]; // 20 is a very high upper limit!
      // Find neighbours list
    int currNeib2 = corners(neibours2,xc,yc,xwidth,ywidth);
    int stored = 0;
    for(int k=0; k<currNeib2; k++){
      if( find(&neibours[0], &neibours[currNeib], neibours2[k]) ==
	  &neibours[currNeib] ){ // corner cell does not exist
	gnei.ncod = neibours2[k].index();
	ArtePointer<GNEI>::append(gnei);
	stored++;
      }
    }
    currCell->neib += (stored<<8);
    // update counter of neibours
    totNeibours += currNeib+stored;
  }
}


//--
//   Find neibours using the Crux algorithm
int ELNearNeiCellLUT::crux(CellIdx_t * neibours,
			 Coord_t xc, Coord_t yc, 
			 Coord_t xw, Coord_t yw){
  // Set the testing point numbers and values
  const int maxNei = 12;
  static float xofs[maxNei] = {0.51,0.51,0.51,-0.49,0.01,0.49,
                               -0.51,-0.51,-0.51,-0.49,0.01,0.49};
  static float yofs[maxNei] = {-0.49,0.01,0.49,0.51,0.51,0.51,
                               -0.49,0.01,0.49,-0.51,-0.51,-0.51};  
  return sample(neibours, xc, yc, xw, yw, maxNei, xofs, yofs);
}
  

//--
//   Find neighbours using a 3x3 basic square
int ELNearNeiCellLUT::square(CellIdx_t * neibours,
			 Coord_t xc, Coord_t yc, 
			 Coord_t xw, Coord_t yw){
  // Set the testing point numbers and values
  const int maxNeiSquare = 16;
  static float xofsq[maxNeiSquare] = {0.51,0.51,0.51,-0.49,0.01,0.49,
                               -0.51,-0.51,-0.51,-0.49,0.01,0.49,
				0.51,0.51,-0.51,-0.51};
  static float yofsq[maxNeiSquare] = {-0.49,0.01,0.49,0.51,0.51,0.51,
                               -0.49,0.01,0.49,-0.51,-0.51,-0.51,
				0.51,-0.51,0.51,-0.51};  
  return sample(neibours, xc, yc, xw, yw, maxNeiSquare, xofsq, yofsq);
}

//--
//   Find neighbours in the corners of a basic square
int ELNearNeiCellLUT::corners(CellIdx_t * neibours,
			 Coord_t xc, Coord_t yc, 
			 Coord_t xw, Coord_t yw){
  // Set the testing point numbers and values
  const int maxNeiCorners = 4;
  static float xofsc[maxNeiCorners] = {0.51,0.51,-0.51,-0.51};
  static float yofsc[maxNeiCorners] = {0.51,-0.51,0.51,-0.51};  
  return sample(neibours, xc, yc, xw, yw, maxNeiCorners, xofsc, yofsc);
}

//--
//   Find neibours using the generic list of sample points 
int ELNearNeiCellLUT::sample(CellIdx_t * neibours,
			   Coord_t xc, Coord_t yc, 
			   Coord_t xw, Coord_t yw,
			   int samples, float *x, float *y){
  // Get the Cell grid reference
  ELLUTsManager LUTs;
  const ELCellGrid & CG = LUTs.cellsGrid();
  //
  ELCellGrid::ErrorTypes ierr;
  int i;
  float xtest,ytest;
  CellIdx_t testIdx = 0;
  CellIdx_t *neib_end = neibours;
  int  neibNum;
# ifdef DEBUG
  cout<<"Testing same cell:"<<CG.cellIdx(xc,yc,ierr)<<endl;
# endif
  //
  // loop on testing points
  for(i=0; i<samples; i++){
    xtest = xc+xw*x[i]; 
    ytest = yc+yw*y[i];
#   ifdef DEBUG
    cout<<"Trying at ("<<xtest<<","<<ytest<<")"<<endl;
#   endif
    testIdx = CG.cellIdx(xtest,ytest,ierr);
    if( ierr==ELCellGrid::OK ){
      // a neibouring cell has been found. Store it.
#   ifdef DEBUG
      cout<<"Neighbour:"<<testIdx<<endl;
#   endif
      *neib_end = testIdx;
      neib_end++;
    }
  }
  // Now order array of cell indecex
  sort(neibours,neib_end,sortneib);
  // Now reduce multiple indeces to same cell
  // and evaluate number of elements stored
  neibNum = unique(neibours,neib_end) - neibours; 
  return neibNum;
}

int sortneib(const CellIdx_t arg1, const CellIdx_t arg2){
 return arg1.index() > arg2.index();
}



