//                              -*- Mode: C++ -*-
//: EAHypAnalyzer.hh
//
//
// Author: Mauro Villa   
// ECAL group 
// HERA-B experiment, Desy
// e-mail: villa@bo.infn.it
//
// Created 2/5/97
// Last Update ........
//

#ifndef EAHYPANALYZER_HH_
#define EAHYPANALYZER_HH_

#include "care/EAClusterAnalyzer.hh"

#ifdef _OLD_NAMES_
#include <vector.h>
#else
#  include <vector>
#endif
#include "care/ESPosStrategy.hh"
#include "care/ESPIDStrategy.hh"
#include "care/EALinkAnalyzer.hh"

class ESCorCOGPos;
/**
class EAHypAnalyzer.
  Class for finding the hypothetical showers.
  It stores a vector of pointers to the hyp showers.

*/

class EAHypAnalyzer : public EAClusterAnalyzer {
public:
  //-- Constructors
  /// Constructor store an empty vector of EMCluster pointers 
  EAHypAnalyzer(vector<const EMCluster*> & hypShow);
  /// The virtual destructor
  virtual ~EAHypAnalyzer();

  /**@name Beheavioral methods */
  //@{ 
  /// EMClusterMember Analyzer
  virtual void doAnalysis(const EMClusterMember * clsMem);
  //--
  /// EMHitCell Analyzer
  virtual void doAnalysis(const EMHitCell * hitC) ;
  //--
  /// EMCluster Analyzer
  virtual void doAnalysis(const EMCluster * clus);
  //--
  /// EMTrackOnEcal Analyzer
  virtual void doAnalysis(const EMTrackOnEcal * clus);
 //--
  ///  Base function to be called with the TopCluster List
  virtual void doAnalysis(const EMClusterTree * tree);
  //--
  ///  Base function to be called with the TopCluster List
  void analyzeTopList(const EMClusterTree & topList);
  //@}
protected:
  /**@name Protected Beheavioral methods */
  //@{ 
  //--
  /**  Evaluate the information for the cluster 
      and fill a NEW entry in the bank. */
 
  void fillNewEntry(const EMCluster * clus);
  //--
  /**  Evaluate the information for the cluster 
       and fill an OLD entry in the bank */
  void updateOldEntry(const EMCluster * clus, ArtePointer<RCCL> rccl);
  //--
  ///  Test the links
  void testLinks(const EMCluster * clus, ArtePointer<RCCL> rccl);
  //@}

  /**@name Protected Data Members  */
  //@{ 
  //--
  ///  vector containing the cluster pointers to hypothetical showers
  vector<const EMCluster*>* hypShowers;
  //--
  ///  Strategy to evaluate position
  ESPosStrategy* poscog;
  //--
  ///  Strategy to evaluate corrected position
  ESCorCOGPos* poscor;
  //--
  ///  Strategy to evaluate the PID likelyhood 
  ESPIDStrategy* pidstra;
  //--
  ///  Object to define link between the RCCL back and the HITC and RTRA ones
  EALinkAnalyzer  rcLinks;
  //@}

  //--
  /// disallow default constructor
  EAHypAnalyzer(){};
  /// disallow copy constructor
  EAHypAnalyzer(const EAHypAnalyzer & ){};
  /// disallow assignment operator
  void operator = (const EAHypAnalyzer & ){};
 

private:
  //--
  ///  Data for making relations in RCCL bank
  int   nrccl;
};

#endif // EAHYPANALYZER_HH_

