//                              -*- Mode: C++ -*-
//: EAHypTraAnalyzer.hh
//
//
// Author: Mauro Villa   
// ECAL group 
// HERA-B experiment, Desy
// e-mail: villa@bo.infn.it
//
// Created 14/7/97
// Last Update ........
//

#ifndef EAHypTraANALYZER_HH_
#define EAHypTraANALYZER_HH_


#include "care/EAHypAnalyzer.hh"

/** 
class  EAHypTraAnalyzer.
  Concrete class for cluster analysis: it evaluates total energy and
  the center of gravity of a cluster after the matching with the tracking
  system. It updates the RCCL bank
*/

class EAHypTraAnalyzer : public EAHypAnalyzer {
public:
  //-- Constructors
  /// Constructor store an already filled vector of EMCluster pointers 
  EAHypTraAnalyzer(vector<const EMCluster*> & hypSh, 
		   vector<const EMCluster*> & hypSignals);
  /// The virtual destructor
  virtual ~EAHypTraAnalyzer();

  /**@name Beheavioral methods */
  //@{
  ///  analyze the Top list
  void analyzeTopList(const EMClusterTree& topList);

  //-- 
  /// EMClusterMember Analyzer
  virtual void doAnalysis(const EMClusterMember * clsMem);
  //--
  /// EMHitCell Analyzer
  virtual void doAnalysis(const EMHitCell * hitT);
  //--
  /// Do a EMCluster analysis
  virtual void doAnalysis(const EMCluster* clus);
  //--
  /// Do a EMClusterTree analysis
  virtual void doAnalysis(const EMClusterTree* tree);
  //--
  /// Do a EMTrackOnEcal analysis
  virtual void doAnalysis(const EMTrackOnEcal * track);
  //@}

private:
  //--
  /// disallow copy constructor
  EAHypTraAnalyzer(const EAHypTraAnalyzer & ){};
  /// disallow assignment operator
  void operator = (const EAHypTraAnalyzer & ){};
  //--
  ///  private data part
  ArtePointer<RCCL> nextRCCL;
  vector<const EMCluster*>::iterator  nextClus;
  bool passed;
  //--
  ///  vector containing the cluster pointers to hypothetical signals
  vector<const EMCluster*>* hypSignals;
};

#endif // EAHYPTRAANALYZER_HH_
