//                              -*- Mode: C++ -*-
//:ELNearClusters.hh
//
// Author: Mauro Villa   
// ECAL group 
// HERA-B experiment, Desy
// e-mail: villa@bo.infn.it
//
// Created 27/5/97
// Last Update ........
//
//

#ifndef ELNEARCLUSTERS_HH_
#define ELNEARCLUSTERS_HH_

#include "care/ECALtypes.hh"
#include "care/EMCluster.hh"
#include "care/EMHitCell.hh"
class ELNearHitCells;
class EMClusterTree;

/**
class ELNearClusters.
  Get the list of clusters nearby a given cell
  and provide access to the list
*/

class ELNearClusters {
public:
  //--
  /// default constructor
  ELNearClusters();
  //--
  /// constructor with an already initialized ELNearHitCells object
  ELNearClusters(ELNearHitCells & nearT);
  //--
  ///  destructor
  virtual ~ELNearClusters();
  //--

  /**@name Beheavioral methods */
  //@{ 
  //--
  ///  Evaluate the neighbouring cluster of a given cell
  void of(const EMHitCell* cell, const EMClusterTree & tree);
  //--
  ///  Evaluate neighbouring clusters of a given cell
  void of(CellIdx_t cell, const EMClusterTree & tree);
  //--
  ///  Sort in energy
  void sortClusters();
  //--
  ///  Minimize the internal near cluster array
  void minimize();
  //@}
  
  /**@name Iterator methods */
  //@{ 
  //--
  ///  start an iteration 
  inline void begin();
  //--
  ///  test if there are more clusters
  inline bool more() const;
  //--
  ///  consider next cell
  inline void next();
  //--
  ///  get the current cluster
  inline EMCluster* operator*() const;
  //--
  ///  get the number of neibours
  inline unsigned int  howMany() const;
  //@}

protected:
  //--
  ///  Insert the current cluster if it is not already present
  void insert(EMCluster* clus);
  //--
  /**  Test if there are more than one occurrence of a given cluster.
    If it is so put to 0 all occurreces after the first. */
  void oneOccurrence();
  //--
  ///  Reduce the array removing null pointers
  void reduce();
  /// Ordering function
  inline static int comp_cluster_pointers(EMCluster *p1,
					  EMCluster *p2);
  //--
  ///  Array containing the cells
  EMCluster*  neibClusters[20];

  //--
  ///  pointer to current cell
  unsigned int curr;

  //--
  ///  Number of store neibours
  unsigned int numNeib;
};

#include "care/ELNearClusters.icc"
#endif // ELNEARCLUSTERS_HH_
