//                              -*- Mode: C++ -*-
//: ESPeakPos.hh
//
//
// Author: Mauro Villa   
// ECAL group 
// HERA-B experiment, Desy
// e-mail: villa@bo.infn.it
//
// Created 12/6/97
// Last Update ........
//

#ifndef ESPEAKPOS_HH_
#define ESPEAKPOS_HH_


#include "care/ESPosStrategy.hh"

/**
class  ESPeakPos.
  Evaluate the peak position of a cluster (the position of its biggest energy
  release).
*/

class ESPeakPos : public ESPosStrategy {
public:
 
  /// Constructor
  ESPeakPos();
  /// Destructor: currently void but needs to be virtual
  virtual ~ESPeakPos();
  
  /**@name Beheavioral methods */
  //@{ 
  ///   Evaluate the position for a cluster.
  virtual void evaluatePos(const EMCluster* clus);
   //-- 
  /// EMClusterMember Analyzer.
  virtual void doAnalysis(const EMClusterMember * clsMem);
  //--
  /// EMHitCell Analyzer
  virtual void doAnalysis(const EMHitCell * hitT);
  //--
  /// EMCluster Analyzer
  virtual void doAnalysis(const EMCluster * clus);
  //--
  /// EMClusterTree Analyzer
  virtual void doAnalysis(const EMClusterTree * tree);
  //--
  /// EMTrackOnEcal Analyzer
  virtual void doAnalysis(const EMTrackOnEcal * track);
  //@}

  //--
  /// Static function used to initialize values on vars
  static void initialize();

private:
  //                Data Part 
  /// Pointer to the biggest energy release
  const EMHitCell * hitt;
  //--
  /// static data to store variances due to cell size
  static Coord_t varsx[6];
  static Coord_t varsy[6];

};

#endif  //   ESPEAKPOS_HH_
