//                              -*- Mode: C++ -*-
//:EcalDB.hh
//
//
// Author: Vincenzo Vagnoni & Mauro Villa   
// ECAL group 
// HERA-B experiment, Desy
// e-mail: villa@bo.infn.it
//
//

#ifndef __EcalDB_hh__
#define __EcalDB_hh__

extern "C" 
{
#include "table.h"
}

#include <stdio.h>
#include <string.h>

class EcalDB {

private:
  Tables x;
  int xe[5];

  static char *tableName[5];

  static char infoDesc[];

  static int infoDim[6];

  static int infoType[6];

  static char *tableDesc[5];

  static int fieldNum[5];

  static int fieldDim[5][11];

  static int fieldType[5][11];

  static char *tableComment[5];

  static EcalDB * theEcalDB;

  EcalDB();

public:

  struct TableInfo {
    int  firstTime;
    int  lastTime;
    int  storageTime;
    char name[12];
    int  version;
    TableInfo() : firstTime(0), lastTime(0), storageTime(0), 
      version(0){name[0] = 0;};
  };

  class TableMeth {
  private:
    int xeCur;
    EcalDB &EcalDBase;
    int tSize;
    int nrow;
    int major;
    int minor;
    TableInfo info;

  public:
    TableMeth(int i, EcalDB &T);
    ~TableMeth();

    int Book();
    int Unbook();
    int Store(int nrow, void *ptr, int major, int minor,
	     const TableInfo & info, int access);
    int Nrow(int major, int minor, int access);
    int Fetch(void *ptr);
    int Match(int major, int minor, int access);
    int Major() const;
    int Minor() const;
    const TableInfo & Info() const;
  };

  friend class EcalDB::TableMeth;

  static inline EcalDB* instance();
  ~EcalDB();

  TableMeth ECBO;
  TableMeth ECHS;
  TableMeth ECPD;
  TableMeth ECSN;
  TableMeth ECHT;

  Tables Open(char *dbName, int access);
  int Close();

};

EcalDB* EcalDB::instance(){
  if( theEcalDB==0){
    theEcalDB = new EcalDB;
  }
  return theEcalDB;
}

#endif
