//----------------------------------------------------------------------------//
//
//   File:        kaecpt.C
//
//   Description: simulation of the ECAL pretrigger
//                interface for kumac scripts
//
//   Author:      Michael Noerenberg
//                HERA-B Collaboration
//                DESY Hamburg
//                Notkestrasse 85
//                D-22603 Hamburg
//                Germany
//
//   e-mail:      mino@mail.desy.de
//
//----------------------------------------------------------------------------//


// ********************************************************************
//
//  Routine:      kaecpt
//
//  Description:  stand alone reconstruction
//
//  Modification:
//
// ********************************************************************

#include "care/CmdBase.hh"

#ifdef USEKUIP

#include <iostream.h>
#include <stdlib.h>
#include <stdio.h>
/************************************************************************
* KAECPT
*
* called from  KUIP
***********************************************************************/
#include <stdio.h>
#include <string.h>
#include <ctype.h>

//#include "arte/Arte.hh"

#include "care/EcalPretrigger.h"

extern "C" void kugeti_(int*);
extern "C" void kugetr_(float*);
extern "C" void kugetc_(char[], int*, int);

extern "C" void kaecpt_() {

//---------------------------------------------------------------
//   read in subdetector
//---------------------------------------------------------------

  int cmd_len;
  int cmd_len1 = 4;
  char command[5];

  float arg1;
  float arg2;
  float arg3;
  float arg4;

  kugetc_(command, &cmd_len, cmd_len1);
  kugetr_(&arg1);
  kugetr_(&arg2);
  kugetr_(&arg3);
  kugetr_(&arg4);

  char* cmd_string;
  cmd_string  = (char*) malloc(cmd_len+1);
  sprintf(cmd_string  ,"%s", command);
  cmd_string[cmd_len] = (char) 0;
 
  // printf("*** call of kaecpt(%d, %d) ***\n", cmd_len, cmd_len1);
  // printf("*** call of kaecpt(%s, %d) ***\n", cmd_string, strlen(cmd_string));

  for(unsigned int i = 0; i < strlen(cmd_string); ++i)
    cmd_string[i] = tolower(cmd_string[i]);

  // printf("*** call of kaecpt(%s, %d) ***\n", cmd_string, strlen(cmd_string));

  // printf("*** args: %f, %f, %f, %f ***\n", arg1, arg2, arg3, arg4);

  if (!strcmp(cmd_string, "init")) {
    // printf("*** ECPT: init ***\n");

    EcalPretrigger::setup();
    EcalPretrigger::reset();
    EcalPretrigger::setMagnetOn((int) arg1);
    EcalPretrigger::setCut(arg2);
    EcalPretrigger::setGeoMode((int) arg3);

  } else if (!strcmp(cmd_string, "rect")) {
    // printf("*** ECPT: rect ***\n");

    EcalPretrigger::addRectangle(arg1, arg2, arg3, arg4);

  } else if (!strcmp(cmd_string, "fdid")) {
    // printf("*** ECPT: fdid ***\n");

    EcalPretrigger::addBoard((int) arg1);

  } else if (!strcmp(cmd_string, "dump")) {
    // printf("*** ECPT: dump ***\n");

    EcalPretrigger::dumpCells((int) arg1);

  } else if (!strcmp(cmd_string, "tows")) {
    // printf("*** ECPT: tows ***\n");

    printf("%d\n", EcalPretrigger::processTowers());

  } else if (!strcmp(cmd_string, "clus")) {
    // printf("*** ECPT: clus ***\n");

    printf("%d\n", EcalPretrigger::processClusters());

  } else if (!strcmp(cmd_string, "none")) {
    printf("*** ECPT: you have to specify some arguments... ***\n");
  } else {
    printf("*** ECPT: unknown command: '%s' ***\n", cmd_string);
  }

};

#endif // USEKUIP //






