//                              -*- Mode: C++ -*-
//: sparseDCAL.C
//
//
// Author: Mauro Villa   
// ECAL group 
// HERA-B experiment, Desy
// e-mail: villa@bo.infn.it
//
// Created 28/1/99
// Last Update ........
//

#include "arte/arte.hh"
#include "care/ECDcalIO.hh"
#include "care/MMBank.hh"
#include <vector>

//void sparseDCAL(EcalEvent_t * event){
void sparseDCAL(){
  float sigmas = 3.0;  // number of sigmas to cut

  // initialize

  static int init=0;
  static vector<int> threshs;
  if(init!=2){
    if(init==0){
      init = 1;
      return;
    }
    init = 2;
    threshs.push_back(10000);
    for(ArteTable<ECHF>::iterator echf = ArteTable<ECHF>::begin(); 
	echf!=ArteTable<ECHF>::end(); ++echf){
      threshs.push_back((int)(echf->pedf+sigmas*echf->wid));
    }
  }
  // Prepare a DCAL-type bank
  MMBank<TMP1>::Drop();
  MMBank<TMP1>::Book(ArtePointer<ECBO>::nrow()*54+10);
  // loop on info in DCAL
  unsigned int total_size = 0; 
  unsigned short siz =0;
  int thresh;
  int ofs = 0;
  unsigned short head1=0, head2=0, channel=0, sfed_id=0, idmk=0, bx =0;

  ECDcalIO dcal;
  dcal.start(); 
  dcal.getBankHeader(head1, head2);
  ECDcalIO dcalout;
  dcalout.start( (unsigned short*) &(ArtePointer<TMP1>(1)->temp));
  dcalout.putBankHeader(head1, head2);
  total_size += 4;

  dcalout.nextFED();           // get ready for the next event
  unshort fed_id = dcal.nextFED(); // read the Front End Detector IDentifier
  //cout<<" First fed:"<<fed_id<<endl;
  while( fed_id>=16){
    // read the header      
    int nch = dcal.getFedHeader(siz, sfed_id, idmk, bx);
    fed_id = sfed_id & 255;
    //cout<<" Fed with "<<nch<<" data (size="<<siz<<")"<<endl;
    if( (fed_id>=16 && fed_id<=ArtePointer<ECBO>::nrow()+15) &&
	( (sfed_id & 0xFF00)==0 || (sfed_id & 0xFF00)==0xEC00)  ){
      ArteTable<ECBO>::iterator ecbo(ArtePointer<ECBO>(fed_id-15));

      dcalout.putFedHeader(siz, sfed_id, idmk | 0x0700,bx); // sparsified  data       
      int counts = 0;
      // correct and active ecbo table entry
      for(int j=0; j<nch; j++){
	  // readout channel 
	dcal.getNextShort(ofs,channel);
	int cellidx = ecbo->chan[ofs];
	//cout<<"Channel:"<<j<<"("<<ofs<<") ADC:"<<channel<<" idx:"<<cellidx;
	
	if( cellidx>0){
	  thresh = threshs[cellidx];
	  //cout<<" Thresh:"<<thresh;
	  if( channel > thresh && channel < 4096 ){
	    dcalout.putNextShort(ofs, channel);
	    counts++;
	  }
	}
	//cout<<endl;
      }
      siz = dcalout.FEDdone(); // fix FED block size and 
                                   // reset internal pointers
      if( counts >0 ){
        //cout<<siz<<" bytes written"<<endl;
	dcalout.nextFED();           // get ready fo the next event
	/*
	unsigned char * p = (unsigned char *) &(ArtePointer<TMP1>(1)->temp);
	p += total_size; 
	for(int i=0; i<siz; i++, p++)
	  //cout<<" "<<(int)(*p);
	cout<<endl;
	*/
	total_size += siz;
	
      } else {
	//cout<<" Skipping FED block:"<<fed_id<<endl;
      }
    } else {
      //cout<<" skip sparsification on this fed!"<<endl;
      dcalout.putFedHeader( siz, sfed_id, idmk, bx); // sparsified data 
       for(int j=0; j<nch; j++){
	  // readout channel 
	 dcal.getNextShort(ofs,channel);
	 dcalout.putNextShort(channel);
	 // **************** 
       }
    }
    fed_id = dcal.nextFED();
    //cout<<" fed_ID :"<<fed_id<<endl;
  }

  cout<<"Sparsification done!"<<endl
      <<"Initial DCAL size:"<<ArtePointer<DCAL>::nrow()*4<<endl
      <<"Final DCAL size:"<<total_size<<endl;
  if( total_size<ArtePointer<DCAL>::nrow()*4){
    cout<<" Copying sparsified data to DCAL"<<endl;
    unsigned int *ps, *pd;
    ps = (unsigned int *) &(ArtePointer<TMP1>(1)->temp);
    pd = (unsigned int *) &(ArtePointer<DCAL>(1)->bits);
    for(unsigned int k=total_size/4; k!=0; k--)
      *pd++ = *ps++;
    // remove entries
    MMBank<DCAL>::Book( total_size/4-ArtePointer<DCAL>::nrow());
  }
}






