//
//  Authors:  Ivan Kisel                            Created:  05-Feb-99 1.01
//            Silvia Masciocchi                     Modified: 07-Jun-99 2.01
//            Max-Planck-Institut fuer Physik                 01-Jul-99 2.02 
//            Munchen                                         03-Jul-99 2.03 
//                                                            14-Jul-99 2.04
//                                                            30-Jul-99 2.05
//                                                            01-Sep-99 2.06
//
//  Email:    sma@mppmu.mpg.de ikisel@mppmu.mpg.de
//            HERA-B Collaboration                           
//
// cats.c
// VDS reconstruction program - the main part, needs fill_banks.C .
// The algorithm is based on the cellular automaton NIM A 387 (1997) 433-442. 
// Tracks are reconstructed first in projections and then combined 
// into space tracks. 

#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#define f2cFortran

#include <arte/Arte.h>
#include <arte/ArteUtil.h>

#include <auto/MTRA.h>
#include <auto/MIMP.h>
#include <auto/HITB.h>
#include <auto/GEDE.h>
#include <auto/GESL.h>
#include <auto/GTAR.h>
#include <auto/RTRA.h>
#include <auto/TOPP.h>
#include <auto/MVER.h>

#include "cats/cfortran.h"
#include "cats/hbook.h"
#include "cats/higz.h"
#include "cats/cats.h"

PROTOCCALLSFSUB6(SORTZV,sortzv,PFLOAT,PINT,INT,INT,INT,INT)
#define SORTZV(A1,A2,A3,A4,A5,A6)  CCALLSFSUB6(SORTZV,sortzv,PFLOAT,PINT,INT,INT,INT,INT,A1,A2,A3,A4,A5,A6)


#define NCHAMBERS  10
#define NPLANES   200
#define NPROJ       4
#define NTARGETS    8
#define MAXNUMBEROFHITS     20000
#define MAXNUMBEROFSEGMENTS 100000
#define MAXNUMBEROFTRACKS   4000
#define maxcomb             40000
#define MAXNUMBEROFVERTICES 100

extern void fill_banks(int, int *, float *, float *, float *, float *, int *,
		       float *, float *);
extern void dinv_(int* n, double* da, int* m, int* work, int* ifail);

 
//int NXPAW = 5000;
static int ESC = 65307;

static int Black =  1, 
    DarkGray     =  2, 
    Blue         =  3, 
    Green        =  4, 
    Cyan         =  5, 
    Red          =  6, 
    Magenta      =  7, 
    Brown        =  8, 
    LightBlue    =  9, 
    LightGreen   = 10, 
    LightCyan    = 11, 
    LightRed     = 12, 
    LightMagenta = 13, 
    Yellow       = 14, 
    LightGray    = 15, 
    White        =  0;

static void start_cats();
static void Find_Space_Tracks();
static void Automaton();
static void Track_Finder();
static void Find_Track_Chi_2(int n, int k);
static void Draw_Event();
static void graph_ini();
static void graph_close();
static void TrackRefit();

                 /* distances in cm */

static int   nchambers, ichamber[NCHAMBERS+1];/* real number of superlayers  */
                                              /*and numbers in the GESL table*/
static int   nplanes;
static int   Nsl_Sec[NCHAMBERS+1][5][5];      /* plane from nsl, sect, proj  */

static int   ntargets,  itarget[NTARGETS+1];  /* numbers of targets wires    */

static float Z_TARGET;                        /* mean target z-position      */
static float GTAR_x[NTARGETS+1], GTAR_y[NTARGETS+1], GTAR_z[NTARGETS+1];

static int   fh[NPLANES], lh[NPLANES], fhc[NPLANES], lhc[NPLANES], npr[NPLANES];
static int   fpl, npl; 
static float zpl[NPLANES], coeff[NPLANES];

static int   GEDE_cmp[NPLANES],  GEDE_nsl[NPLANES],
             GEDE_sec[NPLANES],  GEDE_lay[NPLANES];
static float GEDE_sxmn[NPLANES], GEDE_sxmx[NPLANES];
static float GEDE_symn[NPLANES], GEDE_symx[NPLANES];
static float GEDE_cos[NPLANES],  GEDE_sin[NPLANES]; 
static float GEDE_xfw[NPLANES],  GEDE_nwi_x_pit[NPLANES];

static int   cell_f_tmp[201][51], cell_l_tmp[201][51];
static int   cell_f[201][51], cell_l[201][51];

static int   ihit[MAXNUMBEROFHITS+1], ifl[MAXNUMBEROFHITS+1];
static int   ihc[MAXNUMBEROFHITS+1], jhc[MAXNUMBEROFHITS+1];

static int   h_t_pointer[MAXNUMBEROFHITS+1]; // hit to track pointer

static float HITB_x[MAXNUMBEROFHITS];      /* original value                */
static float X_Hit[MAXNUMBEROFHITS];       /* coordinate ???                */
static float Z_Hit[MAXNUMBEROFHITS];       /* coordinate ???                */
static int   L_Hit[MAXNUMBEROFHITS];       /* layer                         */
static int   M_Hit[MAXNUMBEROFHITS];       /* which part of a layer 1,2,3,4 */
static int   T_Hit[MAXNUMBEROFHITS];       /* type of a plane 1,2,3,4       */

static int    N_Segments, Total_N_Segments;

static int   FirstSegment[MAXNUMBEROFHITS+1], LastSegment[MAXNUMBEROFHITS+1];
static int   TrackHit[MAXNUMBEROFHITS+1]; // free or on a track at the depth level


/*
The first and the last segments starting at a hit and ending at
another hit on the next plane. Index is the number of the hit.
*/

/* Arrays describing segments. Index is the number of the segment.  */


static int   StartOfSegment[MAXNUMBEROFSEGMENTS+1], 
      EndOfSegment[MAXNUMBEROFSEGMENTS+1];


/* Starting and ending hits of a segment. */

static int   First_Segment[MAXNUMBEROFSEGMENTS+1], 
      Last_Segment[MAXNUMBEROFSEGMENTS+1];

/*
The first and the last segments starting at the ending hit of a
segment and making a small [ < arccos(COSPHIMAX) ] angle with the segment.
I: I would prefer FirstNeighbor & LastNeighbor instead.
*/

static int   HasNeighbour[MAXNUMBEROFSEGMENTS+1];
static int   LevelOfSegment[MAXNUMBEROFSEGMENTS+1];
static int   CorrectIndex[MAXNUMBEROFSEGMENTS+1];

//static int   StartOnPlane[NCHAMBERS+1], EndOnPlane[NCHAMBERS+1];
/*
The first and the last hit on a plane. Index is the number of the plane.
(!) StartOnPlane>=EndOnPlane.
*/

static int   SetNo;

static int   PlaneSet[10][10]={{8,1,2,3,4,5,6,7,8},{4,1,2,4,5},{4,1,3,4,5}};
//static int   CoPlaneSet[10][10]={{0},{1,3},{1,2}};

static int   NumberOfTracks, Level;
static int   LengthOfTrack[MAXNUMBEROFTRACKS+1];
static int   GoodTrack[MAXNUMBEROFTRACKS+1];
static float Chi_2[MAXNUMBEROFTRACKS+1];
static int   idx_tr[MAXNUMBEROFTRACKS+1];
static int   n_sector[MAXNUMBEROFTRACKS+1];

static int   CurrentTrackStart[NCHAMBERS+1];
static int   TrackStart[MAXNUMBEROFTRACKS+1][NCHAMBERS+1];

static int   Call;

static float SHIFT      =  10.0;
static float shift;
static int   base_plane[5] = {0, 1, 11, 2, 7};
static int   shift_coeff[5] = {0, 3, 1, -1, -3};
static int   projection;

static int   Ntk_prj[5];
static float Xp_prj[5][MAXNUMBEROFTRACKS+1], X0_prj[5][MAXNUMBEROFTRACKS+1];
static int   N_Sector[5][MAXNUMBEROFTRACKS+1];
static int   ok_prj[5][MAXNUMBEROFTRACKS+1];   //good track for space matching
static int   nh_prj[5][MAXNUMBEROFTRACKS+1];
static int   free_h_prj[5][MAXNUMBEROFTRACKS+1];   // free hits on a track
static int   fpl_prj[5][MAXNUMBEROFTRACKS+1], lpl_prj[5][MAXNUMBEROFTRACKS+1];
static int   chi_2_prj[5][MAXNUMBEROFTRACKS+1];

static int   Ntk, nh_trk[MAXNUMBEROFTRACKS+1];
static float Xp_trk[MAXNUMBEROFTRACKS+1], X0_trk[MAXNUMBEROFTRACKS+1];
static float Yp_trk[MAXNUMBEROFTRACKS+1], Y0_trk[MAXNUMBEROFTRACKS+1];
static float C_trk[10*(MAXNUMBEROFTRACKS+1)], Chi2_trk[MAXNUMBEROFTRACKS+1];
static int   hits[MAXNUMBEROFHITS+MAXNUMBEROFTRACKS+10];

static int   nhits, nfound, hh[NPLANES];
static int   gfound, gg[NPLANES];

static int   MINLEVEL   = 2;
static int   n_at_least = 5;
static float TARGETSCALE = 10.0;

static float MINDIST;        /* distance for hit searching around a track  */
static float TARGETREGION;   /* half of a target window                    */
static int   ind_pl;         /* plane to calculate det. efficiency         */
static int   DRAW;           /* switch on the display                      */

/* structure for IOL */
Cats_Iol_t cats_iol = {
  "cats"                  , IOL_NVERBS(Cats_Iol_t,0),
  {"search window"        ,0.0090            ,  " " , IOL_TYPE_FLOAT},
  {"target window"        ,0.3               ,  " " , IOL_TYPE_FLOAT},
  {"det. plane off"       ,0                 ,  " " , IOL_TYPE_INT},
  {"graphics on"          ,0                 ,  " " , IOL_TYPE_FLAG},

};

static int   first = (1==1);


void cats_param_init()
{
  /* initialize iol parameters of silarte package */
  iol_declare_structure((IOL_GENERIC *) &cats_iol);

}


void  CatsEvent(){
  int   ih, nh, ifh, ilh, ipl, ibpl, ifpl, ilpl, ipr, ich, nch;
  int   itr, ic, ntk, istart, iend, ilength;
  float xt, yt, zt;

#ifdef XXX
  float target_scale, target_shift;
  int   i_v, n_v[10], it_itr_v[10][500], itr_ptr[10][500];
  float x_v[10][500], y_v[10][500], w_v[10][500];
  float x_best, y_best, w_best, dist;
  int   ok_vertex, it_best, j;
  int   iver, itver[100];
  float dtver[100];
  float xpl[1000], ypl[1000];
  int   it, i;
  float xx, yy;
  int   itr_vertex[MAXNUMBEROFTRACKS+1];
  int   n_vertex;
  float x_vertex[MAXNUMBEROFVERTICES+1], y_vertex[MAXNUMBEROFVERTICES+1],
    z_vertex[MAXNUMBEROFVERTICES+1], w_vertex[MAXNUMBEROFVERTICES+1];
#endif

  MINDIST      = cats_iol.search_window.value;
  TARGETREGION = cats_iol.target_window.value;
  ind_pl       = cats_iol.gede_plane_off.value;
  DRAW         = cats_iol.draw_display.value;

  if (first){
    first = (1==0);
    start_cats();
  }

  /* recall the target wires for each event */
  ntargets = 0;
  Z_TARGET = 0.0;
  for(ih=1; ih <=NGTAR(); ih++){
    if (GTAR(ih).wnr!=0){
      xt = GTAR(ih).x;
      yt = GTAR(ih).y;
      zt = GTAR(ih).z;

      if (xt*xt+yt*yt<1.0){
	ntargets++;
        itarget[ntargets] = ih;
	Z_TARGET = Z_TARGET + zt;
	GTAR_x[ntargets] = xt;
	GTAR_y[ntargets] = yt;
	GTAR_z[ntargets] = zt;
      }
    }
    else
      goto no_more_targets;
  }
no_more_targets:;  
  if (ntargets!=0){
    Z_TARGET = Z_TARGET/ntargets;
  }
  else{
    printf("********                   NO TARGETS                ********\n");
    ntargets = 1;
    Z_TARGET = -4.7;
    GTAR_x[ntargets] = 0.0;
    GTAR_y[ntargets] = 0.0;
    GTAR_z[ntargets] = -4.7;
    printf("******** Dummy target at (0.0, 0.0, -4.7) introduced ********\n");
  }

  shift = SHIFT; 
  
  if(DRAW) Draw_Event();

  for(ipl=1; ipl <= nplanes; ipl++){ 
    fh[ipl] = 0;
    lh[ipl] =-1;
    fhc[ipl]= 0;
    lhc[ipl]=-1;
    for(ic=0; ic <= 50; ic++){ 
      cell_f[ipl][ic]= 100000;
      cell_l[ipl][ic]=-100000;
      cell_f_tmp[ipl][ic]= 100000;
      cell_l_tmp[ipl][ic]=-100000;
    }
  }


  fpl = 1000;
  npl = 0;
  nh = 0;
  for(ich=1; ich <= nchambers; ich++){
    nch = ichamber[ich];
    ifpl = GESL(nch).fpl;
    ilpl = ifpl + GESL(nch).npl;
    for(ipl=ifpl; ipl < ilpl; ipl++){ 
      ifh = GEDE(ipl).hit1;
      ilh = GEDE(ipl).hitl;
      if (ilh>=ifh){ 
	fh[ipl] = nh+1;
	lh[ipl] = nh+1+(ilh-ifh);
	fhc[ipl] = nh+1;
	lhc[ipl] = nh+1+(ilh-ifh);
	if (ipl > npl) npl = ipl;
	if (lh[ipl]>MAXNUMBEROFHITS){
	  printf("*** MXNUMBEROFHITS reached %d ***\n", nh);
	  return;
	}
	for(ih=ifh; ih <= ilh; ih++){ 
	  nh++;
	  ihit[nh] = ih;
	  ihc[nh]  = nh;
	  jhc[nh]  = nh;
	  ifl[nh]  = 0;
	  h_t_pointer[nh] = 0;

	  HITB_x[nh] = HITB(ih).x;
	  X_Hit[nh]  = coeff[ipl]*HITB(ih).x;
	  Z_Hit[nh]  = HITB(ih).z;
	  L_Hit[nh]  = GEDE_nsl[ipl];
	  M_Hit[nh]  = GEDE_sec[ipl];
	  T_Hit[nh]  = GEDE_lay[ipl];

	  TrackHit[nh] = 0;
	  FirstSegment[nh]= 0;
	  LastSegment[nh] =-1;

	  ic = 50.0*((HITB_x[nh]-GEDE_xfw[ipl])/GEDE_nwi_x_pit[ipl]);
	  
	  if ((ic<0)||(ic>50)) continue;
	  if (cell_f[ipl][ic] == 100000){
	    cell_f[ipl][ic]     = nh;
	    cell_f_tmp[ipl][ic] = nh;
	  }
	  cell_l[ipl][ic]     = nh+1;
	  cell_l_tmp[ipl][ic] = nh+1;

	}

	if (cell_f_tmp[ipl][1]<cell_f[ipl][0])
	  cell_f[ipl][0]=cell_f_tmp[ipl][1];
	if (cell_l_tmp[ipl][1]>cell_l[ipl][0])
	  cell_l[ipl][0]=cell_l_tmp[ipl][1];
	
	for(ic=1; ic <= 49; ic++){
	  if (cell_f_tmp[ipl][ic-1]<cell_f[ipl][ic])
	    cell_f[ipl][ic]=cell_f_tmp[ipl][ic-1];
	  if (cell_f_tmp[ipl][ic+1]<cell_f[ipl][ic])
	    cell_f[ipl][ic]=cell_f_tmp[ipl][ic+1];
	  if (cell_l_tmp[ipl][ic+1]>cell_l[ipl][ic])
	    cell_l[ipl][ic]=cell_l_tmp[ipl][ic+1];
	  if (cell_l_tmp[ipl][ic-1]>cell_l[ipl][ic])
	    cell_l[ipl][ic]=cell_l_tmp[ipl][ic-1];
	}
	
	if (cell_f_tmp[ipl][49]<cell_f[ipl][50])
	  cell_f[ipl][50]=cell_f_tmp[ipl][49];
	if (cell_l_tmp[ipl][49]>cell_l[ipl][50])
	  cell_l[ipl][50]=cell_l_tmp[ipl][49];

      }
    }
  }

  /* treat data only in a projection corresponding to this plane slope */

  Call = 0;

  for(ipr=1; ipr <= 4; ipr++){ 
    ibpl = base_plane[ipr];
    projection = ipr;
    shift = shift_coeff[ipr]*SHIFT;

    Track_Finder();

    ntk = 0;
    for (itr=1;itr<=NumberOfTracks;itr++){
      if (GoodTrack[itr]==1){ 
	ntk++;
	istart = TrackStart[itr][1];
	iend   = TrackStart[itr][LengthOfTrack[itr]];
	ilength = LengthOfTrack[itr];
	Xp_prj[ipr][ntk] =
	  (X_Hit[iend]-X_Hit[istart])/(Z_Hit[iend]-Z_Hit[istart]);
	X0_prj[ipr][ntk] = X_Hit[istart] - Xp_prj[ipr][ntk]*Z_Hit[istart];

	nh_prj[ipr][ntk] = ilength;
	ok_prj[ipr][ntk] = 1;
	fpl_prj[ipr][ntk]=HITB(ihit[istart]).gede;
	lpl_prj[ipr][ntk]=HITB(ihit[iend]).gede;
	N_Sector[ipr][ntk] = M_Hit[istart];
	chi_2_prj[ipr][ntk] = (ilength-2)-Chi_2[itr]/MINDIST;
	free_h_prj[ipr][ntk] = ilength;
	for (ih=1;ih<=ilength;ih++){
	  h_t_pointer[TrackStart[itr][ih]] = ntk;
	}
      }
    }
    Ntk_prj[ipr] = ntk;

  }

  
  Ntk = 0;
  nhits  = 0;

  Find_Space_Tracks();

  ic=0;
  for(ipl=1; ipl <= npl; ipl++){ 
    fhc[ipl] = 0;
    lhc[ipl] =-1;
    for (ih=fh[ipl]; ih<=lh[ipl]; ih++){
      if (ifl[ih]==0){
	ic++;
	jhc[ic]=ih;
	TrackHit[ih]=0;
	FirstSegment[ih]= 0;
	LastSegment[ih] =-1;
	if (fhc[ipl] == 0) fhc[ipl] = ic;
	lhc[ipl] = ic;
      }
      else
	TrackHit[ih]=1;
    }
  }    


  Call = 1;

  for(ipr=1; ipr <= 4; ipr++){ 
    ibpl = base_plane[ipr];
    projection = ipr;
    shift = shift_coeff[ipr]*SHIFT;

    Track_Finder();

    ntk = 0;
    for (itr=1;itr<=NumberOfTracks;itr++){
      if (GoodTrack[itr]==1){ 
	ntk++;
	istart = TrackStart[itr][1];
	iend   = TrackStart[itr][LengthOfTrack[itr]];
	ilength = LengthOfTrack[itr];
	Xp_prj[ipr][ntk] =
	  (X_Hit[iend]-X_Hit[istart])/(Z_Hit[iend]-Z_Hit[istart]);
	X0_prj[ipr][ntk] = X_Hit[istart] - Xp_prj[ipr][ntk]*Z_Hit[istart];

	nh_prj[ipr][ntk] = ilength;
	ok_prj[ipr][ntk] = 1;
	fpl_prj[ipr][ntk]=HITB(ihit[istart]).gede;
	lpl_prj[ipr][ntk]=HITB(ihit[iend]).gede;
	N_Sector[ipr][ntk] = M_Hit[istart];
	chi_2_prj[ipr][ntk] = (ilength-2)-Chi_2[itr]/MINDIST;
	free_h_prj[ipr][ntk] = ilength;
	for (ih=1;ih<=ilength;ih++){
	  h_t_pointer[TrackStart[itr][ih]] = ntk;
	}
      }
    }
    Ntk_prj[ipr] = ntk;

  }


  Find_Space_Tracks();

#ifdef XXX
  //===================== Vertex ==============================//

  if(DRAW){


    target_scale = 5.0;

    for (itr=0;itr< Ntk;itr++){
      itr_vertex[itr]=(1==0);
    }
    n_vertex = 0;

  new_vertex:;

    ismk(3);
    ismksc(0.5);


    for (it=1; it<=ntargets; it++){
      /* draw the wires */
      isplci(it);
      ispmci(it);
      target_shift = 2.0*(it-1);
      ih = itarget[it];

      xpl[0] = target_scale*(GTAR(ih).x+target_shift);
      ypl[0] = target_scale*(GTAR(ih).y-0.5*GTAR(ih).dy);
      xpl[1] = target_scale*(GTAR(ih).x+target_shift);
      ypl[1] = target_scale*(GTAR(ih).y+0.5*GTAR(ih).dy);
      ipl(2,xpl,ypl);

      xpl[0] = target_scale*(GTAR(ih).x-0.5*GTAR(ih).dx+target_shift);
      ypl[0] = target_scale*(GTAR(ih).y);
      xpl[1] = target_scale*(GTAR(ih).x+0.5*GTAR(ih).dx+target_shift);
      ypl[1] = target_scale*(GTAR(ih).y);
      ipl(2,xpl,ypl);

      n_v[it] = 0;
      i_v     = 0;
      for (itr=0;itr< Ntk;itr++){
        if ((itr_vertex[itr]!=0)||(nh_trk[itr]<8)) continue;
        xx=X0_trk[itr]+Xp_trk[itr]*GTAR(ih).z;
        yy=Y0_trk[itr]+Yp_trk[itr]*GTAR(ih).z;

        xpl[0] = target_scale*(xx+target_shift);
        ypl[0] = target_scale*yy;
        ipm(1,xpl,ypl);

        ok_vertex=(1==0);
        if (GTAR(ih).dx>GTAR(ih).dy){
          if (fabs(yy-GTAR(ih).y)<10.0*MINDIST) ok_vertex = (1==1);
        }
        else{
          if (fabs(xx-GTAR(ih).x)<10.0*MINDIST) ok_vertex = (1==1);
        }

        if (ok_vertex){
          i_v++;
          n_v[it]++;
          it_itr_v[it][itr] = i_v; // pointer into a sequence
          x_v[it][i_v] = xx;
          y_v[it][i_v] = yy;
          itr_ptr[it][i_v] = itr;  // pointer to a track
        }
      }
    }


    ok_vertex=(1==0);
    x_best = 0.0;
    y_best = 0.0;
    w_best = 0.0;
    it_best= 0;
    dist   = 0.0;
    for (it=1; it<=ntargets; it++){
      for (i=1;i<=n_v[it];i++){
        w_v[it][i] = 0.0;
        for (j=1;j<=n_v[it];j++){
          /*     use Tukey's bi-weight */
          dist = sqrt((x_v[it][i]-x_v[it][j])*(x_v[it][i]-x_v[it][j])+
                      (y_v[it][i]-y_v[it][j])*(y_v[it][i]-y_v[it][j]));
          if (dist<10.0*MINDIST){
            w_v[it][i] = w_v[it][i]+
              (1.0-(dist/(10.0*MINDIST))*(dist/(10.0*MINDIST)))*
              (1.0-(dist/(10.0*MINDIST))*(dist/(10.0*MINDIST)));
          }
        }
        if (w_v[it][i]>=w_best){
          x_best = x_v[it][i];
          y_best = y_v[it][i];
          w_best = w_v[it][i];
          it_best= it;
          ok_vertex=(1==1);
        }
      }
    }

    if (ok_vertex){
      ok_vertex=(1==0);
      ih = itarget[it_best];
      if (GTAR(ih).dx>GTAR(ih).dy){
        dist = fabs(y_best-GTAR(ih).y);
        if (fabs(y_best-GTAR(ih).y)<10.0*MINDIST) ok_vertex = (1==1);
      }
      else{
        dist = fabs(x_best-GTAR(ih).x);
        if (fabs(x_best-GTAR(ih).x)<10.0*MINDIST) ok_vertex = (1==1);
      }
    }

    if (ok_vertex){
      n_vertex++;                // new primary vertex found
      x_vertex[n_vertex] = 0.0;
      y_vertex[n_vertex] = 0.0;
      w_vertex[n_vertex] = 0.0;
      it=it_best;
      i_v = 0;
      dist = 0.0;
      for (j=1;j<=n_v[it];j++){
        /*     use Tukey's bi-weight */
        dist = sqrt((x_best-x_v[it][j])*(x_best-x_v[it][j])+
                    (y_best-y_v[it][j])*(y_best-y_v[it][j]));
        hfill(105, dist, 0.0, 1.0);
        if (dist<10.0*MINDIST){
          i_v++;                   // add new track to a vertex cluster
          itr = itr_ptr[it][i_v];  // pointer to a track
          itr_vertex[itr] = n_vertex;
          x_vertex[n_vertex]+=w_v[it][j]*x_v[it][j];
          y_vertex[n_vertex]+=w_v[it][j]*y_v[it][j];
          w_vertex[n_vertex]+=w_v[it][j];
        }
      }
      if (w_vertex[n_vertex]!=0.0){
        x_vertex[n_vertex]=x_vertex[n_vertex]/w_vertex[n_vertex];
        y_vertex[n_vertex]=y_vertex[n_vertex]/w_vertex[n_vertex];
        z_vertex[n_vertex]=GTAR(itarget[it_best]).z;
      }
      else{
        printf("*** Zeros vertex weight ***\n");
        n_vertex--;
      }
      if (i_v<5) goto no_more_vertices;
    }
    else
      goto no_more_vertices;

    printf("OK %d/%f ==> Best target %d at (%f, %f) with w=%f\n",
           ok_vertex, dist,
           it_best, x_best, y_best, w_best);

    if (ok_vertex) hfill(101, x_best, y_best, 1.0);

    ispmci(it_best);
    ismksc(w_best);
    ismk(4);

    target_shift = 2.0*(it_best-1);

    xpl[0] = target_scale*(x_best+target_shift);
    ypl[0] = target_scale*y_best;
    ipm(1,xpl,ypl);

    goto new_vertex;
  no_more_vertices:;

    for (iver=1; iver<=NMVER(); iver++){
      dtver[iver] = 1000.0;
      itver[iver] = 0;
    }
    for (it=1; it<=ntargets; it++){
      ih = itarget[it];
      for (iver=1; iver<=NMVER(); iver++){
        if (GTAR(ih).dx>GTAR(ih).dy){
          if (sqrt((MVER(iver).z-GTAR(ih).z)*(MVER(iver).z-GTAR(ih).z)+
                   (MVER(iver).y-GTAR(ih).y)*(MVER(iver).y-GTAR(ih).y))
              <dtver[iver]) {
            dtver[iver]= sqrt(
                              (MVER(iver).z-GTAR(ih).z)*
                              (MVER(iver).z-GTAR(ih).z)+
                              (MVER(iver).y-GTAR(ih).y)*
                              (MVER(iver).y-GTAR(ih).y));
            itver[iver] = it;
          }
        }
        else{
          if (sqrt((MVER(iver).z-GTAR(ih).z)*(MVER(iver).z-GTAR(ih).z)+
                   (MVER(iver).x-GTAR(ih).x)*(MVER(iver).x-GTAR(ih).x))
              <dtver[iver]) {
            dtver[iver]= sqrt(
                              (MVER(iver).z-GTAR(ih).z)*
                              (MVER(iver).z-GTAR(ih).z)+
                              (MVER(iver).x-GTAR(ih).x)*
                              (MVER(iver).x-GTAR(ih).x));
            itver[iver] = it;
          }
        }
      }
    }

    printf("Total vertices %d:", NMVER());
    for (iver=1; iver<=NMVER(); iver++){
      printf(" %d", MVER(iver).flag);
      if ((itver[iver]!=0)&&(MVER(iver).flag==1)){

        hfill(100, MVER(iver).x, MVER(iver).y, 1.0);
        hfill(103, dtver[iver], 0.0, 1.0);
        ih = itarget[itver[iver]];
        if (GTAR(ih).dx>GTAR(ih).dy)
          dist= fabs(MVER(iver).y-GTAR(ih).y);
        else
          dist= fabs(MVER(iver).x-GTAR(ih).x);
        hfill(104, dist, 0.0, 1.0);

        hfill(105,sqrt(
                       (x_best-MVER(iver).x)*(x_best-MVER(iver).x)+
                       (y_best-MVER(iver).y)*(y_best-MVER(iver).y)),
              0.0, 1.0);
        if (ok_vertex)
          hfill(102,sqrt(
                         (x_best-MVER(iver).x)*(x_best-MVER(iver).x)+
                         (y_best-MVER(iver).y)*(y_best-MVER(iver).y)),
                0.0, 1.0);

        ispmci(itver[iver]);
        ismksc(3.0);
        ismk(28);

        target_shift = 2.0*(itver[iver]-1);

        xpl[0] = target_scale*(MVER(iver).x+target_shift);
        ypl[0] = target_scale*MVER(iver).y;
        ipm(1,xpl,ypl);
      }
    }
    printf("\n");

    iuwk(1,1);

  }/* draw */

#endif

  fill_banks(Ntk, nh_trk, Xp_trk, X0_trk, Yp_trk, Y0_trk, hits, 
	     C_trk, Chi2_trk);

  return;
  
}

void Automaton(){
  /*******************************************
    create and make relations between segments
    ******************************************/
  int   i, ii, j, ij, n, m, nc, mc, it;
  int   index;
  int   start_on_plane, end_on_plane, iplane, eplane;
  float x_n, z_n, x_m, z_m;
  int   start, end, first, last;
  int   has_i, level_i, level_j;
  float k1, k2;
  float x, x_t[NTARGETS+1];
  float A1, B1, A2, B2;
  int   from_a_target, ibpl, is;
  float targetcut;

  ibpl = base_plane[projection];
  for (it=1; it<=ntargets; it++)
    x_t[it] = GTAR_x[it]*GEDE_cos[ibpl] - GTAR_y[it]*GEDE_sin[ibpl];

  if (Call==0)
    targetcut = TARGETREGION;
  else 
    targetcut = TARGETSCALE*TARGETREGION;

  
  /* create segments in upstream direction pointing from the target    */
  /* to have the last segment on each track connected with the target  */


  index = 0;
  for (is=1; is<=4; is++){
    for (ii=PlaneSet[SetNo][0]; ii>1; ii--){
      eplane = Nsl_Sec[ii][is][projection];
      if (eplane==0) continue;
      iplane = Nsl_Sec[ii-1][is][projection];
      if (iplane==0) continue;
      
      start_on_plane = fhc[eplane];
      end_on_plane   = lhc[eplane];
      if (end_on_plane<start_on_plane) continue;
      for (nc=fhc[iplane]; nc<=lhc[iplane]; nc++){
	n = jhc[nc];
	x_n = X_Hit[n];
	z_n = Z_Hit[n];
	FirstSegment[n] = index+1;
	for (mc = start_on_plane; mc<=end_on_plane; mc++){
	  m = jhc[mc];
	  x_m = X_Hit[m];
	  z_m = Z_Hit[m];
	  
	  from_a_target = 0;
	  for (it=1; it<=ntargets; it++){
	    A1 = (x_m-(x_t[it]-targetcut))/(z_m-GTAR_z[it]);
	    B1 = x_m-A1*z_m;
	    A2 = (x_m-(x_t[it]+targetcut))/(z_m-GTAR_z[it]);
	    B2 = x_m-A2*z_m;
	    if ((x_n>(A1*z_n+B1-MINDIST))&&(x_n<(A2*z_n+B2+MINDIST))){
	      from_a_target = 1;
	      break;
	    }
	  }
	  if (from_a_target) {
	    
	    if (index==MAXNUMBEROFSEGMENTS){
	      printf("<<< MAXIMUM NUMBER OF SEGEMENTS EXCEEDED >>>\n");
	      goto a;
	    }
	    index++;
	    
	    StartOfSegment[index] = n; /* from the target direction */ 
	    EndOfSegment[index] = m;
	    
	    has_i = 0;
	    first =  0;
	    last  = -1;
	    start = FirstSegment[m];
	    end   = LastSegment[m];
	    
	    for (j=start; j<=end; j++){
	      k1=(Z_Hit[EndOfSegment[j]]-z_m)/(Z_Hit[EndOfSegment[j]]-z_n);
	      k2=(z_m-z_n)/(Z_Hit[EndOfSegment[j]]-z_n);
	      x=k1*x_n+k2*X_Hit[EndOfSegment[j]];
	      if (fabs(x-x_m)<MINDIST){
		has_i = 1;
		HasNeighbour[j] = 1;
		if (first==0) first = j;
		last = j;
	      }
	    }
	    
	    HasNeighbour[index]  = has_i;
	    First_Segment[index] = first;
	    Last_Segment[index]  = last;
	  }
	}
	LastSegment[n]  = index;
      }
    }
  }
  
a:
  
 
  Total_N_Segments = index;

  j=0;
  Level = 0;
  for (i=1; i<=index; i++){
    if (HasNeighbour[i]){
      j++;
      CorrectIndex[j] = i;
      HasNeighbour[i] = j;
      if (First_Segment[i]>0)
	First_Segment[i] = HasNeighbour[First_Segment[i]];
      if (Last_Segment[i]>0)
	Last_Segment[i]  = HasNeighbour[Last_Segment[i]];
      first = First_Segment[i];
      last  = Last_Segment[i];
      level_i = 0;
      for (ij=first;ij<=last;ij++){
	level_j = LevelOfSegment[CorrectIndex[ij]];
	if (level_j > level_i)
	  level_i = level_j;
      }
      LevelOfSegment[i] = ++level_i;
      if (level_i > Level)
	Level = level_i;
    }
  }
  N_Segments = j;
  
}





void Track_Finder(){
  
  int   i, j, jj, ic, jc, jjc;
  int   k, kc;
  int   first, last;
  int   level_i, level_j;
  int   first_track;
  int   ReCorrectIdx[MAXNUMBEROFSEGMENTS+1];
  int   itr, jtr, itrc, jtrc;
  int   MinLevel;
  float xpl[10], ypl[10];
  float x_start, s, sx, sy, sxx, sxy, delta, ax, bx;
  int   ipl, ih, is, ie;


  if((DRAW)&&(Call==0)){
    ismksc(0.5);
    ismk(3);
    ispmci(Red);
    
    for(ipl=1; ipl <= npl; ipl++){ 
      if (npr[ipl] != projection) continue;
      for (ih=fh[ipl]; ih<=lh[ipl]; ih++){
	xpl[0] = Z_Hit[ih];
	ypl[0] = X_Hit[ih]+shift;
	ipm(1,xpl,ypl);
      }
    }
    
    iuwk(1,1);
    
  }

  NumberOfTracks = 0;
  MinLevel = MINLEVEL;

  SetNo = 0;
  Automaton();

  while (Level>=MinLevel){
    first_track = NumberOfTracks+1;
    
    for (k=1;k<=N_Segments;k++){
      kc = CorrectIndex[k];
      
      if (LevelOfSegment[kc] == Level){
	CurrentTrackStart[1] = StartOfSegment[kc];
	
	Find_Track_Chi_2(1, kc);
      }
    }

    for (itrc=first_track;itrc<=NumberOfTracks;itrc++){
      itr = idx_tr[itrc];
      /* sort tracks */
      if (GoodTrack[itr]==1){
	jjc=itrc;
	jj =itr ;
	for (jtrc=(itrc+1);jtrc<=NumberOfTracks;jtrc++){
	  jtr = idx_tr[jtrc];
	  if (GoodTrack[jtr]==1){
	    if ( (LengthOfTrack[jtr]>LengthOfTrack[jj])||
		 ( (LengthOfTrack[jtr] == LengthOfTrack[jj])&&
		   (Chi_2[jtr] < Chi_2[jj]) ) ){
	      jjc=jtrc;
	      jj =jtr ;
	    }
	  }
	}
	
	idx_tr[itrc]=jj;
	idx_tr[jjc ]=itr;
	
	/* flag used hits */
	for (i=1;i<=LengthOfTrack[jj];i++) {
	  TrackHit[TrackStart[jj][i]] = 1;
	}
	
	for (jtrc=(itrc+1);jtrc<=NumberOfTracks;jtrc++){
	  jtr = idx_tr[jtrc];
	  if ((GoodTrack[jtr]==1)&&(n_sector[jtr]==n_sector[jj])){
	    for (j=1;j<=LengthOfTrack[jtr];j++){
	      if (TrackHit[TrackStart[jtr][j]]==1){
		GoodTrack[jtr] = 0;
		goto continue_break;
	      }
	    }
	  }
	continue_break:;
	}
      }
    }
    
    while ((NumberOfTracks>0)&&(GoodTrack[NumberOfTracks]==0))
      NumberOfTracks--;
    
    if (Level>=MinLevel){
      jc=0; //calculate new set of segments
      for (ic=1; ic<=N_Segments; ic++){ // on previouse set
	i = CorrectIndex[ic];           // real segment
	ReCorrectIdx[ic] = 0;           // pointer to the new set
	if ((TrackHit[StartOfSegment[i]] == 0)&&
	      (TrackHit[EndOfSegment[i]] == 0)){
	  jc++;                         // add to the new set
	  ReCorrectIdx[ic] = jc;        // point from old set to the new one
	  CorrectIndex[jc] = i;         // point from new set to real segment  
	  first=First_Segment[i];       // remember first/last
	  last=Last_Segment[i];
	  First_Segment[i]=0;
	  Last_Segment[i]=-1;
	  level_i=0;
	  for (k=first; k<=last; k++) { // check status of neighbours
	    kc=ReCorrectIdx[k];         // if there is corresp. in new set
	    if (kc>0) {                 // found
	      if (First_Segment[i]==0) First_Segment[i]=kc;
	      Last_Segment[i]=kc;      
	      level_i=1;
	      LevelOfSegment[CorrectIndex[kc]]=1;
	    }
	  }
	  LevelOfSegment[i]=level_i;
	}
      }
      N_Segments = jc;

      Level = 0;
      for (ic=1; ic<=N_Segments; ic++){
	i = CorrectIndex[ic];
	if (LevelOfSegment[i]){
	  first = First_Segment[i];
	  last  = Last_Segment[i];
	  level_i = 0;
	  for (jc=first;jc<=last;jc++){
	    level_j = LevelOfSegment[CorrectIndex[jc]];
	    if (level_j > level_i)
	      level_i = level_j;
	  }
	  LevelOfSegment[i] = ++level_i;
	    if (level_i > Level)
	      Level = level_i;
	}
      }

    }
  }/* while (Level>= MinLevel); */

  for (i=1; i<=Total_N_Segments; i++){
    if (NumberOfTracks == MAXNUMBEROFTRACKS){
      printf("<<< MAXIMUM NUMBER OF TRACKS EXCEEDED >>>\n");
      goto max_tracks;
    }
    is = StartOfSegment[i];
    ie = EndOfSegment[i];
    if ((TrackHit[is]!=0)||(TrackHit[ie]!=0)) continue;
    NumberOfTracks++;
    TrackStart[NumberOfTracks][1] = is;
    TrackStart[NumberOfTracks][2] = ie;
    LengthOfTrack[NumberOfTracks] = 2;
    Chi_2[NumberOfTracks] = 0.0;
    GoodTrack[NumberOfTracks]=1;
    n_sector[NumberOfTracks] = M_Hit[is];
  }
  
max_tracks:;
  

  if(DRAW){
    
    itr = 0;
    isplci(Blue);
    ispmci(Blue);
    ismksc(0.5);
    ismk(3);
    for (i=1;i<=NumberOfTracks;i++){
      if (GoodTrack[i]==1){ 
	s   = 0.0;
	sx  = 0.0;
	sy  = 0.0;
	sxx = 0.0;
	sxy = 0.0;
	
	for (j=1;j<=LengthOfTrack[i];j++){
	  s = s + 1.0;
	  sx = sx + Z_Hit[TrackStart[i][j]];
	  sy = sy + X_Hit[TrackStart[i][j]];
	  sxx = sxx + Z_Hit[TrackStart[i][j]]*Z_Hit[TrackStart[i][j]];
	  sxy = sxy + Z_Hit[TrackStart[i][j]]*X_Hit[TrackStart[i][j]];
	}
	delta = s*sxx-sx*sx;
	ax = (s*sxy-sx*sy)/delta;
	bx = (sxx*sy-sx*sxy)/delta;
	x_start = ax*0.0+bx;
	
	itr++;
	
	xpl[0] = Z_Hit[TrackStart[i][1]];
	ypl[0] = X_Hit[TrackStart[i][1]]+shift;
	xpl[1] = Z_Hit[TrackStart[i][LengthOfTrack[i]]];
	ypl[1] = X_Hit[TrackStart[i][LengthOfTrack[i]]]+shift;
//	ipl(2,xpl,ypl);
	
      }
    }
    
    /*
      printf(" *** ---> %d\n",itr);
      */
    iuwk(1,1);

  }


  
}






void Find_Track_Chi_2(int n, int k){
  
  int   i;
  int   l, lc, lvl;
  int   first, last;
  float chi_2;
  int   start, end, hit;
  float k1, k2, x;
  /*
    n is the number of segments in track processed (CurrentTrack).
    Level is the largest possible length of the track.
    */
  
  /*
    A track is treated processed if the number of the last segment (n) is
    equal to Level. If the first segment of the track is the same as that of
    the previously processed track than the best track is chosen.
    Otherwise a new track is formed.
    */
  
  
  lvl=Level-n; n++;
  if (lvl == 0){
    start = CurrentTrackStart[1];
    end   = EndOfSegment[k];
    chi_2 = 0.0;
    for (i=2;i<n;i++){
      hit=CurrentTrackStart[i];
      k1=(Z_Hit[end]-Z_Hit[hit])/(Z_Hit[end]-Z_Hit[start]);
      k2=(Z_Hit[hit]-Z_Hit[start])/(Z_Hit[end]-Z_Hit[start]);
      x=k1*X_Hit[start]+k2*X_Hit[end];

      chi_2 += fabs(X_Hit[hit]-x);
    }

    if (TrackStart[NumberOfTracks][1] != CurrentTrackStart[1]){
      NumberOfTracks++;
      idx_tr[NumberOfTracks]=NumberOfTracks;
      GoodTrack[NumberOfTracks] = 1;
      LengthOfTrack[NumberOfTracks] = n;
      n_sector[NumberOfTracks] = M_Hit[CurrentTrackStart[1]];
    }
    else
      if (chi_2 > Chi_2[NumberOfTracks])
	goto a;
    
    Chi_2[NumberOfTracks] = chi_2;
    for (i=1;i<n;i++)
      TrackStart[NumberOfTracks][i] = CurrentTrackStart[i];
    TrackStart[NumberOfTracks][n]=EndOfSegment[k];
  a:;
  }
  else {
    first = First_Segment[k];
    last  = Last_Segment[k];
    for (l=first;l<=last;l++){
      lc = CorrectIndex[l];
      if (LevelOfSegment[lc] == lvl){
	CurrentTrackStart[n] = StartOfSegment[lc];
	
	Find_Track_Chi_2(n, lc);
      }
    }
  }
}








void Find_Space_Tracks()
{
  int   nh, ih, ipr, i, itr, nt1, nt2, it;
  int   ipl, jpl, ibpl, ibpl1, ibpl2;
  float MinDist; 
  float x;
  float c1, s1, c2, s2, det;
  float x_1, xp_1, x_2, xp_2;
  float x_1_s2, xp_1_s2, x_1_c2, xp_1_c2;

  float x0, y0, xp, yp, xx, yy;
  float qual;
  float Xsl[maxcomb], X0[maxcomb], Ysl[maxcomb], Y0[maxcomb], Nh[maxcomb];
  int   ind[maxcomb], ncomb;
  int   ip1, ip2, it1, it2;
  int   m, l;
  
  int   iplc[NPLANES], nplc;
  int   jplc[NPLANES], mplc;

  float xpl[1000], ypl[1000];

  int   il, im, ic;
  int   n_3d[NPLANES];
  float z_3d[NPLANES];

  int   i_pr1[maxcomb], i_pr2[maxcomb];
  int   i_tr1[maxcomb], i_tr2[maxcomb];

  int   n_s, n_s_co, nsl_f, nsl_l, nsl_max, nsl_min;
  int   n_sec_co[5] = {0, 3, 4, 1, 2};

  float DOUBLETARGETCUT;// = 2.0*TARGETREGION;

  int   ipr12_first, ipr12_last, ipr34_first, ipr34_last;
  float gMinDist;

//  Ntk = 0;
//  nhits  = 0;
  ip2 = 0; // dummy to evoid compilation warning 

  if (Call == 0)
    DOUBLETARGETCUT = 2.0*TARGETREGION;
  else
    DOUBLETARGETCUT = 2.0*TARGETSCALE*TARGETREGION;

  ncomb = 0;
  for (ip1=1; ip1 <= 2; ip1++){ /* loop over 1-st base proj.*/
    nt1=Ntk_prj[ip1];
    ibpl1 = base_plane[ip1];
    c1=GEDE_cos[ibpl1]; s1=-GEDE_sin[ibpl1];
    
//    for (ip2=3; ip2 <= 4; ip2++){ /* loop over 2-nd base proj.*/
    if (Call==0) ip2=ip1+2;
    else{
      if (ip1==1) ip2=4;
      if (ip1==2) ip2=3;
    }
      nt2=Ntk_prj[ip2];
      ibpl2 = base_plane[ip2];
      c2=GEDE_cos[ibpl2]; s2=-GEDE_sin[ibpl2];
      det=c1*s2-c2*s1;
      s1 = s1/det;
      c1 = c1/det;
      s2 = s2/det;
      c2 = c2/det;

      
      for (it1=1; it1 <= nt1; it1++){ /* loop over tracks on 1-st base proj.*/

	x_1 =X0_prj[ip1][it1];
	xp_1=Xp_prj[ip1][it1];
	x_1_s2 =x_1*s2;
	xp_1_s2=xp_1*s2;
	x_1_c2 =x_1*c2;
	xp_1_c2=xp_1*c2;

	n_s = N_Sector[ip1][it1];
	n_s_co = n_sec_co[n_s];
	nsl_f = GEDE_nsl[fpl_prj[ip1][it1]];
	nsl_l = GEDE_nsl[lpl_prj[ip1][it1]];

	nplc = 0;
	mplc = 0;
	for(ipl=1; ipl <= npl; ipl++){
	  if ((npr[ipl]==ip1)||(npr[ipl]==ip2)||(GEDE_sec[ipl]==n_s_co))
	    continue;
	  
	  if ((GEDE_sec[ipl]==n_s)&&
	      ((GEDE_nsl[ipl]==nsl_f)||(GEDE_nsl[ipl]==nsl_l))){
	    mplc++;
	    jplc[mplc] = ipl;
	  }
	  else{
	    nplc++;
	    iplc[nplc] = ipl;
	  }
	}
	 
	for (it2=1; it2 <= nt2; it2++){ /* loop over tracks on 2-d base proj.*/

	  if (n_s != N_Sector[ip2][it2]) continue;/* not 3D coincided */

	  if ((nsl_f>GEDE_nsl[lpl_prj[ip2][it2]])||
	      (GEDE_nsl[fpl_prj[ip2][it2]]>nsl_l)) continue;

	  x_2 =X0_prj[ip2][it2];
	  xp_2=Xp_prj[ip2][it2];
	  
	  /* space track candidate params*/
	  x0= (x_1_s2-s1*x_2);
	  y0=-(x_1_c2-c1*x_2);
	  xp= (xp_1_s2-s1*xp_2);
	  yp=-(xp_1_c2-c1*xp_2);

	  
	  nfound=0;
	  qual = 0.0;

	  for(jpl=1; jpl <= mplc; jpl++){
	    ipl = jplc[jpl];
	    xx=x0+xp*zpl[ipl];
	    if ( xx < GEDE_sxmn[ipl] ||
		 xx > GEDE_sxmx[ipl] ) continue;
	    yy=y0+yp*zpl[ipl]; 
	    if ( yy < GEDE_symn[ipl] ||
		 yy > GEDE_symx[ipl] ) continue;
	    
	    x = xx*GEDE_cos[ipl] - yy*GEDE_sin[ipl];

	    ic = 50.0*((x-GEDE_xfw[ipl])/GEDE_nwi_x_pit[ipl]);
	    
	    if ((ic<0)||(ic>50)) continue;
	    
	    MinDist = MINDIST;
	    for (ih=cell_f[ipl][ic]; ih < cell_l[ipl][ic]; ih++){ 
	      if (fabs(x-HITB_x[ih])<MinDist)
		MinDist = fabs(x-HITB_x[ih]);
	    }
	    
	    if (MinDist< MINDIST){
	      nfound++;
	      qual+=1.0-MinDist/MINDIST;
	    }
	  }
	  
	  if (nfound==0) continue;
	  
	  if (nsl_l>GEDE_nsl[lpl_prj[ip2][it2]])
	    nsl_max = nsl_l+1;
	  else
	    nsl_max = GEDE_nsl[lpl_prj[ip2][it2]]+1;
	  if (nsl_max>nchambers) 
	    nsl_max = nchambers;

	  if (nsl_f<GEDE_nsl[fpl_prj[ip2][it2]])
	    nsl_min = nsl_f-1;
	  else
	    nsl_min = GEDE_nsl[fpl_prj[ip2][it2]]-1;
	  if (nsl_min<1) 
	    nsl_min = 1;


	  for(jpl=1; jpl <= nplc; jpl++){
	    ipl = iplc[jpl];
	    if (GEDE_nsl[ipl]<nsl_min) continue; // search around a track
	    if (GEDE_nsl[ipl]>nsl_max) break;    // search upstream
	    xx=x0+xp*zpl[ipl];
	    if ( xx < GEDE_sxmn[ipl] ||
		 xx > GEDE_sxmx[ipl] ) continue;
	    yy=y0+yp*zpl[ipl]; 
	    if ( yy < GEDE_symn[ipl] ||
		 yy > GEDE_symx[ipl] ) continue;
	    
	    x = xx*GEDE_cos[ipl] - yy*GEDE_sin[ipl];

	    ic = 50.0*((x-GEDE_xfw[ipl])/GEDE_nwi_x_pit[ipl]);
	    
	    if ((ic<0)||(ic>50)) continue;
	    
	    MinDist = MINDIST;
	    for (ih=cell_f[ipl][ic]; ih < cell_l[ipl][ic]; ih++){ 
	      if (ifl[ih]==0)
		if (fabs(x-HITB_x[ih])<MinDist)
		  MinDist = fabs(x-HITB_x[ih]);
	    }
	    
	    if (MinDist<MINDIST){
	      nfound++;
	      qual+=1.0-MinDist/MINDIST;
	    }
	  }
	  
	  
	  nfound+=nh_prj[ip1][it1]+nh_prj[ip2][it2];
	  if (nfound < n_at_least) continue;

          /* crossing the target wires? */

	  MinDist = DOUBLETARGETCUT;
          for (it=1; it<=ntargets; it++){
            xx=x0+xp*GTAR_z[it];
            yy=y0+yp*GTAR_z[it];
            if ( fabs(xx-GTAR_x[it])+fabs(yy-GTAR_y[it])<MinDist )
	      MinDist = fabs(xx-GTAR_x[it])+fabs(yy-GTAR_y[it]);
          }

	  /* Store combination ??? */


	  Xsl[ncomb] = xp;
	  X0 [ncomb] = x0;
	  Ysl[ncomb] = yp;
	  Y0 [ncomb] = y0;
          i_pr1[ncomb]=ip1;
          i_tr1[ncomb]=it1;
          i_pr2[ncomb]=ip2;
          i_tr2[ncomb]=it2;
	  Nh [ncomb] = nfound+
	    qual+chi_2_prj[ip1][it1]+chi_2_prj[ip2][it2]+
	    4.0*(1.0-MinDist/DOUBLETARGETCUT);
	  ind[ncomb] = ncomb+1;
	  if( ++ncomb >= maxcomb) {
	    printf("*** Space tracks *** == > too many combinations %d\n", ncomb);
	    goto sorting;
	  }



	}
      }
//    }
  }
  
sorting:

  SORTZV(Nh[0],ind[0],ncomb,1,1,0);

  
  for(l=0; l < ncomb; l++){ /* loop over combinations starting from Nhit max */
    m=ind[l]-1;

    if ((ok_prj[i_pr1[m]][i_tr1[m]]==0)||(ok_prj[i_pr2[m]][i_tr2[m]]==0))
      continue;

    n_s_co = n_sec_co[N_Sector[i_pr1[m]][i_tr1[m]]];

    x0 = X0[m];
    xp = Xsl[m];
    y0 = Y0[m];
    yp = Ysl[m];

    nfound = 0;
    gfound = 0;
    ipr12_first = 0;
    ipr12_last  = 0;
    ipr34_first = 0;
    ipr34_last  = 0;
    for(ipl=1; ipl <= npl; ipl++){ 
      if (GEDE_sec[ipl]==n_s_co) continue;
      xx=x0+xp*zpl[ipl];
      if ( xx < GEDE_sxmn[ipl] ||
	   xx > GEDE_sxmx[ipl] ) continue;
      yy=y0+yp*zpl[ipl]; 
      if ( yy < GEDE_symn[ipl] ||
	   yy > GEDE_symx[ipl] ) continue;

      x = xx*GEDE_cos[ipl] - yy*GEDE_sin[ipl]; 
      
      ic = 50.0*((x-GEDE_xfw[ipl])/GEDE_nwi_x_pit[ipl]);
      
      if ((ic<0)||(ic>50)) continue;
      
      MinDist = MINDIST;
      gMinDist= MINDIST;
      for (ih=cell_f[ipl][ic]; ih < cell_l[ipl][ic]; ih++){ 
	if (fabs(x-HITB_x[ih])<gMinDist) {
	  gMinDist   = fabs(x-HITB_x[ih]);
	  gg[gfound] = ih;
	}
	if (ifl[ih]==0)
	  if (fabs(x-HITB_x[ih])<MinDist) {
	    MinDist = fabs(x-HITB_x[ih]);
	    hh[nfound] = ih;
	  }
      }

      if (gMinDist<MINDIST){
	gfound++;
	if ((npr[ipl]==1)||(npr[ipl]==2)){
	  if (ipr12_first==0) 
	    ipr12_first = GEDE_nsl[ipl];
	  ipr12_last = GEDE_nsl[ipl];
	}
	else{
	  if (ipr34_first==0) 
	    ipr34_first = GEDE_nsl[ipl];
	  ipr34_last = GEDE_nsl[ipl];
	}
      }

      if (MinDist<MINDIST){
	nfound++;
      }
    }
    if (nfound < n_at_least) continue; 
    if ((ipr12_last-ipr12_first==0)||(ipr34_last-ipr34_first==0)) continue;

    ok_prj[i_pr1[m]][i_tr1[m]] = 0;
    ok_prj[i_pr2[m]][i_tr2[m]] = 0;


    /* store found hits */

    for(i=0; i < nfound; i++) {
      ih = hh[i];
      hits[nhits++]=ihit[ih];
      ifl[ih]=1;  /* flag hit as used */
      /*      
      ihc[hh[i]] = ihc[lh[ipl]];
      lh[ipl]--; 
      */
      /*
      ipl = HITB(ihit[ih]).gede;
      itr = h_t_pointer[ih];
      if (itr!=0){
	ipr = npr[ipl];
	free_h_prj[ipr][itr]--;
	if (free_h_prj[ipr][itr]==0) ok_prj[ipr][itr]=0;
      }
      */
    }

    hits[nhits++]=-1; /* end of hit list */

    //store found track parameters
    X0_trk[Ntk]=X0[m];
    Xp_trk[Ntk]=Xsl[m];
    Y0_trk[Ntk]=Y0[m];
    Yp_trk[Ntk]=Ysl[m];
    nh_trk[Ntk]=nfound;

    TrackRefit();

    if(++Ntk >= MAXNUMBEROFTRACKS)  {
      printf("*** Space track *** == > too many tracks %d\n", Ntk); 
      return;
    }

  } /* end of loop over combinations*/

  if(DRAW){
    printf("Space tracks  N comb. = %5u N found tracks = %4u :\n", ncomb, Ntk);
  }


  if(DRAW){

    isplci(Black);
    
    ismk(24);
    ismksc(1.5);
    
    for(ipr=1; ipr <= 4; ipr++){ 

      ibpl = base_plane[ipr];
      shift = shift_coeff[ipr]*SHIFT;
      
      nh = 0;//!
      
      for (itr=0;itr< Ntk;itr++){
	
	xx=X0_trk[itr]+Xp_trk[itr]*Z_TARGET;
	yy=Y0_trk[itr]+Yp_trk[itr]*Z_TARGET;
	
	x = xx*GEDE_cos[ibpl] - yy*GEDE_sin[ibpl]; 
	
	xpl[0] = Z_TARGET;
	ypl[0] = x+shift+0.1;
	
	
	xx=X0_trk[itr]+Xp_trk[itr]*zpl[npl];
	yy=Y0_trk[itr]+Yp_trk[itr]*zpl[npl];
	
	x = xx*GEDE_cos[ibpl] - yy*GEDE_sin[ibpl]; 
	
	xpl[1] = zpl[npl];
	ypl[1] = x+shift+0.1;

	ipl(2,xpl,ypl);

	/* find and draw 3D points */
	
	for(ipl=1; ipl <= npl; ipl++){ 
	  n_3d[ipl] = 0;
	  z_3d[ipl] = 0.0;
	}

	for (ih=nh;ih<nh+nh_trk[itr];ih++){
	  il = GESL(GEDE(HITB(hits[ih]).gede).ogesl).nsl;
	  im = GEDE(HITB(hits[ih]).gede).sec;// & 65535;
	  n_3d[il]++;
	  z_3d[il]=z_3d[il]+HITB(hits[ih]).z;
	}
	nh=nh+nh_trk[itr]+1; /*skip (-1)*/


	for(ipl=1; ipl <= npl; ipl++){ 
	  il = GESL(GEDE(ipl).ogesl).nsl;
	  if (n_3d[il] != 0){
	    xx=X0_trk[itr]+Xp_trk[itr]*z_3d[il]/n_3d[il];
	    yy=Y0_trk[itr]+Yp_trk[itr]*z_3d[il]/n_3d[il];
	    x = xx*GEDE_cos[ibpl] - yy*GEDE_sin[ibpl]; 
	    
	    xpl[0] = z_3d[il]/n_3d[il];
	    ypl[0] = x+shift+0.1;

	    if (n_3d[il] == 1) ispmci(Black);
	    if (n_3d[il] == 2) ispmci(Blue);
	    if (n_3d[il] == 3) ispmci(Magenta);
	    if (n_3d[il] >= 4) ispmci(Red);
	    
//	    ipm(1,xpl,ypl);
	    
	  }
	}
      }
    }
    
    iuwk(1,1);

  }/* draw */


}



void TrackRefit()
{/*============================================================
   refit a track (copy of holmes' refit to be compatible)
   ============================================================
   */
  int    i, j, ih, n, gedeP;
  double u, x, y, tx, ty, chi2;
  double da[16], db[4];
  double scale = MINDIST*MINDIST;
  float  v[4];     // projector for track vectors on local coordinate
  float  vv[16];   // projector on v
//  float  C[10];          // (triangular) covariance matrix  
  int    work[4];
  int    ifail;
  //-----------------------------------------------

  // fill weight matrix and constraint vector

  for(i=0; i<16; i++) da[i] = 0;
  for(i=0; i<4;  i++) db[i] = 0;

  chi2 = 0;
  for(ih=0; ih<gfound; ih++) {
    u     = HITB(ihit[gg[ih]]).x;
    gedeP = HITB(ihit[gg[ih]]).gede;
    v[0]  =  GEDE_cos[gedeP];
    v[1]  = -GEDE_sin[gedeP];
    v[2]  =  GEDE_cos[gedeP]*zpl[gedeP];
    v[3]  = -GEDE_sin[gedeP]*zpl[gedeP];
    for(i=0; i<4; i++) {
      for(j=0; j<4; j++) {
        vv[4*i+j] = v[i]*v[j];
      }
    }
    for(j=0; j<16; j++) da[j] += (double)(vv[j]);
    for(j=0; j<4;  j++) db[j] += (double)(v[j]) * u; 
    chi2 += u*u;
  }

  // get the covariance matrix 

  n = 4;
  dinv_(&n, da, &n, work, &ifail);
  if(ifail) {
     printf("track refit failed \n");
     return;
  }

  // determine the fit parameters

  x = y = tx = ty = 0.;
  for(i=0; i<4; i++) {
    x  += da[4*0+i]*db[i];
    y  += da[4*1+i]*db[i];
    tx += da[4*2+i]*db[i];
    ty += da[4*3+i]*db[i];
  }

  // fill the covariance matrix

  n=0;
  for(i=0; i<4; i++) {
    for(j=0; j<=i; j++) {
      C_trk[10*Ntk+n] = scale*da[4*i+j];
    }
  }


  // calculate the chi2
 
  chi2 = (chi2 - db[0]*x - db[1]*y - db[2]*tx - db[3]*ty)/scale;
  if(chi2 < 0) chi2 = 0;

  Chi2_trk[Ntk] = chi2;

  // finally fill track parameters
 
  //store found track parameters
//  printf("*** %d : x0 %f xp %f/%f y0 %f yp %f/%f \n", Ntk, 
//	 X0_trk[Ntk], Xp_trk[Ntk], tx, Y0_trk[Ntk], Yp_trk[Ntk], ty);

  X0_trk[Ntk]=x;
  Y0_trk[Ntk]=y;
  Xp_trk[Ntk]=tx;
  Yp_trk[Ntk]=ty;

}



void start_cats(){
  
  int   ih, ipl, ipr, is;
  float tmp, sxmn, sxmx, symn, symx, alpha;

  printf("*** Cats version 2.06 started *** \n");

  ntargets = 0;
  Z_TARGET = 0.0;
  for(ih=1; ih <=NGTAR(); ih++){
    if (GTAR(ih).wnr!=0){
      ntargets++;
      itarget[ntargets] = ih;
      Z_TARGET = Z_TARGET + GTAR(ih).z;
      if(DRAW){
	printf("Target %d wire %d (dx %f - dy %f), position (%f, %f, %f)\n",
	       ntargets, GTAR(ih).wnr, GTAR(ih).dx, GTAR(ih).dy, 
	       GTAR(ih).x, GTAR(ih).y, GTAR(ih).z);
      }
    }
  }
  if (ntargets!=0) Z_TARGET = Z_TARGET/ntargets;
  
  nchambers = 0;
  for(ih=1; ih <= 20; ih++){
    if (GESL(ih).cmp==1){
      nchambers++;     
      ichamber[nchambers] = ih;
      if(DRAW){
      printf("%d (%d) cmp= %d, nsl= %d, lay= %d, npl= %d, fpl= %d z= %f\n", 
	     nchambers, ih, GESL(ih).cmp, GESL(ih).nsl, GESL(ih).lay, 
	     GESL(ih).npl, GESL(ih).fpl, GESL(ih).z1);
      }
      for(is=1; is <= 4; is++){
	for(ipr=1; ipr <= 4; ipr++){
	  Nsl_Sec[nchambers][is][ipr] = 0;
	}
      }
      nplanes= GESL(ih).fpl+GESL(ih).npl-1;
    }
    if (GESL(ih).cmp==0) goto no_more_chambers;
  }
no_more_chambers:;



  for(ipl=1; ipl <= nplanes; ipl++){ 
    GEDE_cmp[ipl] = GEDE(ipl).cmp;
    GEDE_nsl[ipl] = GESL(GEDE(ipl).ogesl).nsl;
    GEDE_sec[ipl] = GEDE(ipl).sec;
    GEDE_lay[ipl] = GEDE(ipl).lay;
    GEDE_xfw[ipl] = GEDE(ipl).xfw[0]; 
    GEDE_nwi_x_pit[ipl] = GEDE(ipl).nwi[0]*GEDE(ipl).pit;
    zpl[ipl] = 0.5*(GEDE(ipl).zmin+GEDE(ipl).zmax);

    GEDE_cos[ipl] = GEDE(ipl).rot[0] * GEDE(ipl).cos  -
      GEDE(ipl).rot[1] * GEDE(ipl).sin;
    GEDE_sin[ipl] = GEDE(ipl).rot[4] * GEDE(ipl).sin  -
      GEDE(ipl).rot[3] * GEDE(ipl).cos;
    
    sxmn = GEDE(ipl).sxmn * GEDE(ipl).rot[0] +
      GEDE(ipl).symn * GEDE(ipl).rot[1];
    sxmx = GEDE(ipl).sxmx * GEDE(ipl).rot[0] +
      GEDE(ipl).symx * GEDE(ipl).rot[1];
    symn = GEDE(ipl).sxmn * GEDE(ipl).rot[3] +
      GEDE(ipl).symn * GEDE(ipl).rot[4];
    symx = GEDE(ipl).sxmx * GEDE(ipl).rot[3] +
      GEDE(ipl).symx * GEDE(ipl).rot[4];
    
    if( sxmn > sxmx ){
      tmp  = sxmx;
      sxmx = sxmn;
      sxmn = tmp;
    }
    GEDE_sxmn[ipl] = sxmn-0.1;
    GEDE_sxmx[ipl] = sxmx+0.1;
    
    if( symn > symx ){
      tmp  = symx;
      symx = symn;
      symn = tmp;
    }
    GEDE_symn[ipl] = symn-0.1;
    GEDE_symx[ipl] = symx+0.1;
  }

  for(ipl=1; ipl <= nplanes; ipl++){ 
    
    alpha  = 180.0 / 3.1415926 * atan2( GEDE_sin[ipl], GEDE_cos[ipl] );
    if ( alpha < -10.0 )
      alpha += 360.0;
    
    if(  80.0 < alpha && alpha <   90.0 ){
      npr[ipl]   =  1;
      coeff[ipl] =  1.0;
      base_plane[1] = ipl;
    }
    if( 260.0 < alpha && alpha <  270.0 ){
      npr[ipl]   =  1;
      coeff[ipl] = -1.0;
    }

    if(  90.0 < alpha && alpha <  100.0 ){
      npr[ipl]   =  2;
      coeff[ipl] =  1.0;
      base_plane[2] = ipl;
    }
    if( 270.0 < alpha && alpha <  280.0 ){
      npr[ipl]   =  2;
      coeff[ipl] = -1.0;
    }

    if( 170.0 < alpha && alpha <  180.0 ){
      npr[ipl]   =  3;
      coeff[ipl] =  1.0;
      base_plane[3] = ipl;
    }
    if( 350.0 < alpha && alpha <  360.0 ){
      npr[ipl]   =  3;
      coeff[ipl] = -1.0;
    }
    if( -10.0 < alpha && alpha <    0.0 ){
      npr[ipl]   =  3;
      coeff[ipl] = -1.0;
    }

    if( 180.0 < alpha && alpha <  190.0 ){
      npr[ipl]   =  4;
      coeff[ipl] =  1.0;
      base_plane[4] = ipl;
    }
    if(   0.0 < alpha && alpha <   10.0 ){
      npr[ipl]   =  4;
      coeff[ipl] = -1.0;
    }

    Nsl_Sec[GEDE_nsl[ipl]][GEDE_sec[ipl]][npr[ipl]] = ipl;
  }

  if(DRAW){
    graph_ini();
    printf("--> Start drawing? Yes... <ENTER>");
    if (getchar() == ESC){
      graph_close();
    }
  }


}






void stop_cats(){

  if(DRAW){
    graph_close();
  }
}


void Draw_Event(){
  
  float xpl[1000], ypl[1000];
  int   iproj, i, ih, ipl;
  
  
  /* test data only in a projection corresponding to this plane slope */
  
  
  for(iproj=1; iproj <= 4; iproj++){ 
    
    shift = shift_coeff[iproj]*SHIFT;
       
    isfaci(White);
    isplci(Red);
    igbox(-15.0,220.0,-10.0+shift,10.0+shift);
       
       /* draw the targets - only their centers - how to project them?.. */

    ispmci(Red);
    ismk(3);
    ismksc(0.5);
    
    for (i=1; i<=ntargets; i++){
      xpl[0] = GTAR(itarget[i]).z;
      ypl[0] = GTAR(itarget[i]).x+shift;
      ipm(1,xpl,ypl);
    }
    
    /* draw layers */
    
    ispmci(Black);
    ismk(3);
    ismksc(0.5);
    
    isplci(Black);
    for (i=1; i<=nchambers; i++){
      ih = ichamber[i];
      for (ipl=GESL(ih).fpl; ipl<GESL(ih).fpl+GESL(ih).npl; ipl++){

	if (GEDE(ipl).ogesl!=0){
	  
	  if (npr[ipl]==npr[base_plane[iproj]]){
	    xpl[0] = GEDE(ipl).zmin;
	    ypl[0] = coeff[ipl]*GEDE(ipl).xfw[0]+shift;
	    xpl[1] = GEDE(ipl).zmin;
	    ypl[1] = coeff[ipl]*(GEDE(ipl).xfw[0]+
				 GEDE(ipl).nwi[0]*GEDE(ipl).pit)+shift;
	    ipl(2,xpl,ypl);
	  }
	  
	}
      }
    }    
    
  }
  iuwk(1,1); 
  
  
}



/***************** Graphics ******************/



/******************************************/
/************ graph_ini() ******************/
/******************************************/



void graph_ini(){
//  int nxpaw=NXPAW;
//  hlimit_(&nxpaw); 

  iginit(0);
  igsse(0, 8);


  iscr(1, Black       ,0.0000, 0.0000, 0.0000);
  iscr(1, DarkGray    ,0.5000, 0.5400, 0.5300);
  iscr(1, Blue        ,0.0000, 0.0000, 1.0000);
  iscr(1, Green       ,0.1333, 0.5451, 0.1333);
  iscr(1, Cyan        ,0.0000, 0.8078, 0.8196);
  iscr(1, Red         ,1.0000, 0.0000, 0.0000);
  iscr(1, Magenta     ,1.0000, 0.0000, 1.0000);
  iscr(1, Brown       ,0.5000, 0.1647, 0.1647);
  iscr(1, LightBlue   ,0.6784, 0.8471, 0.9020);
  iscr(1, LightGreen  ,0.5961, 0.9843, 0.5961);
  iscr(1, LightCyan   ,0.4980, 1.0000, 0.8314);
  iscr(1, LightRed    ,0.9804, 0.5020, 0.4471);
  iscr(1, LightMagenta,0.8549, 0.4392, 0.8392);
  iscr(1, Yellow      ,1.0000, 1.0000, 0.0000);
  iscr(1, LightGray   ,0.8275, 0.8275, 0.8275);
  iscr(1, White       ,1.0000, 1.0000, 1.0000);



  iswn(2,-15.00,220.00,-40.00,40.00);
  isvp(2,0.0,1.0,0.0,1.0);
  iselnt(2);

  isfais(1);

  isfaci(LightGray);
  isplci(Red);

  igbox(-15.00,220.00,-40.00,40.00);

  iuwk(1,1);

}
/**************************************************/
/************** graph_close() *********************/
/**************************************************/

void graph_close(){
  igend();
}















