      subroutine dgomod(ipl)
************************************************************************
* This routine translates GEANT output hits into ZEBRA bank structures
* JHITB and JDIGB for Honeycomb chambers.
*
* Called from DGEVNT.
*
* Written by S.Nowak, Ch.Stegmann.
* Debugged for new OTR geometry by A.Lanyov,   Nov 1997.
* Updated Aug 1999.
*
* Arguments  kind         description
*-----------------------------------------------------------------------
* IPL        in/out       Detector number in GEDE bank
************************************************************************

      implicit none

#include "arte/Arte.fi"
#include "auto/GEO.fi"
#include "auto/MC.fi"
#include "auto/DIGB.fi"
#include "auto/HITB.fi"
#include "gean/ZGESL.fi"
#include "digi/dglev.fi"

      character*4 dgmod
      common /digmod/ dgmod(20)
      integer nhitde(4),nmim,ndigs,nhits,nn,nn_old,nn1,ntube
      integer i,ih,nmi,j,n,nf,nl,isn,ipl,ifmimp,id,itube,itnei
      integer ix,isec,nsec,nnl,nnr,in,maxwi,maxwic
      real wired,rout,rin,cosx,sinx,rndm,sqrt3,tx,ty,cx,dd,f
      real xt,xtv,xth,xfw,xwir,xtref,xfwc,xwirc,yt,ytv,yth,zt,zwir,zwirc
      real y1w,y2w,ytref,xlu,xru,xlm,xrm,maxg,zc,tcn,ddion
      real dlen,dpath,ztv,xent(4),zent(4),xad,xran
      real acc            ! electronics smearing
      real alam , blam    ! number of clusters per unit length
      real lam
      real fcross         ! fraction of cross talk
      integer nnoise      ! noise
      real  fnoise
      logical lbigr

      integer mxhit,mxwir,iwir,ind0,ixmimp,idigx,ndigx,kdigx
      real xmimp,digx
      parameter (mxhit=1000,mxwir=500)

      real ydg10

c Was error: Dimension of iwir was wrong: iwir(mxwir,2).
c Why mxwir is not equal to mxhit?
      common/wrksp1/iwir(mxwir,4),xmimp(2,mxhit),ixmimp(2,mxhit),
     +              digx(mxhit,4),idigx(mxhit,4),ndigx(mxhit,4),
     +              kdigx(mxhit,4),ind0(mxhit)

      data sqrt3/1.732050808/, acc /0.012/, alam /-58./, blam /20./,
     + fcross /0.02/, fnoise /0.05/
      data ydg10 /2.5/
#include "arte/Arte.ff"
#include "auto/GEO.ff"
#include "auto/MC.ff"
#include "auto/DIGB.ff"
#include "auto/HITB.ff"


c     define some often used quantitites identical for all sublayers

      DO I=1,4
         IF (IGEDE_GCOP(IPL+I).NE.IGEDE_GCOP(IPL)) GO TO 1
      END DO

      call meerro('Next Geant volume not found')

 1    NSEC = I 
      WIRED = FGEDE_PIT(IPL)     ! wire distance
      ROUT =  WIRED/3.           ! cell''s outer radius
      RIN  =  0.5*SQRT3*ROUT     ! cell''s inner radius
      COSX =  FGEDE_ROT(1,IPL)   ! cosine of the rotation angle
      SINX =  FGEDE_ROT(2,IPL)   ! sine of the rotation angle


c     calculate number of mimps
      NMIM = 0
      IFMIMP  = IGEDE_MIM1(IPL)
      IF (IFMIMP.NE.0) NMIM  = IGEDE_MIML(IPL) - IFMIMP + 1

      IF (NMIM.LE.0) THEN
         DO I = 0,NSEC-1
         IQ(IQGEDE_DIG1(IPL+I)) = 0
         IQ(IQGEDE_HIT1(IPL+I)) = 0
         IQ(IQGEDE_DIGL(IPL+I)) = -1
         IQ(IQGEDE_HITL(IPL+I)) = -1
         END DO
         GO TO 998
      END IF


       if (nmim.gt.mxhit) then
         c78 = 'DGO96: nmim='
         call ch78ai(nmim)
         call ch78ac(' > mxhit for ipl=')
         call ch78ai(ipl)
         call me78wc(18282894)
         nmim = mxhit
      end if

c     decide how to sort (always from out- to inside)
      LBIGR = .FALSE.
      IF (ABS(FGEDE_XMAX(IPL)).LE.ABS(FGEDE_XMIN(IPL))) LBIGR = .TRUE.

c* In GEAN96 only the first detector of a module is filled with MIMPs, which
c* may belong also to other detectors. Therefore the first step must be a
c* separation of hits into different detectors. The x coordinate will be
c* transformed into the plane zmin of the given detector and rotated

      nmi = 0
      do i=1,mxhit
         ixmimp(2,i) = 0
      end do

      do i=1,nmim
         ih = i+ifmimp-1
         xt = cosx*fmimp_x(ih)  -  sinx*fmimp_y(ih)
         yt = sinx*fmimp_x(ih)  +  cosx*fmimp_y(ih)
         tx = cosx*fmimp_tx(ih) - sinx*fmimp_ty(ih)
         ty = sinx*fmimp_tx(ih) + cosx*fmimp_ty(ih)
         zt = fmimp_z(ih)
         ytv = yt + ty * (fgede_zmin(ipl)-zt)
         yth = yt + ty * (fgede_zmax(ipl)-zt)


         do j = 0,nsec-1

            if ( (    ytv.ge.fgede_symn(ipl+j)
     @          .and. ytv.le.fgede_symx(ipl+j))
     @         .or.
     @           (    yth.ge.fgede_symn(ipl+j)
     @          .and. yth.le.fgede_symx(ipl+j))) then   ! was error: ytv

               if (nmim.ge.mxhit) then
                  c78 = 'DGO96: nmi='
                  call ch78ai(nmi)
                  call ch78ac(' > mxhit for ipl=')
                  call ch78ai(ipl)
                  call me78ec(18282804)
                  nmi = mxhit
               end if

               nmi = nmi + 1
               xmimp(1,nmi) = xt + tx * (fgede_zmin(ipl)-zt)
               xmimp(2,nmi) = ytv
               ixmimp(1,nmi) = ih
               ixmimp(2,nmi) = j + ipl


            end if
         end do
      end do

C*****************************************************************************
C* The next step fills the intermediate data structures DIGX.
C*****************************************************************************

      ZTV = FGEDE_ZMIN(IPL)

      DO 50 J = 0,NSEC-1       !=============== Loop on sectors (=GEDE volumes)
      ISEC = J+1
      NHITDE(ISEC) = 0

      NTUBE = IGEDE_NSL(IPL+J)     ! number of monolayers within detector

      DO 20 I=1,NMI            !=============== Loop on MIMPs
      IF (IXMIMP(2,I).NE.IPL+J) GO TO 20    ! mimp doesnt belong to that sector
      ID = IXMIMP(1,I)
      TX = COSX*FMIMP_TX(ID)-SINX*FMIMP_TY(ID)
      CX = 0.
      IF (ABS(TX).LT.1.E+10) CX = 1./SQRT(1.+ TX**2)
      TY = SINX*FMIMP_TX(ID)+COSX*FMIMP_TY(ID)

      DO 30 ITUBE=1,NTUBE      !=============== Loop on monolayers (=tubes)

      ZWIR = FGEDE_ZFW(ITUBE,IPL+J)
      Y1W = FGEDE_Y1W(ITUBE,IPL+J) + 0.5 * ydg10
      Y2W = FGEDE_Y2W(ITUBE,IPL+J) - 0.5 * ydg10
      YTREF = (ZWIR- ZTV) * TY + XMIMP(2,I)
      YTV = YTREF - RIN*TY
      IF (YTV.GE.Y1W .AND. YTV.LE.Y2W)  GO TO 31
      YTH = YTREF + RIN*TY
      IF (YTH.GE.Y1W .AND. YTH.LE.Y2W)  GO TO 31
      GO TO 30

 31   XFW = FGEDE_XFW(ITUBE,IPL+J)
      MAXWI = IGEDE_NWI(ITUBE,IPL+J)
      IF (MAXWI.GT.MXWIR) CALL MEERRO('DGO96: MAXWI .GT.MXWIR ')
      XTREF = (ZWIR- ZTV) * TX + XMIMP(1,I)
      XTV = XTREF - RIN*TX
      XTH = XTREF + RIN*TX

      NF = INT((XTV-(XFW-ROUT))/WIRED) + 1    ! first cell hidden by the track 
      NL = INT((XTH-(XFW-ROUT))/WIRED) + 1    ! last cell  hidden by the track 
      IF ((NF.LE.0.AND.NL.LE.0) .OR. (NF.GT.MAXWI.AND.NL.GT.MAXWI)) THEN
         C78 = 'DGO96: NF/NL outside'
         CALL ME78MC(46783,0,ILARGEST)
         GO TO 30
      END IF

      IF (NF.GT.NL) THEN
         ISN=-1
         NF = MIN0(NF,MAXWI)
         NL = MAX0(1,NL)
      ELSE
         ISN = 1
         NF = MAX0(1,NF)
         NL = MIN0(NL,MAXWI)
      END IF

      DO 40 N=NF,NL,ISN           !========================= Loop on wires
      XWIR = XFW + (FLOAT(N)-1.)*WIRED
      DD = (XTREF-XWIR) * CX        ! smallest distance to wires
      IF (ABS(DD).GT.ROUT) GO TO 40  !  distance greater than outer circle?

      IF (IDGLEV(LGESL_WCH) .NE. 0.) THEN  !******** long version ****

C calculate path length in cell

C counter for crossings with cell
         IX = 0
C corners of a cell
         XLU = XWIR - 0.5*ROUT
         XRU = XWIR + 0.5*ROUT
         XLM = XWIR -     ROUT
         XRM = XWIR +     ROUT

C lower boundary
         IF (XTV.GT.XLU.AND.XTV.LT.XRU) THEN
            IX = IX + 1
            XENT(IX) = XTV
            ZENT(IX) = ZWIR-RIN
         END IF

C upper boundary
         IF (XTH.GT.XLU.AND.XTH.LT.XRU)THEN
            IX = IX + 1
            XENT(IX) = XTH
            ZENT(IX) = ZWIR+RIN
         END IF

C lower left side and upper right side ( same slope)
         TCN = -1./SQRT3
         ZC = 0.
         IF (TX.NE.TCN) ZC = ( XLU - XTV ) / ( TX - TCN ) + ZWIR - RIN
         IF (ZC.LT.ZWIR.AND.ZC.GT.ZWIR-RIN)THEN
            IX = IX + 1
            XENT(IX) = TCN*(ZC-(ZWIR-RIN)) + XLU
            ZENT(IX) = ZC
         END IF
         ZC = 0.
         IF (TX.NE.TCN) ZC = ( XRU - XTH) / ( TX - TCN ) + ZWIR +RIN
         IF (ZC.GT.ZWIR.AND.ZC.LT.ZWIR+RIN)THEN
            IX = IX + 1
            XENT(IX) = TCN*(ZC-(ZWIR+RIN)) + XRU
            ZENT(IX) = ZC
         END IF

C lower right side and upper left side ( same slope)
         TCN = -TCN
         ZC = 0.
         IF (TX.NE.TCN) ZC = ( XLU - XTH ) / ( TX - TCN ) + ZWIR + RIN
         IF (ZC.GT.ZWIR.AND.ZC.LT.ZWIR+RIN)THEN
            IX = IX + 1
            XENT(IX) = TCN*(ZC-(ZWIR+RIN)) + XLU
            ZENT(IX) = ZC
         END IF
         ZC = 0.
         IF (TX.NE.TCN) ZC = ( XRU - XTV) / ( TX - TCN ) + ZWIR-RIN
         IF (ZC.LT.ZWIR.AND.ZC.GT.ZWIR-RIN)THEN
            IX = IX + 1
            XENT(IX) = TCN*(ZC-(ZWIR-RIN)) + XRU
            ZENT(IX) = ZC
         END IF

         IF (IX.NE.2) GOTO 40    !  track does not hit cell

C calculate path length in cell
         DPATH = SQRT( (XENT(1)-XENT(2))**2 + (ZENT(1)-ZENT(2))**2)

C throw position of first cluster. the number of clusters per unit
C length depends on the distance from the wire ( attachment in
C drift space)

         LAM = ALAM*DD + BLAM
         IF (DD.GT.0.12) LAM = 13.
C maximum of gamma distribution
         MAXG = 2.*LAM
 26      CALL GRNDM(DLEN,1)
         XAD = 2*LAM*EXP(-2*LAM*DLEN)
         CALL GRNDM(RNDM,1)
         XRAN = MAXG*RNDM
         IF (XRAN.GT.XAD) GOTO 26

C if length needed for the first cluster is longer then half the
C path length in the cell then no hit is produced
         IF (DLEN.GT.0.5*DPATH) GOTO 30

C get distance to wire
         DDION = SQRT( DD**2 + DLEN**2 )

C simulate electronic and mechanical smearing
         CALL GRANOR(F,RNDM)
         DD = DDION + ACC*F
         IF (NHITDE(ISEC).GE.MXHIT) GO TO 49
         NHITDE(ISEC) = NHITDE(ISEC) + 1
         IDIGX(NHITDE(ISEC),ISEC) = ID
         NDIGX(NHITDE(ISEC),ISEC) = N
         KDIGX(NHITDE(ISEC),ISEC) = ITUBE
         DIGX (NHITDE(ISEC),ISEC) = DD

C simulate cross talk

C neighboring sector ???
C
C     j     0 1 2 3
C   sectors 1 2 3 4
C    itnei  2 1 1 1
C
         IF (ITUBE.EQ.1) THEN 
            ITNEI = 2
            NNL = N - 1
            NNR = N
         ELSE
            ITNEI = 1
            NNL = N
            NNR = N + 1
         END IF
C
C cross talk. save the same measured time
C
         MAXWIC = IGEDE_NWI(ITNEI,IPL+J)
         XFWC  = FGEDE_XFW(ITNEI,IPL+J)
         DO 28 IN=NNL,NNR
          IF (IN.LE.0.OR.IN.GT.MAXWIC) GOTO 28
          CALL GRNDM(RNDM,1)
          IF (RNDM.LT.FCROSS) THEN
             XWIRC = XFWC + (FLOAT(IN)-1)*WIRED
             ZWIRC = FGEDE_ZFW(ITNEI,IPL+J)
C smear cross talk hit by electronics noise
             CALL GRANOR(F,RNDM)
             DD = DDION + ACC*F
             IF (NHITDE(ISEC).GE.MXHIT) GO TO 49
             NHITDE(ISEC) = NHITDE(ISEC) + 1
             IDIGX(NHITDE(ISEC),ISEC) = 0
             NDIGX(NHITDE(ISEC),ISEC) = IN
             KDIGX(NHITDE(ISEC),ISEC) = ITNEI
             DIGX (NHITDE(ISEC),ISEC) = DD
          END IF
 28      CONTINUE
      ELSE              ! ***************** short version ****
C* smear the drift distance according to ACC
         CALL GRANOR(F,RNDM)
         DD = DD + FGEDE_ACC(IPL+J)*F
         IF (ABS(DD) .GT. FGEDE_DOD(IPL+J)) then
            GO TO 30
         end if
         CALL GRNDM(RNDM,1)
         IF (RNDM.GT.FGEDE_EFF(IPL+J)) then
            GO TO 30
         end if
         IF (NHITDE(ISEC).GE.MXHIT) GO TO 49
         NHITDE(ISEC) = NHITDE(ISEC) + 1
         IDIGX(NHITDE(ISEC),ISEC) = ID
         NDIGX(NHITDE(ISEC),ISEC) = N
         KDIGX(NHITDE(ISEC),ISEC) = ITUBE
         DIGX (NHITDE(ISEC),ISEC) = DD
      END IF            ! ****************** End of short version

 40   CONTINUE          !======================== End of loop on wires
 30   CONTINUE          !======================== End of loop on monolayers
 20   CONTINUE          !======================== End of loop on MIMPs

C throw a few noise hits per plane
      IF (IDGLEV(LGESL_WCH) .NE. 0.) THEN  !******** long version ****
         DO ITUBE = 1,NTUBE
C number of hits
         CALL GRNDM(RNDM,1)
         NNOISE=INT(FNOISE*MAXWI*RNDM)
         DO IH=1,NNOISE
          CALL GRNDM(RNDM,1)
          IN = INT(MAXWI*RNDM)
          IF (IN.NE.MAXWI) IN = IN + 1
C count hits
          IF (NHITDE(ISEC).GE.MXHIT) GO TO 49
          NHITDE(ISEC) = NHITDE(ISEC) + 1
          IDIGX(NHITDE(ISEC),ISEC) = 0
          NDIGX(NHITDE(ISEC),ISEC) = IN
C throw a radius
          CALL GRNDM(DD,1)
          DIGX (NHITDE(ISEC),ISEC) = DD
          KDIGX(NHITDE(ISEC),ISEC) = ITUBE         
         END DO
         END DO
      END IF                                    !***** end of long version ****
      GO TO 50
 
 49   C78 =  'DGO96: NHITDE gt. MXHIT'
      CALL ME78MC(910673,0,ILARGEST)

 50   CONTINUE          !======================== End of loop on sectors


C*****************************************************************************
C* since cross talk may have inserted digits in other detector planes, one
C* should loop again over all planes
C*****************************************************************************

      DO 60 J=0,NSEC-1  !======================== Loop on sectors
      ISEC = J+1 
      IF (NHITDE(ISEC).EQ.0) GO TO 60
      CALL VZERO(IWIR,4*MXWIR)

C* sort the  matrix DIGX  due to increasing/decreasing wire number
      IF (LBIGR) THEN
         CALL SORTZV(NDIGX(1,ISEC),IND0,NHITDE(ISEC),-1,0,0)
      ELSE
         CALL SORTZV(NDIGX(1,ISEC),IND0,NHITDE(ISEC),-1,1,0)
      END IF

C* set digit and hit bank

      NDIGS = 0
      NHITS = 0
      DO 70 IH = 1, NHITDE(ISEC)!======================== End of digitized hits
      I = IND0(IH)
      N = NDIGX(I,ISEC)
      if (n.gt.mxwir) then
         c78 = 'DGO96: n='
         call ch78ai(n)
         call ch78ac(' > mxwir for ipl=')
         call ch78ai(ipl)
         call me78ec(18282804)
         n = mxwir
      end if

      ID = IDIGX(I,ISEC)
      ITUBE = KDIGX(I,ISEC)
      if (itube .gt. 4) then
         c78 = 'DGO96: itube='
         call ch78ai(itube)
         call ch78ac(' > 4 for ipl=')
         call ch78ai(ipl)
         call meerro(c78)
      end if

      CALL MMBOOK(JDIGB,1)
      NN1 = NDIGB()
      IF (NDIGS.EQ.0) IQ(IQGEDE_DIG1(IPL+J)) = NN1
      NDIGS = NDIGS+1

c ID = 111 0068 means wire: n=28, monolayer: itube=3, GEDE: ipl+j=68

      IQ(IQDIGB_ID_L()) = 10000*((N-1)*4 + ITUBE) + IPL + J
      Q (QDIGB_Q_L()) = DIGX(I,ISEC)


      IF (ID.NE.0) CALL MMBREL(JMIDI,ID,NN1)
      IF (IWIR(N,ITUBE).EQ.0) THEN      !************* Book a new HITB bank
         CALL MMBOOK(JHITB,1)
         NN = NHITB()
         IF (NHITS.EQ.0) THEN
            IQ(IQGEDE_HIT1(IPL+J)) = NN
         END IF
         NHITS = NHITS+1
         IQ(IQHITB_GEDE_L()) = IPL + J
         Q (QHITB_X_L()) = FGEDE_XFW(ITUBE,IPL+J)+(FLOAT(N)-1.)*WIRED
         Q (QHITB_Z_L()) = FGEDE_ZFW(ITUBE,IPL+J)
         Q (QHITB_Q_L()) = DIGX(I,ISEC)
         Q (QHITB_R_L()) = FGEDE_ACC(IPL+J)

ccccccccccc * === MMBREL: relation MIHI: row number out of range
c NN - row in HITB
c ID - row in MIMP (or 0, if this a pure noise hit)
c
c Usage:
c    relmany  MIMP many hits to many MC impact points
c    indx = PHITB_MIMP(ir)           ! 1st index
c    row# = MHITB_MIMP(indx)         ! row in table  MIMX
c    indx = XHITB_MIMP(indx)         ! next index
c
         IF (ID.NE.0) then
            CALL MMBREL(JMIHI,ID,NN)
         end if
         CALL MMBREL(JHIDI,NN,NN1)
         IWIR(N,ITUBE) = NN


      ELSE                      !************* Update existing HITB bank
         nn_old = iwir(n,itube)
         if (nn_old.le.0 .or. nn_old.gt.nhitb()) then
            c78 = 'Illegal HITB #'
            call ch78bi(nn_old)
            call ch78ac(' (already filled)')
            call meerro(c78)
         end if

         if (abs(digx(i,isec)).lt.abs(fhitb_q(nn_old))) then
            q(qhitb_q(nn_old)) = digx(i,isec)


         end if

         if (id.ne.0) then
            call mmbrel(jmihi,id,nn_old)
         end if
         call mmbrel(jhidi,nn_old,nn1)
      end if

 70   continue          !======================== End of loop on digitized hits

      iq(iqgede_digl(ipl+j)) = nn1
      iq(iqgede_hitl(ipl+j)) = nn
 60   continue          !======================== End of loop on sectors

 998  ipl = ipl+nsec-1
      end
