
// *****************************************************************************
//
// source:    faUtil.C
//
// type:      source code
//
// created:   29 Oct 96
// updated:    3 Jun 99 Andreas Gellrich
//
// author:    Andreas Gellrich
//            HERA-B Collaboration
//            DESY Zeuthen
//            Platanenallee 6
//            D-15738 Zeuthen
//            Germany
//            email: Andreas.Gellrich@desy.de
//
// purpose:   various utilities
//
// changes:   polishing (AG: 9 Feb 99)
//            DTAR handling (AG: 10 Feb 99)
//            DTAR has SLB_SUBHDR inside (AG: 22 Feb 99)
//            removed faUtil.hh (AG: 23 Feb 99)
//            provide target and all daq info (2 Mar 99)
//            Fortran/C callable functions
//            polishing (AG: 11 Mar 99)
//            change default return values of utility functions (AG: 26 Mar 99)
//            prepare for DKEY (AG: 27 Apr 99)
//            introduce function ARTE_get_event() (AG: 6 May 99)
//            improve ARTE_get_event() (AG: 11 May 99)
//            avoid using inline functiuon (AG: 14 May 99)
//            fixing of typos (AG: 17 May 99)
//            EVHD has more elements now (AG: 26 May 99)
//            remove DKEY again (AG: 27 May 99)
//            make it more secure (AG: 28 May 99)
//            massive clean-up (AG: 31 May 99)
//            put includes (AG: 1 Jun 99)
//            repair DST reading (AG: 3 Jun 99)
//
// *****************************************************************************

// includes
#include <iostream>
#include <stdio.h>
#include <time.h>
#include <math.h>

#include "etb.h"

#include "arte/arte.hh"
#include "arte/ArteFlag.h"

#include "farm/fa.hh"
#include "farm/farm.hh"

// externals
extern int FADEB;

extern char *evtBuf; // event buffer
extern int  SEG_LEN; // event buffer length (set in faLine.C)

//
// get record id from table DEVT
//
int
ARTE_get_recid_DEVT( int *recid )
{
  if( ArteTable<DEVT>::size() == 0 ){
    return ETB_corrupted_length;
  }
  else {
    ArtePointer<DEVT> dnnn( 1 );
    ESP_get_int( (int *) &dnnn->bits, recid, 1 );
    return ETB_ok;
  }
}

//
// get raw event from DEVT and store it in a save place
//
EVENT_HDR *
ARTE_get_event()
{
  // check table
  if( ArteTable<DEVT>::size() == 0 ){
    if( FADEB ){
      cout << "ARTE_get_event(): NO DEVT table found." << endl;
    }
    return 0;
  }  ArtePointer<DEVT> devt( 2 ); // DEVT table pointer
  int length = ArteTable<DEVT>::size() * sizeof( int ); // length of table
  if( evtBuf == 0 ){
    if( FADEB ){
      cout << "malloc() length=" << length << endl;
    }
    evtBuf = (char *) malloc( length ); // create buffer
    SEG_LEN = length;          // store new length
  }
  else if( SEG_LEN < length ){
    if( FADEB ){
      cout << "realloc() length=" << length << endl;
    }
    evtBuf = (char *) realloc( evtBuf, length ); // re-size buffer
  }
  if( evtBuf == 0 ) return 0; // buffer could not be created

  // copy data
  memcpy( evtBuf, &devt->bits, length ); // copy

  return (EVENT_HDR *) evtBuf;
}

//
// print run/event
//
void
faPrintEvtCount( int run, int event, int exp )
{
  if( run > 0 && event > 0 ){
    if( ( event % int( pow( 10, int( log10( event ) ) ) ) ) == 0 ){
      cout << "PRINT: "
	   << "run="   << run   << " "
	   << "event=" << event << " "
	   << "exp="   << exp   << endl;
    }
  }
}

//
// print rows of tables
//
int
faPrintNrow()
{
  cout << "NROW: ";
  if( ArteTable<EVHD>::size() ) cout << "EVHD=" << ArteTable<EVHD>::size() << " ";
  if( ArteTable<DIHD>::size() ) cout << "DIHD=" << ArteTable<DIHD>::size() << " ";

  if( ArteTable<DEVT>::size() ) cout << "DEVT=" << ArteTable<DEVT>::size() << " ";

  if( ArteTable<DDAQ>::size() ) cout << "DDAQ=" << ArteTable<DDAQ>::size() << " ";
  if( ArteTable<DSVD>::size() ) cout << "DSVD=" << ArteTable<DSVD>::size() << " ";
  if( ArteTable<DITR>::size() ) cout << "DITR=" << ArteTable<DITR>::size() << " ";
  if( ArteTable<DPTC>::size() ) cout << "DPTC=" << ArteTable<DPTC>::size() << " ";
  if( ArteTable<DOTR>::size() ) cout << "DOTR=" << ArteTable<DOTR>::size() << " ";
  if( ArteTable<DRIC>::size() ) cout << "DRIC=" << ArteTable<DRIC>::size() << " ";
  if( ArteTable<DTRD>::size() ) cout << "DTRD=" << ArteTable<DTRD>::size() << " ";
  if( ArteTable<DCAL>::size() ) cout << "DCAL=" << ArteTable<DCAL>::size() << " ";
  if( ArteTable<DMUT>::size() ) cout << "DMUT=" << ArteTable<DMUT>::size() << " ";
  if( ArteTable<DMUX>::size() ) cout << "DMUX=" << ArteTable<DMUX>::size() << " ";
  if( ArteTable<DMUP>::size() ) cout << "DMUP=" << ArteTable<DMUP>::size() << " ";
  if( ArteTable<DFLT>::size() ) cout << "DFLT=" << ArteTable<DFLT>::size() << " ";
  if( ArteTable<DSLT>::size() ) cout << "DSLT=" << ArteTable<DSLT>::size() << " ";
  if( ArteTable<DTLT>::size() ) cout << "DTLT=" << ArteTable<DTLT>::size() << " ";
  if( ArteTable<D4LT>::size() ) cout << "D4LT=" << ArteTable<D4LT>::size() << " ";
  if( ArteTable<DTAR>::size() ) cout << "DTAR=" << ArteTable<DTAR>::size() << " ";
  if( ArteTable<DFCS>::size() ) cout << "DFCS=" << ArteTable<DFCS>::size() << " ";
  cout << endl;

  return ARTE_GOOD;
}

//
// print EVHD-table
//
int
faPrintEVHD()
{
  if( ArteTable<EVHD>::size() ){
    ArtePointer<EVHD> evhd( 1 );
    cout << "EVHD: "
	 << "evt="     << evhd->evt     << " "
	 << "run="     << evhd->run     << " "
	 << "exp="     << evhd->exp     << " "
	 << "date="    << evhd->date    << " "
	 << "time="    << evhd->time    << " "
	 << "vfld="    << evhd->vfld    << " "
	 << "flagdst=" << evhd->flagdst << " "
	 << "dbmajor=" << evhd->dbmajor << " "
	 << "dbminor=" << evhd->dbminor << " "
	 << "flag="    << evhd->flag    << endl;
    return ARTE_GOOD;
  }
  else {
    return ARTE_ERR;
  }
}

//
// print DIHD-table
//
int
faPrintDIHD()
{
  if( ArteTable<DIHD>::size() ){
    ArtePointer<DIHD> dihd( 1 );
    cout << "DIHD: "
	 << "bx="            << dihd->bx          << " "
	 << "flt=0x"  << hex << dihd->flt  << dec << " "
	 << "slt=0x"  << hex << dihd->slt  << dec << " "
	 << "slb=0x"  << hex << dihd->slb  << dec << " "
	 << "cmsk=0x" << hex << dihd->cmsk << dec << " "
	 << "emsk=0x" << hex << dihd->emsk << dec << endl;
    return ARTE_GOOD;
  }
  else {
    return ARTE_ERR;
  }
}

//
// print DDAQ-table
//
int
faPrintDDAQ()
{
  if( ArteTable<DDAQ>::size() ){
    ArtePointer<DDAQ> ddaq( 2 );
    struct EVT_DATA *p = (EVT_DATA *)( &ddaq->bits );

    int run_number = 0;
    ESP_get_int( &p->run_number, &run_number, 1 );
    
    int event_time = 0;
    ESP_get_int( &p->event_time, &event_time, 1 );
    
    int event_status = 0;
    ESP_get_int( &p->event_status, &event_status, 1 );
    
    int flt_status = 0;
    ESP_get_int( &p->flt_status, &flt_status, 1 );

    int slp_node = 0;
    ESP_get_int( &p->slp_node, &slp_node, 1 );

    int slt_status = 0;
    ESP_get_int( &p->slt_status, &slt_status, 1 );

    int exp_number = 0;
    ESP_get_int( &p->exp_number, &exp_number, 1 );

    int farm_node = 0;
    ESP_get_int( &p->farm_node, &farm_node, 1 );

    int BX_long = 0;
    ESP_get_int( &p->BX_long, &BX_long, 1 );

    short BX_upper = 0;
    ESP_get_short( &p->BX_upper, &BX_upper, 1 );

    short FCS_tcode = 0;
    ESP_get_short( &p->FCS_tcode, &FCS_tcode, 1 );

    short physBX = 0;
    ESP_get_short( &p->physBX, &physBX, 1 );

    short FLT_BX = 0;
    ESP_get_short( &p->FLT_BX, &FLT_BX, 1 );

    short fltevt = 0;
    ESP_get_short( &p->fltevt, &fltevt, 1 );

    cout << "DDAQ: " 
	 << "run_number="            << run_number                              << " "
	 << "event_time="            << event_time                              << " "
	 << "date="                  << faDate( (time_t *) &event_time )        << " "
	 << "time="                  << faTime( (time_t *) &event_time )        << " "
	 << "event_status=0x" << hex << event_status                     << dec << " "
	 << "flt_status=0x"   << hex << flt_status                       << dec << " "
	 << "slp_node="              << slp_node                                << " "
	 << "slt_status=0x"   << hex << slt_status                       << dec << " "
	 << "exp_number="            << exp_number                              << " "
	 << "farm_node="             << farm_node                               << " "
	 << "BX_long="               << BX_long                                 << " "
	 << "BX_upper="              << BX_upper                                << " "
	 << "FCS_tcode="             << FCS_tcode                               << " "
	 << "physBX="                << physBX                                  << " "
	 << "FLT_BX="                << FLT_BX                                  << " "
	 << "fltevt="                << fltevt                                  << endl;
    return ARTE_GOOD;
  }
  else {
    return ARTE_ERR;
  }
}

//
// daq run_number
//
int
faDDAQrun_number()
{
  if( ArteTable<DDAQ>::size() ){
    ArtePointer<DDAQ> ddaq( 2 );
    struct EVT_DATA *p = (EVT_DATA *)( &ddaq->bits );
    int run_number = 0;
    ESP_get_int( &p->run_number, &run_number, 1 );
    return run_number;
  }
  else {
    return -1;
  }
}

//
// daq event_time
//
int
faDDAQevent_time()
{
  if( ArteTable<DDAQ>::size() ){
    ArtePointer<DDAQ> ddaq( 2 ); 
    struct EVT_DATA *p = (EVT_DATA *)( &ddaq->bits );
    int event_time = 0;
    ESP_get_int( &p->event_time, &event_time, 1 );
    return event_time;
  }
  else {
    return -1;
  }
}

//
// daq event_status
//
int
faDDAQevent_status()
{
  if( ArteTable<DDAQ>::size() ){
    ArtePointer<DDAQ> ddaq( 2 ); 
    struct EVT_DATA *p = (EVT_DATA *)( &ddaq->bits );
    int event_status = 0;
    ESP_get_int( &p->event_status, &event_status, 1 );
    return event_status;
  }
  else {
    return -1;
  }
}

//
// daq flt_status
//
int
faDDAQflt_status()
{
  if( ArteTable<DDAQ>::size() ){
    ArtePointer<DDAQ> ddaq( 2 ); 
    struct EVT_DATA *p = (EVT_DATA *)( &ddaq->bits );
    int flt_status = 0;
    ESP_get_int( &p->flt_status, &flt_status, 1 );
    return flt_status;
  }
  else {
    return -1;
  }
}

//
// daq slp_node
//
int
faDDAQslp_node()
{
  if( ArteTable<DDAQ>::size() ){
    ArtePointer<DDAQ> ddaq( 2 ); 
    struct EVT_DATA *p = (EVT_DATA *)( &ddaq->bits );
    int slp_node = 0;
    ESP_get_int( &p->slp_node, &slp_node, 1 );
    return slp_node;
  }
  else {
    return -1;
  }
}

//
// daq slt_status
//
int
faDDAQslt_status()
{
  if( ArteTable<DDAQ>::size() ){
    ArtePointer<DDAQ> ddaq( 2 ); 
    struct EVT_DATA *p = (EVT_DATA *)( &ddaq->bits );
    int slt_status = 0;
    ESP_get_int( &p->slt_status, &slt_status, 1 );
    return slt_status;
  }
  else {
    return -1;
  }
}

//
// daq exp_number
//
int
faDDAQexp_number()
{
  if( ArteTable<DDAQ>::size() ){
    ArtePointer<DDAQ> ddaq( 2 ); 
    struct EVT_DATA *p = (EVT_DATA *)( &ddaq->bits );
    int exp_number = 0;
    ESP_get_int( &p->exp_number, &exp_number, 1 );
    return exp_number;
  }
  else {
    return -1;
  }
}

//
// daq farm_node
//
int
faDDAQfarm_node()
{
  if( ArteTable<DDAQ>::size() ){
    ArtePointer<DDAQ> ddaq( 2 ); 
    struct EVT_DATA *p = (EVT_DATA *)( &ddaq->bits );
    int farm_node = 0;
    ESP_get_int( &p->farm_node, &farm_node, 1 );
    return farm_node;
  }
  else {
    return -1;
  }
}

//
// daq BX_long
//
int
faDDAQBX_long()
{
  if( ArteTable<DDAQ>::size() ){
    ArtePointer<DDAQ> ddaq( 2 ); 
    struct EVT_DATA *p = (EVT_DATA *)( &ddaq->bits );
    int BX_long = 0;
    ESP_get_int( &p->BX_long, &BX_long, 1 );
    return BX_long;
  }
  else {
    return -1;
  }
}

//
// daq BX_upper
//
short
faDDAQBX_upper()
{
  if( ArteTable<DDAQ>::size() ){
    ArtePointer<DDAQ> ddaq( 2 ); 
    struct EVT_DATA *p = (EVT_DATA *)( &ddaq->bits );
    short BX_upper = 0;
    ESP_get_short( &p->BX_upper, &BX_upper, 1 );
    return BX_upper;
  }
  else {
    return -1;
  }
}

//
// daq FCS_tcode
//
short
faDDAQFCS_tcode()
{
  if( ArteTable<DDAQ>::size() ){
    ArtePointer<DDAQ> ddaq( 2 ); 
    struct EVT_DATA *p = (EVT_DATA *)( &ddaq->bits );
    short FCS_tcode = 0;
    ESP_get_short( &p->FCS_tcode, &FCS_tcode, 1 );
    return FCS_tcode;
  }
  else {
    return -1;
  }
}

//
// daq physBX
//
short
faDDAQphysBX()
{
  if( ArteTable<DDAQ>::size() ){
    ArtePointer<DDAQ> ddaq( 2 ); 
    struct EVT_DATA *p = (EVT_DATA *)( &ddaq->bits );
    short physBX = 0;
    ESP_get_short( &p->physBX, &physBX, 1 );
    return physBX;
  }
  else {
    return 0;
  }
}

//
// daq FLT_BX
//
short
faDDAQFLT_BX()
{
  if( ArteTable<DDAQ>::size() ){
    ArtePointer<DDAQ> ddaq( 2 ); 
    struct EVT_DATA *p = (EVT_DATA *)( &ddaq->bits );
    short FLT_BX = 0;
    ESP_get_short( &p->FLT_BX, &FLT_BX, 1 );
    return FLT_BX;
  }
  else {
    return -1;
  }
}

//
// daq fltevt
//
short
faDDAQfltevt()
{
  if( ArteTable<DDAQ>::size() ){
    ArtePointer<DDAQ> ddaq( 2 ); 
    struct EVT_DATA *p = (EVT_DATA *)( &ddaq->bits );
    short fltevt = 0;
    ESP_get_short( &p->fltevt, &fltevt, 1 );
    return fltevt;
  }
  else {
    return -1;
  }
}

//
// print DTAR-table
//
int
faPrintDTAR()
{
  if( ArteTable<DTAR>::size() ){
    ArtePointer<DTAR> dtar( 2 ); //  pointer to DTAR
    char *p = (char *) &dtar->bits + sizeof( SLB_SUBHDR );  // pointer to data
    struct TGTS_RAW tgts; // target data structure
    ESP_get_int( (int *) p, (int *) &tgts, sizeof( TGTS_RAW ) / sizeof( int ) ); // swap

    // print
    cout << "DTAR: " 
	 << "date="   << faDate( (time_t *) &tgts.time ) << " "
	 << "time="   << faTime( (time_t *) &tgts.time ) << " "
	 << "ia="     << tgts.ia                         << "MHz "
	 << "bg="     << tgts.bg * 1000                  << "kHz "
	 << "pos[0]=" << tgts.pos[0]                     << "mm "
	 << "pos[1]=" << tgts.pos[1]                     << "mm "
	 << "pos[2]=" << tgts.pos[2]                     << "mm "
	 << "pos[3]=" << tgts.pos[3]                     << "mm "
	 << "pos[4]=" << tgts.pos[4]                     << "mm "
	 << "pos[5]=" << tgts.pos[5]                     << "mm "
	 << "pos[6]=" << tgts.pos[6]                     << "mm "
	 << "pos[7]=" << tgts.pos[7]                     << "mm"   << endl;

    return ARTE_GOOD;
  }
  else {
    return ARTE_ERR;
  }
}

//
// targets ([MHz] and [mm])
//
void
faDTARtgts( TGTS_RAW *tgts )
{
  if( tgts && ArteTable<DTAR>::size() ){
    ArtePointer<DTAR> dtar( 2 ); // pointer to DTAR
    char *p = (char *) &dtar->bits + sizeof( SLB_SUBHDR ); // pointer to data
    ESP_get_int( (int *) p, (int *) tgts, sizeof( TGTS_RAW ) / sizeof( int ) ); //swap
  }
}

//
// interaction rate
//
float
faDTARia()
{
  if( ArteTable<DTAR>::size() ){
    ArtePointer<DTAR> dtar( 2 ); 
    char *p = (char *) &dtar->bits + sizeof( SLB_SUBHDR );
    struct TGTS_RAW tgts;
    ESP_get_int( (int *) p, (int *) &tgts, sizeof( TGTS_RAW ) / sizeof( int ) );  
    return tgts.ia;
  }
  else {
    return -1;
  }
}

//
// interaction rate
//
extern "C" float
faDTARia_()
{
  return faDTARia();
}


//
// background rate
//
float
faDTARbg()
{
  if( ArteTable<DTAR>::size() ){
    ArtePointer<DTAR> dtar( 2 );
    char *p = (char *) &dtar->bits + sizeof( SLB_SUBHDR );
    struct TGTS_RAW tgts;
    ESP_get_int( (int *) p, (int *) &tgts, sizeof( TGTS_RAW ) / sizeof( int ) );     
    return tgts.bg;
  }
  else {
    return -1;
  }
}

//
// background rate
//
extern "C" float
faDTARbg_()
{
  return faDTARbg();
}

//
// run number
//
int
faEVHDrun()
{
  if( ArteTable<EVHD>::size() ){
    ArtePointer<EVHD> p( 1 );
    return p->run;
  }
  else {
    return -1;
  }
}

//
// event number
//
int
faEVHDevt()
{
  if( ArteTable<EVHD>::size() ){
    ArtePointer<EVHD> p( 1 );
    return p->evt;
  }
  else {
    return -1;
  }
}

//
// experiment number
//
int
faEVHDexp()
{
  if( ArteTable<EVHD>::size() ){
    ArtePointer<EVHD> p( 1 );
    return p->exp;
  }
  else {
    return -1;
  }
}

//
// date
//
int
faEVHDdate()
{
  if( ArteTable<EVHD>::size() ){
    ArtePointer<EVHD> p( 1 );
    return p->date ;
  }
  else {
    return -1;
  }
}

//
// time
//
int
faEVHDtime()
{
  if( ArteTable<EVHD>::size() ){
    ArtePointer<EVHD> p( 1 );
    return p->time;
  }
  else {
    return -1;
  }
}

//
// dbmajor
//
int
faEVHDdbmajor()
{
  if( ArteTable<EVHD>::size() ){
    ArtePointer<EVHD> p( 1 );
    return p->dbmajor;
  }
  else {
    return -1;
  }
}

//
// dbminor
//
int
faEVHDdbminor()
{
  if( ArteTable<EVHD>::size() ){
    ArtePointer<EVHD> p( 1 );
    return p->dbminor;
  }
  else {
    return -1;
  }
}

// ****************************************************************************
//
// name:      faNameIDInit
//
// purpose:   initialize Name/ID array
//
// input:     void
// return:    int  lenght of array
//
// hints:     component.h
//
// **************************************************************************** 

const int iNameIDLen = 16; // 0 - 15
int iNameID[iNameIDLen];

int
faNameIDInit()
{
  memset( &iNameID, 0, iNameIDLen );
  iNameID[COMP_ID_DAQ /0x0100] = faName2Int( "DDAQ" ); //  0
  iNameID[COMP_ID_SVD /0x0100] = faName2Int( "DSVD" ); //  1
  iNameID[COMP_ID_ITR /0x0100] = faName2Int( "DITR" ); //  2
  iNameID[COMP_ID_HIPT/0x0100] = faName2Int( "DPTC" ); //  3
  iNameID[COMP_ID_OTR /0x0100] = faName2Int( "DOTR" ); //  4
  iNameID[COMP_ID_RICH/0x0100] = faName2Int( "DRIC" ); //  5
  iNameID[COMP_ID_TRD /0x0100] = faName2Int( "DTRD" ); //  6
  iNameID[COMP_ID_ECAL/0x0100] = faName2Int( "DCAL" ); //  7
  iNameID[COMP_ID_MUON/0x0100] = faName2Int( "DMUT" ); //  8
  iNameID[COMP_ID_MPIX/0x0100] = faName2Int( "DMUX" ); //  9
  iNameID[COMP_ID_MPAD/0x0100] = faName2Int( "DMUP" ); // 10
  iNameID[COMP_ID_FLT /0x0100] = faName2Int( "DFLT" ); // 11
  iNameID[COMP_ID_SLT /0x0100] = faName2Int( "DSLT" ); // 12
  iNameID[COMP_ID_TLT /0x0100] = faName2Int( "DTLT" ); // 13
  iNameID[COMP_ID_TARG/0x0100] = faName2Int( "DTAR" ); // 14
  iNameID[COMP_ID_FCS /0x0100] = faName2Int( "DFCS" ); // 15

#ifdef _DEB2_
  if( FADEB ){
    for( int i=0; i<iNameIDLen; i++ ){
      cout << "faNameIDInit:"
	   << "iNameID[" << i      << "]="
	   << iNameID[i]           << " "
	   << faName( iNameID[i] ) << endl;
    }
  }
#endif
  return iNameIDLen;
}

// ****************************************************************************
//
// name:      faID2Name
//
// purpose:   convert ID to table name
//
// input:     ID
// return:    table name
//
// **************************************************************************** 

int
faID2Name( int ID )
{
  // initialize array once
  static int b = 1;
  static int l = 0;
  if( b ){
    b = 0;
    l = faNameIDInit();
  }

  int i = ID / 0x0100; // index
  if( i >= l ){
    cerr << "faID2Name: Fatal index i=" << i << endl;
    return ARTE_ERR;
  }
#ifdef _DEB2_
  if( FADEB ){
    cout << "faID2Name: "
	 << "ID=0x" << hex << ID << dec << " " 
	 << "iNameID[" << i << "]=" << iNameID[i] << endl;
  }
#endif
  return iNameID[i];
}

//
//  functions
//
int
faName2Int( char *c )
{
  return *( (int *) c );
}

char *
faName( int i )
{
  static char c[5];
  strncpy( c, (char *) &i, 4 );
  return c;
}

// *****************************************************************************
//
// name:      faDate
//
// purpose:   providing current date
//
// called by: anywhere
//
// input:     seconds since Unix 0
// return:    int date yymmdd
//
// *****************************************************************************

int
faDate( time_t *tp )
{
  if( tp == 0 ){
    time_t tmp;
    tp = &tmp;
    time( tp );
  }

  struct tm *t = localtime( tp );
  
  char s[7];
  strftime( s, 7, "%y%m%d", t );
  return atoi( s );
}

// *****************************************************************************
//
// name:      faTime
//
// purpose:   providing current time
//
// called by: anywhere
//
// input:     seconds since Unix 0
// return:    int date hhmmss
//
// *****************************************************************************

int
faTime( time_t *tp )
{
  if( tp == 0 ){
    time_t tmp;
    tp = &tmp;
    time( tp );
  }

  struct tm *t = localtime( tp );
  
  char s[7];
  strftime( s, 7, "%H%M%S", t );
  return atoi( s );
}

// *****************************************************************************
//
// name:      fatime_
//
// purpose:   providing timing information
//
// called by: anywhere
//
// input:     void
// return:    float elapsed user_time
//
// *****************************************************************************

extern "C" float
fatime_()
{
  static int first = 1;
  static clock_t clock0;
  static time_t  time0;
  
  if( first ){
    first  = 0;
    clock0 = clock();
    time0  = time( 0 );
  }
  
  // double user_time = ( clock() - clock0 ) / double( CLOCKS_PER_SEC );
  double real_time = difftime( time( 0 ), time0 );
  
  // return float( user_time );
  return float( real_time );
}

// *****************************************************************************
//
// name:      faCopySlb
//
// purpose:   providing timing information
//
// called by: anywhere
//
// input:     void
// return:    float elapsed user_time
//
// *****************************************************************************
int
faCopySlb( int *oldRecID, char *pData , int *newRecID, char *pTable, int nData )
{

#ifdef _DEB2_
  // print data
  cout << "faSparseNNNN: oldRecID=0x" << hex << *oldRecID << dec << endl;
  cout << "faSparseNNNN: nData="             << nData            << endl;
  cout << "faSparseNNNN: Data: ";
  for( int i=0; i<nData; i++ ) {
    cout << "0x" << hex << int( *( pData + i ) ) << dec << " ";
  }
  cout << endl;
#endif

  // pre-set
  int nWrite = 0;

  // no data
  if( nData == 0 ){
    int error = RID_ERROR_SPARSE;
    ESP_put_int( &error, newRecID, 1 );
    return nWrite;
  }

  // rec ID
  ESP_put_int( oldRecID, newRecID, 1 );

  // data
  nWrite = nData;

  // just copy byte-wise
  for( int i=0; i<nData; i++ ) {
    *( pTable + i ) = *( pData + i );
  }

#ifdef _DEB2_
  if( FADEB ){
    int recID = 0;
    ESP_get_int( newRecID, recID, 1 );
    cout << "faSparseNNNN: newRecID=0x" 
	 << hex << *recID << dec << endl;
    cout << "faSparseNNNN: nWrite="
	 << nWrite << endl;
  }
#endif
  
  return nWrite;
}

//
// ARTE_get_recid() ************************************************************
//

//
// get record id from table
//
int
ARTE_get_recid_DSVD( int *recid )
{
  if( ArteTable<DSVD>::size() == 0 ) return ETB_corrupted_length;
  ArtePointer<DSVD> dnnn( 1 );
  ESP_get_int( (int *) &dnnn->bits, recid, 1 );
  return ETB_ok;
}

//
// get record id from table
//
int
ARTE_get_recid_DITR( int *recid )
{
  if( ArteTable<DITR>::size() == 0 ) return ETB_corrupted_length;
  ArtePointer<DITR> dnnn( 1 );
  ESP_get_int( (int *) &dnnn->bits, recid, 1 );
  return ETB_ok;
}

//
// get record id from table
//
int
ARTE_get_recid_DPTC( int *recid )
{
  if( ArteTable<DPTC>::size() == 0 ) return ETB_corrupted_length;
  ArtePointer<DPTC> dnnn( 1 );
  ESP_get_int( (int *) &dnnn->bits, recid, 1 );
  return ETB_ok;
}

//
// get record id from table
//
int
ARTE_get_recid_DOTR( int *recid )
{
  if( ArteTable<DOTR>::size() == 0 ) return ETB_corrupted_length;
  ArtePointer<DOTR> dnnn( 1 );
  ESP_get_int( (int *) &dnnn->bits, recid, 1 );
  return ETB_ok;
}

//
// get record id from table
//
int
ARTE_get_recid_DRIC( int *recid )
{
  if( ArteTable<DRIC>::size() == 0 ) return ETB_corrupted_length;
  ArtePointer<DRIC> dnnn( 1 );
  ESP_get_int( (int *) &dnnn->bits, recid, 1 );
  return ETB_ok;
}

//
// get record id from table
//
int
ARTE_get_recid_DTRD( int *recid )
{
  if( ArteTable<DTRD>::size() == 0 ) return ETB_corrupted_length;
  ArtePointer<DTRD> dnnn( 1 );
  ESP_get_int( (int *) &dnnn->bits, recid, 1 );
  return ETB_ok;
}

//
// get record id from table
//
int
ARTE_get_recid_DCAL( int *recid )
{
  if( ArteTable<DCAL>::size() == 0 ) return ETB_corrupted_length;
  ArtePointer<DCAL> dnnn( 1 );
  ESP_get_int( (int *) &dnnn->bits, recid, 1 );
  return ETB_ok;
}

//
// get record id from table
//
int
ARTE_get_recid_DMUT( int *recid )
{
  if( ArteTable<DMUT>::size() == 0 ) return ETB_corrupted_length;
  ArtePointer<DMUT> dnnn( 1 );
  ESP_get_int( (int *) &dnnn->bits, recid, 1 );
  return ETB_ok;
}

//
// get record id from table
//
int
ARTE_get_recid_DMUX( int *recid )
{
  if( ArteTable<DMUX>::size() == 0 ) return ETB_corrupted_length;
  ArtePointer<DMUX> dnnn( 1 );
  ESP_get_int( (int *) &dnnn->bits, recid, 1 );
  return ETB_ok;
}

//
// get record id from table
//
int
ARTE_get_recid_DMUP( int *recid )
{
  if( ArteTable<DMUP>::size() == 0 ) return ETB_corrupted_length;
  ArtePointer<DMUP> dnnn( 1 );
  ESP_get_int( (int *) &dnnn->bits, recid, 1 );
  return ETB_ok;
}

//
// ARTE_get_feds() *************************************************************
//

//
// get list of FED blocks (see ETB)
//
int
ARTE_get_feds_DSVD( int max, ETB_fed *feds, int *n )
{
  // inits
  int rc = ETB_ok;
  *n = 0;
  
  // check fed array
  if( ( max < 1 ) || ( feds == 0 ) ){
    return ETB_out_of_memory;
  }
  
  //
  // use DEVT
  //
  EVENT_HDR *event = ARTE_get_event();
  if( event ){
    if( ( rc = ETB_get_feds( event, COMP_ID_SVD, max, feds, n ) ) != ETB_ok ){
      cerr << "ARTE_get_feds_DSVD(): ERROR: fed array too small? "
	   << "max=" << max << " "
	   << "n="   << *n  << " "
	   << "rc="  << rc << endl;
    }
    if( *n > max ) *n = max;
    return rc;
  }
  //
  // use DSVD
  //
  else if( ArteTable<DSVD>::size() ){

    // Arte pointer
    ArtePointer<DSVD> dnnn( 1 );
    
    // data words in table (first word is recid)
    int total = ( ArteTable<DSVD>::size() - 1 ) * sizeof( int );
    
    // recid
    int recid = 0;
    ESP_get_int( (int *) dnnn->bits, &recid, 1 );
    
    // SLB_SUBHDR
    unsigned int  bc   = 0;
    unsigned char *ptr = (unsigned char *) dnnn->bits + sizeof( int );
    int j              = 0;
    
    // loop
    while( bc < ( total - sizeof( SLB_SUBHDR ) ) ){
      
      // check array size
      if( j >= max ){
	return ETB_out_of_memory;
      }
      
      // update FED count
      *n = j + 1;
      
      // ETB_fed header
      SLB_SUBHDR *hdr = &feds[j].hdr; // header of FED block
      ESP_get_slb_subhdr( (SLB_SUBHDR *)( ptr + bc ), hdr ); // copy header
      feds[j].slb  = 0; // pointer to slb record (not existent here)
      feds[j].data = ptr + bc + sizeof( SLB_SUBHDR ); // pointer to data
      
      // increase cunters
      j++; // fed counter
      bc += hdr->length; // byte counter
    }
    return ETB_ok;
  }

  // return
  return ETB_corrupted_event;
}  

//
// get list of FED blocks (see ETB)
//
int
ARTE_get_feds_DITR( int max, ETB_fed *feds, int *n )
{
  // inits
  int rc = ETB_ok;
  *n = 0;
  
  // check fed array
  if( ( max < 1 ) || ( feds == 0 ) ){
    return ETB_out_of_memory;
  }
  
  //
  // use DEVT
  //
  EVENT_HDR *event = ARTE_get_event();
  if( event ){
    if( ( rc = ETB_get_feds( event, COMP_ID_ITR, max, feds, n ) ) != ETB_ok ){
      cerr << "ARTE_get_feds_DITR(): ERROR: fed array too small? "
	   << "max=" << max << " "
	   << "n="   << *n  << " "
	   << "rc="  << rc << endl;
    }
    if( *n > max ) *n = max;
    return rc;
  }
  //
  // use DITR
  //
  else if( ArteTable<DITR>::size() ){

    // Arte pointer
    ArtePointer<DITR> dnnn( 1 );
    
    // data words in table (first word is recid)
    int total = ( ArteTable<DITR>::size() - 1 ) * sizeof( int );
    
    // recid
    int recid = 0;
    ESP_get_int( (int *) dnnn->bits, &recid, 1 );
    
    // SLB_SUBHDR
    unsigned int  bc   = 0;
    unsigned char *ptr = (unsigned char *) dnnn->bits + sizeof( int );
    int j              = 0;
    
    // loop
    while( bc < ( total - sizeof( SLB_SUBHDR ) ) ){
      
      // check array size
      if( j >= max ){
	return ETB_out_of_memory;
      }
      
      // update FED count
      *n = j + 1;
      
      // ETB_fed header
      SLB_SUBHDR *hdr = &feds[j].hdr; // header of FED block
      ESP_get_slb_subhdr( (SLB_SUBHDR *)( ptr + bc ), hdr ); // copy header
      feds[j].slb  = 0; // pointer to slb record (not existent here)
      feds[j].data = ptr + bc + sizeof( SLB_SUBHDR ); // pointer to data
      
      // increase cunters
      j++; // fed counter
      bc += hdr->length; // byte counter
    }
    return ETB_ok;
  }

  // return
  return ETB_corrupted_event;
}  

//
// get list of FED blocks (see ETB)
//
int
ARTE_get_feds_DPTC( int max, ETB_fed *feds, int *n )
{
  // inits
  int rc = ETB_ok;
  *n = 0;
  
  // check fed array
  if( ( max < 1 ) || ( feds == 0 ) ){
    return ETB_out_of_memory;
  }
  
  //
  // use DEVT
  //
  EVENT_HDR *event = ARTE_get_event();
  if( event ){
    if( ( rc = ETB_get_feds( event, COMP_ID_HIPT, max, feds, n ) ) != ETB_ok ){
      cerr << "ARTE_get_feds_DPTC(): ERROR: fed array too small? "
	   << "max=" << max << " "
	   << "n="   << *n  << " "
	   << "rc="  << rc << endl;
    }
    if( *n > max ) *n = max;
    return rc;
  }
  //
  // use DPTC
  //
  else if( ArteTable<DPTC>::size() ){

    // Arte pointer
    ArtePointer<DPTC> dnnn( 1 );
    
    // data words in table (first word is recid)
    int total = ( ArteTable<DPTC>::size() - 1 ) * sizeof( int );
    
    // recid
    int recid = 0;
    ESP_get_int( (int *) dnnn->bits, &recid, 1 );
    
    // SLB_SUBHDR
    unsigned int  bc   = 0;
    unsigned char *ptr = (unsigned char *) dnnn->bits + sizeof( int );
    int j              = 0;
    
    // loop
    while( bc < ( total - sizeof( SLB_SUBHDR ) ) ){
      
      // check array size
      if( j >= max ){
	return ETB_out_of_memory;
      }
      
      // update FED count
      *n = j + 1;
      
      // ETB_fed header
      SLB_SUBHDR *hdr = &feds[j].hdr; // header of FED block
      ESP_get_slb_subhdr( (SLB_SUBHDR *)( ptr + bc ), hdr ); // copy header
      feds[j].slb  = 0; // pointer to slb record (not existent here)
      feds[j].data = ptr + bc + sizeof( SLB_SUBHDR ); // pointer to data
      
      // increase cunters
      j++; // fed counter
      bc += hdr->length; // byte counter
    }
    return ETB_ok;
  }

  // return
  return ETB_corrupted_event;
}  

//
// get list of FED blocks (see ETB)
//
int
ARTE_get_feds_DOTR( int max, ETB_fed *feds, int *n )
{
  // inits
  int rc = ETB_ok;
  *n = 0;
  
  // check fed array
  if( ( max < 1 ) || ( feds == 0 ) ){
    return ETB_out_of_memory;
  }
  
  //
  // use DEVT
  //
  EVENT_HDR *event = ARTE_get_event();
  if( event ){
    if( ( rc = ETB_get_feds( event, COMP_ID_OTR, max, feds, n ) ) != ETB_ok ){
      cerr << "ARTE_get_feds_DOTR(): ERROR: fed array too small? "
	   << "max=" << max << " "
	   << "n="   << *n  << " "
	   << "rc="  << rc << endl;
    }
    if( *n > max ) *n = max;
    return rc;
  }
  //
  // use DOTR
  //
  else if( ArteTable<DOTR>::size() ){

    // Arte pointer
    ArtePointer<DOTR> dnnn( 1 );
    
    // data words in table (first word is recid)
    int total = ( ArteTable<DOTR>::size() - 1 ) * sizeof( int );
    
    // recid
    int recid = 0;
    ESP_get_int( (int *) dnnn->bits, &recid, 1 );
    
    // SLB_SUBHDR
    unsigned int  bc   = 0;
    unsigned char *ptr = (unsigned char *) dnnn->bits + sizeof( int );
    int j              = 0;
    
    // loop
    while( bc < ( total - sizeof( SLB_SUBHDR ) ) ){
      
      // check array size
      if( j >= max ){
	return ETB_out_of_memory;
      }
      
      // update FED count
      *n = j + 1;
      
      // ETB_fed header
      SLB_SUBHDR *hdr = &feds[j].hdr; // header of FED block
      ESP_get_slb_subhdr( (SLB_SUBHDR *)( ptr + bc ), hdr ); // copy header
      feds[j].slb  = 0; // pointer to slb record (not existent here)
      feds[j].data = ptr + bc + sizeof( SLB_SUBHDR ); // pointer to data
      
      // increase cunters
      j++; // fed counter
      bc += hdr->length; // byte counter
    }
    return ETB_ok;
  }

  // return
  return ETB_corrupted_event;
}  

//
// get list of FED blocks (see ETB)
//
int
ARTE_get_feds_DRIC( int max, ETB_fed *feds, int *n )
{
  // inits
  int rc = ETB_ok;
  *n = 0;
  
  // check fed array
  if( ( max < 1 ) || ( feds == 0 ) ){
    return ETB_out_of_memory;
  }
  
  //
  // use DEVT
  //
  EVENT_HDR *event = ARTE_get_event();
  if( event ){
    if( ( rc = ETB_get_feds( event, COMP_ID_RICH, max, feds, n ) ) != ETB_ok ){
      cerr << "ARTE_get_feds_DRIC(): ERROR: fed array too small? "
	   << "max=" << max << " "
	   << "n="   << *n  << " "
	   << "rc="  << rc << endl;
    }
    if( *n > max ) *n = max;
    return rc;
  }
  //
  // use DRIC
  //
  else if( ArteTable<DRIC>::size() ){

    // Arte pointer
    ArtePointer<DRIC> dnnn( 1 );
    
    // data words in table (first word is recid)
    int total = ( ArteTable<DRIC>::size() - 1 ) * sizeof( int );
    
    // recid
    int recid = 0;
    ESP_get_int( (int *) dnnn->bits, &recid, 1 );
    
    // SLB_SUBHDR
    unsigned int  bc   = 0;
    unsigned char *ptr = (unsigned char *) dnnn->bits + sizeof( int );
    int j              = 0;
    
    // loop
    while( bc < ( total - sizeof( SLB_SUBHDR ) ) ){
      
      // check array size
      if( j >= max ){
	return ETB_out_of_memory;
      }
      
      // update FED count
      *n = j + 1;
      
      // ETB_fed header
      SLB_SUBHDR *hdr = &feds[j].hdr; // header of FED block
      ESP_get_slb_subhdr( (SLB_SUBHDR *)( ptr + bc ), hdr ); // copy header
      feds[j].slb  = 0; // pointer to slb record (not existent here)
      feds[j].data = ptr + bc + sizeof( SLB_SUBHDR ); // pointer to data
      
      // increase cunters
      j++; // fed counter
      bc += hdr->length; // byte counter
    }
    return ETB_ok;
  }

  // return
  return ETB_corrupted_event;
}  

//
// get list of FED blocks (see ETB)
//
int
ARTE_get_feds_DTRD( int max, ETB_fed *feds, int *n )
{
  // inits
  int rc = ETB_ok;
  *n = 0;
  
  // check fed array
  if( ( max < 1 ) || ( feds == 0 ) ){
    return ETB_out_of_memory;
  }
  
  //
  // use DEVT
  //
  EVENT_HDR *event = ARTE_get_event();
  if( event ){
    if( ( rc = ETB_get_feds( event, COMP_ID_TRD, max, feds, n ) ) != ETB_ok ){
      cerr << "ARTE_get_feds_DTRD(): ERROR: fed array too small? "
	   << "max=" << max << " "
	   << "n="   << *n  << " "
	   << "rc="  << rc << endl;
    }
    if( *n > max ) *n = max;
    return rc;
  }
  //
  // use DTRD
  //
  else if( ArteTable<DTRD>::size() ){

    // Arte pointer
    ArtePointer<DTRD> dnnn( 1 );
    
    // data words in table (first word is recid)
    int total = ( ArteTable<DTRD>::size() - 1 ) * sizeof( int );
    
    // recid
    int recid = 0;
    ESP_get_int( (int *) dnnn->bits, &recid, 1 );
    
    // SLB_SUBHDR
    unsigned int  bc   = 0;
    unsigned char *ptr = (unsigned char *) dnnn->bits + sizeof( int );
    int j              = 0;
    
    // loop
    while( bc < ( total - sizeof( SLB_SUBHDR ) ) ){
      
      // check array size
      if( j >= max ){
	return ETB_out_of_memory;
      }
      
      // update FED count
      *n = j + 1;
      
      // ETB_fed header
      SLB_SUBHDR *hdr = &feds[j].hdr; // header of FED block
      ESP_get_slb_subhdr( (SLB_SUBHDR *)( ptr + bc ), hdr ); // copy header
      feds[j].slb  = 0; // pointer to slb record (not existent here)
      feds[j].data = ptr + bc + sizeof( SLB_SUBHDR ); // pointer to data
      
      // increase cunters
      j++; // fed counter
      bc += hdr->length; // byte counter
    }
    return ETB_ok;
  }

  // return
  return ETB_corrupted_event;
}  

//
// get list of FED blocks (see ETB)
//
int
ARTE_get_feds_DCAL( int max, ETB_fed *feds, int *n )
{
  // inits
  int rc = ETB_ok;
  *n = 0;
  
  // check fed array
  if( ( max < 1 ) || ( feds == 0 ) ){
    return ETB_out_of_memory;
  }
  
  //
  // use DEVT
  //
  EVENT_HDR *event = ARTE_get_event();
  if( event ){
    if( ( rc = ETB_get_feds( event, COMP_ID_ECAL, max, feds, n ) ) != ETB_ok ){
      cerr << "ARTE_get_feds_DCAL(): ERROR: fed array too small? "
	   << "max=" << max << " "
	   << "n="   << *n  << " "
	   << "rc="  << rc << endl;
    }
    if( *n > max ) *n = max;
    return rc;
  }
  //
  // use DCAL
  //
  else if( ArteTable<DCAL>::size() ){

    // Arte pointer
    ArtePointer<DCAL> dnnn( 1 );
    
    // data words in table (first word is recid)
    int total = ( ArteTable<DCAL>::size() - 1 ) * sizeof( int );
    
    // recid
    int recid = 0;
    ESP_get_int( (int *) dnnn->bits, &recid, 1 );
    
    // SLB_SUBHDR
    unsigned int  bc   = 0;
    unsigned char *ptr = (unsigned char *) dnnn->bits + sizeof( int );
    int j              = 0;
    
    // loop
    while( bc < ( total - sizeof( SLB_SUBHDR ) ) ){
      
      // check array size
      if( j >= max ){
	return ETB_out_of_memory;
      }
      
      // update FED count
      *n = j + 1;
      
      // ETB_fed header
      SLB_SUBHDR *hdr = &feds[j].hdr; // header of FED block
      ESP_get_slb_subhdr( (SLB_SUBHDR *)( ptr + bc ), hdr ); // copy header
      feds[j].slb  = 0; // pointer to slb record (not existent here)
      feds[j].data = ptr + bc + sizeof( SLB_SUBHDR ); // pointer to data
      
      // increase cunters
      j++; // fed counter
      bc += hdr->length; // byte counter
    }
    return ETB_ok;
  }

  // return
  return ETB_corrupted_event;
}  

//
// get list of FED blocks (see ETB)
//
int
ARTE_get_feds_DMUT( int max, ETB_fed *feds, int *n )
{
  // inits
  int rc = ETB_ok;
  *n = 0;
  
  // check fed array
  if( ( max < 1 ) || ( feds == 0 ) ){
    return ETB_out_of_memory;
  }
  
  //
  // use DEVT
  //
  EVENT_HDR *event = ARTE_get_event();
  if( event ){
    if( ( rc = ETB_get_feds( event, COMP_ID_MUON, max, feds, n ) ) != ETB_ok ){
      cerr << "ARTE_get_feds_DMUT(): ERROR: fed array too small? "
	   << "max=" << max << " "
	   << "n="   << *n  << " "
	   << "rc="  << rc << endl;
    }
    if( *n > max ) *n = max;
    return rc;
  }
  //
  // use DMUT
  //
  else if( ArteTable<DMUT>::size() ){

    // Arte pointer
    ArtePointer<DMUT> dnnn( 1 );
    
    // data words in table (first word is recid)
    int total = ( ArteTable<DMUT>::size() - 1 ) * sizeof( int );
    
    // recid
    int recid = 0;
    ESP_get_int( (int *) dnnn->bits, &recid, 1 );
    
    // SLB_SUBHDR
    unsigned int  bc   = 0;
    unsigned char *ptr = (unsigned char *) dnnn->bits + sizeof( int );
    int j              = 0;
    
    // loop
    while( bc < ( total - sizeof( SLB_SUBHDR ) ) ){
      
      // check array size
      if( j >= max ){
	return ETB_out_of_memory;
      }
      
      // update FED count
      *n = j + 1;
      
      // ETB_fed header
      SLB_SUBHDR *hdr = &feds[j].hdr; // header of FED block
      ESP_get_slb_subhdr( (SLB_SUBHDR *)( ptr + bc ), hdr ); // copy header
      feds[j].slb  = 0; // pointer to slb record (not existent here)
      feds[j].data = ptr + bc + sizeof( SLB_SUBHDR ); // pointer to data
      
      // increase cunters
      j++; // fed counter
      bc += hdr->length; // byte counter
    }
    return ETB_ok;
  }

  // return
  return ETB_corrupted_event;
}  

//
// get list of FED blocks (see ETB)
//
int
ARTE_get_feds_DMUX( int max, ETB_fed *feds, int *n )
{
  // inits
  int rc = ETB_ok;
  *n = 0;
  
  // check fed array
  if( ( max < 1 ) || ( feds == 0 ) ){
    return ETB_out_of_memory;
  }
  
  //
  // use DEVT
  //
  EVENT_HDR *event = ARTE_get_event();
  if( event ){
    if( ( rc = ETB_get_feds( event, COMP_ID_MPIX, max, feds, n ) ) != ETB_ok ){
      cerr << "ARTE_get_feds_DMUX(): ERROR: fed array too small? "
	   << "max=" << max << " "
	   << "n="   << *n  << " "
	   << "rc="  << rc << endl;
    }
    if( *n > max ) *n = max;
    return rc;
  }
  //
  // use DMUX
  //
  else if( ArteTable<DMUX>::size() ){

    // Arte pointer
    ArtePointer<DMUX> dnnn( 1 );
    
    // data words in table (first word is recid)
    int total = ( ArteTable<DMUX>::size() - 1 ) * sizeof( int );
    
    // recid
    int recid = 0;
    ESP_get_int( (int *) dnnn->bits, &recid, 1 );
    
    // SLB_SUBHDR
    unsigned int  bc   = 0;
    unsigned char *ptr = (unsigned char *) dnnn->bits + sizeof( int );
    int j              = 0;
    
    // loop
    while( bc < ( total - sizeof( SLB_SUBHDR ) ) ){
      
      // check array size
      if( j >= max ){
	return ETB_out_of_memory;
      }
      
      // update FED count
      *n = j + 1;
      
      // ETB_fed header
      SLB_SUBHDR *hdr = &feds[j].hdr; // header of FED block
      ESP_get_slb_subhdr( (SLB_SUBHDR *)( ptr + bc ), hdr ); // copy header
      feds[j].slb  = 0; // pointer to slb record (not existent here)
      feds[j].data = ptr + bc + sizeof( SLB_SUBHDR ); // pointer to data
      
      // increase cunters
      j++; // fed counter
      bc += hdr->length; // byte counter
    }
    return ETB_ok;
  }

  // return
  return ETB_corrupted_event;
}  

//
// get list of FED blocks (see ETB)
//
int
ARTE_get_feds_DMUP( int max, ETB_fed *feds, int *n )
{
  // inits
  int rc = ETB_ok;
  *n = 0;
  
  // check fed array
  if( ( max < 1 ) || ( feds == 0 ) ){
    return ETB_out_of_memory;
  }
  
  //
  // use DEVT
  //
  EVENT_HDR *event = ARTE_get_event();
  if( event ){
    if( ( rc = ETB_get_feds( event, COMP_ID_MPAD, max, feds, n ) ) != ETB_ok ){
      cerr << "ARTE_get_feds_DMUP(): ERROR: fed array too small? "
	   << "max=" << max << " "
	   << "n="   << *n  << " "
	   << "rc="  << rc << endl;
    }
    if( *n > max ) *n = max;
    return rc;
  }
  //
  // use DMUP
  //
  else if( ArteTable<DMUP>::size() ){

    // Arte pointer
    ArtePointer<DMUP> dnnn( 1 );
    
    // data words in table (first word is recid)
    int total = ( ArteTable<DMUP>::size() - 1 ) * sizeof( int );
    
    // recid
    int recid = 0;
    ESP_get_int( (int *) dnnn->bits, &recid, 1 );
    
    // SLB_SUBHDR
    unsigned int  bc   = 0;
    unsigned char *ptr = (unsigned char *) dnnn->bits + sizeof( int );
    int j              = 0;
    
    // loop
    while( bc < ( total - sizeof( SLB_SUBHDR ) ) ){
      
      // check array size
      if( j >= max ){
	return ETB_out_of_memory;
      }
      
      // update FED count
      *n = j + 1;
      
      // ETB_fed header
      SLB_SUBHDR *hdr = &feds[j].hdr; // header of FED block
      ESP_get_slb_subhdr( (SLB_SUBHDR *)( ptr + bc ), hdr ); // copy header
      feds[j].slb  = 0; // pointer to slb record (not existent here)
      feds[j].data = ptr + bc + sizeof( SLB_SUBHDR ); // pointer to data
      
      // increase cunters
      j++; // fed counter
      bc += hdr->length; // byte counter
    }
    return ETB_ok;
  }

  // return
  return ETB_corrupted_event;
}  
