/************************************************************************
*
* DETGEO
* written by Siegmund Nowak
*
* This routines is the standard geometry installation routine working
* together with the Geometry Data Language 
*
*       called from G..MAIN
*
************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <math.h>
#include "arte/arte.hh"
#include "arte/ArteFlag.h"
#include "arte/ArteMessage.hh"
#include "arte/ArteUtil.h"
#include "gean/zgeom.h"
#include "gean/zgvers.hh"
#include "gean/ZGESL.h"

extern "C" {  
    void detgeo_(); 
    void cggesl_(char*, char*, int*);
    void gsatt_(char*,char*,int*);  
    void gsdet_(char*,char*,int*,char*,int*,int*,int*,int*,int*,int*);
}
    int  read_dat(FILE *,int &);  
    void cr_arte(const int);   
    FILE  *utfilp(const char*,const char*);

void detgeo_() {
    extern  int first_geo[32];
    extern  FILE *filptr[32]; 
    char mot[5]; static char filnam[80];   
    zgrot ZGROT; zgmed ZGMED; zgvol ZGVOL; zgpos ZGPOS; zgvers ZGVERS; 

    ArtePointer<NESL> sl(zgmoth_.CURRSL);   
    int cmp = sl->cmp;
   
    zgvol::Set_ind_pt(cmp);

    if(first_geo[cmp-1] == 0) {

//     select file name and  open the file
       strcpy(filnam,ZGVERS.Get_vers_datafile(cmp));         
       filptr[cmp-1] = utfilp(filnam,"");
    
//     reserve storage for zgrot, zgvol and zgpos 
       zgdet::ini(24); zgrot::ini(6); zgvol::ini(32);
       zgpos::ini(128);
       first_geo[cmp-1] = 1;         
    }

    int icopy = 0;      int imodule = 0;   
    
//  now the card handling begins

    for (;;)   {
//     read in one media, rotation or volume block
       int datagroup=0; int nr=0; 
       datagroup = read_dat(filptr[cmp-1],nr); 

//     no action 
       if (datagroup == 0) continue;                          

//     media
       if (datagroup == 1) {  
           ZGMED.Create_Gmed();   continue; 
       }

//     rotations
       if (datagroup == 2) { 
           ZGROT.Create_Grot(nr);  continue; 
       }

       char volname[5];
       int nr_vol=nr;    
       strncpy(volname,ZGVOL.Get_volname(nr_vol),5);
 
//     components or modules
       if (datagroup == 3 || datagroup==4 ) {   
          if(ZGVOL.Get_sens_flag(nr_vol) != 0 ) {
//         check whether the volume already exists
           int Gvolnum = ZGVOL.Check_Gvol(volname); 
               if (Gvolnum == 0) {
                   ZGVOL.Create_Gvol(nr_vol);
                   if(ZGVOL.Get_sens_flag(nr_vol) > 2 ) {
                      char set[5];  
                      if     (cmp==LGESL_SIL)  strcpy(set ,"SIST"); 
                      else if(cmp==LGESL_WCH)  strcpy(set ,"OTST"); 
                      else if(cmp==LGESL_MSG)  strcpy(set ,"ITST"); 
                      else if(cmp==LGESL_PTC)  strcpy(set ,"PTST"); 
                      else if(cmp==LGESL_RIC)  strcpy(set ,"RIST"); 
                      else if(cmp==LGESL_TRD)  strcpy(set ,"TRST"); 
                      else if(cmp==LGESL_MUO)  strcpy(set ,"MUST"); 
                      else if(cmp==LGESL_MUX)  strcpy(set ,"MXST");
                      else if(cmp==LGESL_HOD3) strcpy(set ,"SZH3"); 
                      int iset,idet; int i1=1, i20=20, i32=32, i50=50; 
                      gsdet_(set,volname,&i1,volname,&i32,&cmp,
                             &i50,&i20,&iset,&idet);
                   }
               }
          }
       }

       else {
//        define the superlayer volume and posit it in gean
          if(ZGVOL.Get_sens_flag(nr_vol) != 0 ) {
              int Gvolnum = ZGVOL.Check_Gvol(volname); 
              if (!Gvolnum) { 
                 if(!strncmp(sl->name,volname,4))  {
                    if (sl->mfl == 0)  strncpy(mot,"MOTH",4);
                    else               strncpy(mot,"MAGN",4); 
                    int i0=0,  color=-7;  char colo[5];   strcpy(colo,"COLO");
                    cggesl_(sl->name,mot,&i0);          
                    gsatt_(sl->name,colo,&color); 
                 }
              }
          }
       }

//     modules 
       if(datagroup == 4 )    {
          if(ZGVOL.Get_sens_flag(nr_vol)!=0) ZGPOS.Gposit_all(nr_vol,icopy); 
          icopy = 0;
       }

//     superlayers
       if(datagroup == 5 || datagroup == -99)    {
          if(ZGVOL.Get_sens_flag(nr_vol) != 0) {
             ZGPOS.Gposit_all(nr_vol,imodule); 
             cr_arte(nr_vol); 
          } 
          break; 
       }

//     end of file
       if ( datagroup == -99 )  {                              
         zgdet::fin(); zgrot::fin(); zgvol::fin(); zgpos::fin();
         filptr[cmp-1] = 0;
         break;  
       }            
    }
}
  
/* detgeo_ */

