/************************************************************************
*
* ITR_ARTE
* written by Siegmund Nowak
*
* This routines allows changes in the automatically produced NEDE/NWAL
* banks for the ITR 
*
*       called from G..MAIN
*
* The transferred parameters are
* void ric_arte_(int nede,int ngwal,char lvols[],int lcopy[],int nlev)
************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <math.h>
#include "arte/arte.hh"
#include "arte/ArteFlag.h"
#include "arte/ArteMessage.hh"
#include "arte/ArteUtil.h"
#include "gean/ZGESL.h"
#include "gean/zgeom.h"
#include "gean/ZGMOTH.h"


void itr_arte(int nede,int nwal,char lvols[],int lcopy[],int nlev) {

    if(nede !=0) {  
      ArtePointer<NESL> sl(zgmoth_.CURRSL);
      ArtePointer<NEDE> gd(nede);

//    select the volume in GDL-storage
      zgvol ZGVOL; zgdet ZGDET;  zguser ZGUSER;
      float z_wire; 
      char volname[5], tag[16];
      strncpy(volname,&lvols[4*nlev],4);
      int type = lcopy[nlev];
      int itype = type>>16;
      int nr_vol = ZGVOL.Get_volnr(volname,itype);
      hgvol *vol = ZGVOL.volpointer(nr_vol) ;

//    definition of the GEM z-plane; 
      char *card = ZGUSER.Get_Buffer(LGESL_MSG,vol->index_hguser-1);
      int nitem=sscanf(card,"%s",tag);
      ToLower(tag);
      if (strncmp(tag,"gem_z",5)) {
        arteMsg << "itr_arte: key is " << tag << " instead of  'gem_z'"; 
          arteMsg.error();
      }
      card+=itemlength(card,nitem);
      nitem=sscanf(card,"%f",&z_wire);
      if (nitem!=1) {
          arteMsg << "itr_arte: check gem_z card " << card; 
          arteMsg.error();
      }
      gd->zfw[1] = .5*(gd->zmin + gd->zmax) + z_wire;
 
//    definition of the sensitive volume
//    in x
      gd->xfw[1] = gd->xfw[0] - .5 * (gd->xmin + gd->xmax) ;

      if(gd->sec < 4) gd->xfw[0] = gd->xfw[1] + gd->xmin;
      else  gd->xfw[0] = gd->xmax - (gd->xfw[1] + (gd->nwi[0]-1)*gd->pit) ;

      gd->sxmn = gd->xfw[0] - 0.5*gd->pit;
      if(fabs(gd->sxmn - gd->xmin) < gd->pit ) gd->sxmn = gd->xmin;
      gd->sxmx = gd->xfw[0] +(gd->nwi[0]-0.5)*gd->pit; 
      if(fabs(gd->sxmx - gd->xmax) < gd->pit) gd->sxmx = gd->xmax;

//    in y
      gd->y1w[1] = gd->y1w[0] - .5 * (gd->ymin + gd->ymax);
      gd->y2w[1] = gd->y2w[0] - .5 * (gd->ymin + gd->ymax);

      if(gd->sec == 1 || gd->sec == 7) {
         gd->y1w[0] = gd->y1w[1] + gd->ymin;         
         gd->y2w[0] = gd->y2w[1] + gd->ymin;
      }         
      else if(gd->sec == 3 || gd->sec == 5) { 
         gd->y2w[0] = gd->ymax - gd->y1w[1];
         gd->y1w[0] = gd->ymax - gd->y2w[1];
       }
      else {
         arteMsg << "check sector numbers in itr_arte" << gd->sec;  
         arteMsg.error();
      } 
      gd->symn = gd->y1w[0];
      gd->symx = gd->y2w[0];

//    temporary to avoid misinterpretation in older ascii files;    
      if(gd->nsl == 2)  gd->nsl = 1;

      int ipl = nede-sl->fpl+1;
      if     (gd->sec < 4 && ipl/2*2 != ipl) gd->sec = gd->sec + 1;
      else if(gd->sec > 4 && ipl/2*2 == ipl) gd->sec = gd->sec + 1;      

    } 
}

/* itr_arte_ */
