
/***********************************************************************
*
* SIL_ARTE
* written by Siegmund Nowak
* changed by Wolfgang Wagner    01.12.1998
* changed by Sven Schaller      03.09.1999
*
* This routines allows changes in the automatically produced NEDE/NWAL
* banks for the SVD 
*
*       called from cr_arte_user
*
* The transferred parameters are
* void sil_arte_(int nede,int nwal,char lvols[],int lcopy[],int nlev)
************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <math.h>
#include "arte/arte.hh"
#include "arte/ArteMessage.hh"
#include "arte/ArteUtil.h"
#include "gean/zgeom.h"


void sil_arte(int nede,int,char lvols[],int lcopy[],int nlev) {
    zgdet ZGDET;
    zgvol ZGVOL;
    zgrot ZGROT;
    char  volname[5]; 

    float x_wire[4], pol[2], x0=0., y0=0., x1=0., y1=0., d0=0., d1=0.;

    if(nede == 0) return; 
    ArtePointer<NEDE> currde(nede);
      
    strncpy(volname,&lvols[4*nlev],4);
    int type = lcopy[nlev];
    int itype = type>>16;
    int nr_vol = ZGVOL.Get_volnr(volname,itype);

//  prepare some values for correct wire posion 
//  x0/y0 and x1/y1: corner with the first read-out strip(front and back side)
//  d0/d1 distance of first wire from x0/y0 (front and backside)
    ZGDET.Get_x_wire(nr_vol, x_wire, currde->nsl); 

//  doublesided upstream
    if (itype == 1 || itype == 4) {
      x0 = currde->sxmn * currde->rot[0] + currde->symn * currde->rot[1];
      y0 = currde->sxmn * currde->rot[3] + currde->symn * currde->rot[4];
      x1 = currde->sxmn * currde->rot[0] + currde->symx * currde->rot[1];
      y1 = currde->sxmn * currde->rot[3] + currde->symx * currde->rot[4];
      d0 = x_wire[0]; 
      d1 = x_wire[1] + (currde->symx-currde->symn)*currde->cos; 
    }
//  doublesided downstream
    else if(itype == 2 || itype == 5) {
      x0 = currde->sxmx * currde->rot[0] + currde->symx * currde->rot[1];
      y0 = currde->sxmx * currde->rot[3] + currde->symx * currde->rot[4];
      x1 = currde->sxmx * currde->rot[0] + currde->symn * currde->rot[1];
      y1 = currde->sxmx * currde->rot[3] + currde->symn * currde->rot[4];
      d0 = x_wire[0] + ((currde->symx-currde->symn)*currde->sin
                     - (currde->sxmx-currde->sxmn)*currde->cos);
      d1 = x_wire[1] + (currde->sxmx-currde->sxmn)*currde->sin;
    }
//  singelsided downstream 
    else if(itype == 3) {
      x0 = currde->sxmx * currde->rot[0] + currde->symn * currde->rot[1];
      y0 = currde->sxmx * currde->rot[3] + currde->symn * currde->rot[4];
      d0 = x_wire[0] - (currde->sxmx-currde->sxmn)*currde->cos;
    }  
//  singelsided upstream
    else if(itype == 6) {
      x0 = currde->sxmn * currde->rot[0] + currde->symn * currde->rot[1];
      y0 = currde->sxmn * currde->rot[3] + currde->symn * currde->rot[4];
      d0 = x_wire[0];
    }  
//  rotation by angle = 0, 90, 180, 270 deg respectively, a0 and a1 are 
//  the new angles on front and backside
    float angle=0.;
    int wr=ZGROT.Checkxy_Grot(currde->rot,angle);

//  change xmin.., sxmn, cos and sin according to that rotation  
    float a0 =  AR_DEGRAD*angle, a1 =  AR_DEGRAD*angle ; 
    if(wr && angle != 0.) {
       ZGROT.Arot_par(currde->xmin,currde->xmax,currde->ymin,currde->ymax,
                                                               currde->rot); 
       ZGROT.Arot_par(currde->sxmn,currde->sxmx,currde->symn,currde->symx,
                                                               currde->rot);
    }        
    if(currde->sin >= 0) a0 = acos(currde->cos) + AR_DEGRAD*angle;
    else             a0 = AR_TWOPI - acos(currde->cos) + AR_DEGRAD*angle;
    a0 = fmod(a0,AR_TWOPI);
    currde->cos = cos(a0);
    currde->sin = sin(a0);
    
    a1 = a0 + AR_PIBY2;
    a1 =fmod(a1,AR_TWOPI);   

// Fill the gede-xfw with d_min and d_0 
    currde->xfw[0] = x0 * currde->cos - y0 * currde->sin + d0;
    currde->xfw[1] = (x0 - currde->sxmn)*currde->cos
                   - (y0 - currde->symn)*currde->sin + d0;

//  fill gede->y1 and gede->y2 with zero,  use sxmn.. sxymx instead
    currde->y1w[0] =0.;
    currde->y2w[0] =0.;

//  fill polarity into xfw[2]
    ZGDET.Get_polarity(nr_vol, pol, currde->nsl);
    currde->xfw[2] = pol[0];
    currde->xfw[3] = 0;

//  correct the rotation matrix
    if(wr && angle != 0.)   ZGROT.Set_UnitMatrix(currde->rot);

//  if back side exists create a second gede plane and copy all parameters
//  create relations 
    if(currde->nsl == 2) {
       struct NEDE gd;
       ArtePointer<NEDE>::append(gd);
       ArtePointer<NEDE> nextde(nede+1);      
       *nextde = *currde; 
       ArtePointer<NESL> currsl(zgmoth_.CURRSL);
       nextde->nesl =0;      
       createRelation(currsl,nextde,&NESL::nede);
       nextde->gmat =0;
       ArtePointer<GMAT> gm = currde->gmat;
       createRelation(gm, nextde, &GMAT::nede);  
       currsl->npl = currsl->npl +1 ;

// insert the zfw values 
       nextde->zfw[0] = currde->zfw[1];
       currde->zfw[1] =0.;
       nextde->zfw[1] =0.;
// insert the cos/sin values 
       nextde->cos = cos(a1);
       nextde->sin = sin(a1);

// insert the xfw values and the polarity
       nextde->xfw[0] =  x1 * nextde->cos - y1 * nextde->sin + d1;
       nextde->xfw[1] = (x1 - nextde->sxmn)*nextde->cos
                      - (y1 - nextde->symn)*nextde->sin + d1;
       nextde->xfw[2] = pol[1] ;
       nextde->xfw[3] = 0;

// insert y1,y2
       nextde->y1w[0] =0.;
       nextde->y2w[0] =0.;

// insert the wire numbers
// indicate by nwi[1] != 0 that chip is double sided 
       nextde->nwi[0] = currde->nwi[1];
       nextde->nwi[1] = currde->nwi[0];

// insert the y boundaries
       nextde->y1w[0] = currde->y1w[1];
       currde->y1w[1] = 0.;
       nextde->y1w[1] = 0.;
       nextde->y2w[0] = currde->y2w[1];
       currde->y2w[1] = 0.;
       nextde->y2w[1] = 0.;

// insert the pitch 
       float pitch[4];      
       ZGDET.Get_pitch(nr_vol,pitch,currde->nsl);      
       nextde->pit = pitch[1];        

// insert the layer number      
       nextde->lay = nextde->lay + 1;

// correct the sublayer number 
       currde->nsl = 1;
       nextde->nsl = 1;
    }
    else    currde->nwi[1] =0;
  }
  
/* sil_arte_ */



