#
# dd.awk
#
# This file creates auto.hh from all.kumac.
# Additionally, it creates the files needed for a generalization of
# the ARTE table concept: GATE (Generic Arte Table Environment).
# This allows the a similar handling of ARTE-FORTRAN objects and
# certain C++ containers.
#
# Author: Christoph Borgmeier
#

# $Source: /afs/desy.de/group/hera-b/repository/ARTE/makefiles/dd.awk,v $
# $Revision: 1.4 $
# $Author: natasha $
# $Date: 1999/09/02 13:46:37 $


#
# Return the comment of the current line
#
function getcomment()
{
  start = index($0,"\'");
  if( start!=0 ) return substr($0,start+1,length()-start-1);
  else return "";
}

#
# Read the comments of this and the next lines and decorate `line' with it.
# Proceed to the next line which is not a comment.
#
function commentline(line)
{
  ret = 0;
  if( index($0,"\'")!=0 ) {
    comment="";
    sep = "";
    do {
      comment = comment sep getcomment();
      sep = "\n";
    } while( (ret = getline) && $1=="Dtext" );
    print "/** " comment " */";
  }
  else {
    ret = getline;
    print "///";
  }
  print line;
  return ret;
}

#
# this should be \"in tablename\" which I do not understand
#
function isTable( chars,    ii )
{
  for( ii=0; ii<tablenum; ++ii ) if( chars==tablename[ii] ) return 1;
  return 0;
}


#
# these are ruins, but still used. sorry.
#
function registertable( tab, flag )
{
  if( !(isTable(tab) ) ) {
    if( !(tab in tablename) ) print "struct " tab ";";
    tablename[ flag? tablenum: tab]=tab;
    if( realtable[tab]=="" ) realtable[tab] = tab;
    if( flag ) {
      tablenum++;
      if( tab in tablename ) delete tablename[tab];
    }
  }
}

#
# Complete a struct definition.
#
function completeTable()
{
  print "#ifdef USE_ROOT";
  print "  void Streamer(TBuffer &R__b);";
  
#   print "void " table "::Streamer(TBuffer &R__b)" >"streamer.C";
#   print "{\n  if( R__b.IsReading()) {" >"streamer.C";
  
#   print "#pragma link C++ class ArteTable<" table ">-;" >"LinkDef.h";
  
#   sep = "";
#   for( j=0; j<thisidx; ++j ) {
#     if( arraycount[j]==0 )
#       print "    R__b >> " tolower(entry[j]) ";" >"streamer.C";
#     else for( k=0; k<arraycount[j]; ++k ) {
#       print "    R__b >> " tolower(entry[j]) "[" k "];" >"streamer.C";
#     }
#   }
#   printf "  } else {\n" >"streamer.C";
#   for( j=0; j<thisidx; ++j ) {
#     if( arraycount[j]==0 )
#       print "    R__b << " tolower(entry[j]) ";" >"streamer.C";
#     else for( k=0; k<arraycount[j]; ++k ) {
#       print "    R__b << " tolower(entry[j]) "[" k "];" >"streamer.C";
#     }
#   }
#   print "  }\n}\n" >"streamer.C";
  
  print "#endif";
  print "};\n ";
}

#
# Start of the main program.
#
{
  print "#ifndef _AUTO_HH_";
  print "#define _AUTO_HH_";
  print "";

#  print "#include <arte/arte.hh>\n" >"streamer.C";
#  print "#include <Gate/RtypesT.h>\n" >"streamer.C";

#  print "#ifdef __CINT__\n\n"\
#     "#pragma link off all globals;\n"\
#     "#pragma link off all classes;\n"\
#     "#pragma link off all functions;\n\n"\
#     "#pragma link C++ global artelink_;\n"\
#     "#pragma link C++ class ArteLinkStruct-;\n"\
#     >"LinkDef.h";
    
  save = $0;
  tablenum = 0;
  relatnum = 0;
# first declare all structs
  while( getline <FILENAME != 0 ) {
    two = toupper($2);
    three = toupper($3);
    if( index( tolower($1), "dtabl" ) ) {
      table = two;              # remember table for Dpar_to
      registertable(two, 0);
      kind[two] = "table";
    } else if( index( tolower($1), "dnamseq" ) ) {
      table = two;
      registertable(two, 1);
    } else if( index(tolower($1),"dpar_to") ) {
      realtable[table] = two;
      registertable(two, 0);
    } else if( tolower($1)=="drelone" ) {
      reltype[three] = "one";
      relleft[three]=two;
      realright[three]=table;
      registertable(two, 0);
      registertable(three, 0);
    } else if( tolower($1)=="dinvone" ) {
      relright[three]=two;
      realleft[three]=table;
      registertable(two, 0);
      registertable(three, 0);
    } else if( index( tolower($1), "drelman" ) ) {
      reltype[three] = "man"
      relleft[three]=two;
      realright[three]=table;
      registertable(two, 0);
      registertable(three, 0);
    } else if( index(tolower($1),"dinvman") ) {
      relright[three]=two;
      realleft[three]=table;
      registertable(two, 0);
      registertable(three, 0);
    } else if( tolower($1)=="drelat" ) {
      if( relatnum==0 ) relatnum = tablenum;
      registertable(two, 0);
      tablename[relatnum] = two;
      relatnum++;
      kind[two] = "relation";
    }
  }
  print "";

  $0 = save;
# start again
  do{
    while( index( tolower($1), "dtabl" ) ) {
      table = toupper($2);
      commentline( "struct " table );
      print "{";

      thisidx = 0;
      
      do {
	command = $1;
	name    = $2;
	type    = toupper($3);
	count   = $4;
	retval = 1;
	if( index( tolower( command), "dtabl" ) ) {
# have reached the next table
	  completeTable();
	  break;
	}
	else {
	  if( index(tolower(command),"datt" ) ) {
	    if(type=="I") {
	      comment = getcomment();
	      
	      if( comment~"((index | row |position )in " \
		  "(table |)|first |last )[A-Z]{4}" \
		  && isTable(substr(comment,match(comment,"[A-Z]{4}"),4)) \
		  ) {
		if( comment~"(first|last)" )
		  ctype = "ArteTableIterator<" \
		    substr(comment,match(comment,"[A-Z]{4}"),4) ">";
		else {
		  print "propose: " table ":ArtePointer<" \
		    substr(comment,match(comment,"[A-Z]{4}"),4) "> " name  >"/dev/null";
		  ctype = "int";
		}
	      }
	      else if( isTable(toupper(name)) \
		       && toupper(name)!="TEMP" ) {
		print "propose: " table ":ArtePointer<" toupper(name) "> " name  >"/dev/null";
		ctype = "int";
	      }
	      else ctype="int";
	    }
	    else if(type=="F") ctype="float";
	    else if(type=="H") {
	      ctype="char";
	      if( count==0 ) count = 1;
	      count *= 4
	    }
	    else if(type=="B") {
	      ctype="char";
	      count = 4;
	    }
	    if(count>0) {
	      if(type=="H") num="[" count "]";
	      else num="[" count "]";
	    } else num="";
	    retval = commentline( "  " ctype " "tolower(name) num ";" );

	    entry[thisidx] = name;
	    arraycount[thisidx] = (count~"[123456789]"? count: 0);
	    thisidx++;
	  }
	  else if( index(tolower(command),"dpoint" )) {
	    ctype="ArtePointer<" type ">";
	    retval = commentline( "  " ctype " "tolower(name) ";" );

	    entry[thisidx] = name;
	    arraycount[thisidx] = 0;
	    thisidx++;
	  }
	  else if( index(tolower(command),"drelone" )) {
	    retval = commentline( "  ArtePointer<" realtable[realleft[type]] \
				  "> " tolower(name)";" );
	  }
	  else if( index(tolower(command),"dinvone" )) {
	    retval = commentline( "  ArtePointer<" realtable[realright[type]] \
				  "> " tolower(name)";" );
	  }
	  else if( index(tolower(command),"drelman")) {
	    retval = commentline( "  ArteManRelationPointer<" \
				  type "," \
				  realtable[realleft[type]] "> " \
				  tolower(name)";" );
	  }
	  else if( index(tolower(command),"dinvman")) {
	    if( reltype[type]=="one" ) {
	      retval = commentline( "  ArteInvOneRelationPointer<" \
				    type "," \
				    realtable[realright[type]] "> " \
				    tolower(name)";" );
	    } else {
	      retval = commentline( "  ArteInvManRelationPointer<" \
				    type "," \
				    realtable[realright[type]] "> " \
				    tolower(name)";" );
	    }
	  } else {
	    retval = getline;
	    continue;
	  }
	}
      } while( retval!=0 );
    }
  } while(getline!=0);

  completeTable();
  print "";
  print "#ifdef __CINT__";

#  print "#include <Gate/RtypesT.h>" >"instanciation.hh";
#  print "ClassImpT(ArteTable,T)" >"instanciation.hh";

#  print "// specializations for egcs >1.1" >"specialize.h";
#  print "#if defined(USE_ROOT) && (defined(__CINT__) || __GNUC_MINOR__>=91)"\
#    >"specialize.h";

  for( i=0; i<tablenum; ++i ) {
    if( kind[tablename[i]]=="table" ) {

      # sdtout
      print "template ArteTable<" tablename[i] ">;";

# instanciation.hh
# print "template class ArteTable<" tablename[i] ">;" >"instanciation.hh";
# print "template class ArteTableIterator<" tablename[i] \
#	">;" >"instanciation.hh";
	
# specialize.hh
#print "void ArteTable<" tablename[i] ">::ShowMembers("\
#	  "TMemberInspector &, char *);" >"specialize.h";
#print "const char* ArteTable<" tablename[i] ">::SetClassName();"\
#	    >"specialize.h";
    } else {
      # fix undeclared relations
      if( !realright[tablename[i]] ) realright[tablename[i]] = "BEAM";
      if( !realleft[tablename[i]] ) realleft[tablename[i]] = "BEAM";

      #stdout
      print "class ArteRelationPointer<" \
	tablename[i]","realtable[realright[tablename[i]]] \
	">;";

#instanciation.hh
#print "template class ArteRelVector<" \
#	tablename[i] ">;" >"instanciation.hh";
    }
  }
  print "#endif /* defined(__CINT__) */";
  print "";
  print "#ifndef CREATELINKS";
  print "extern";
  print "#endif";
  print "struct ArteLinkStruct";
  print "{";
  print "#ifndef NO_ARTE";
  print "ArteVector larln1,larlnl,narlnk,jlink;";
  print "#endif";
  for( i=0; i<tablenum; ++i ) {
    if( kind[tablename[i]]=="table" ) {
      print "ArteVector j" tolower(tablename[i]) ";";
    } else {
      print "ArteRelVector<" tablename[i] "> j" tolower(tablename[i]) ";";
    }
  }  
  print "ArteVector jlast;";
  print "} artelink_;";

  for( i=0; i<tablenum; ++i ) {
    if( kind[tablename[i]]=="table" ) {
      print "template <> inline void* ArteTable<" \
	tablename[i] ">::getLink()" \
	"\n{ return &artelink_.j" tolower(tablename[i]) "; }\n";
    } else {
      if( reltype[tablename[i]]=="one" ) {
	print "template <> inline ArteRelVector<" tablename[i] ">& "\
	  "ArteInvOneRelationPointer<"\
	  tablename[i] "," realtable[realright[tablename[i]]] ">::getLink()" \
	  "\n{ return artelink_.j" tolower(tablename[i]) "; }\n";
      } else {
	print "template <> inline ArteRelVector<" tablename[i] ">& "\
	  "ArteManRelationPointer<"\
	  tablename[i] "," realtable[realleft[tablename[i]]] ">::getLink()" \
	  "\n{ return artelink_.j" tolower(tablename[i]) "; }\n";
	print "template <> inline ArteRelVector<" tablename[i] ">& "\
	  "ArteInvManRelationPointer<"\
	  tablename[i] "," realtable[realright[tablename[i]]] ">::getLink()" \
	  "\n{ return artelink_.j" tolower(tablename[i]) "; }\n";
      }
    }
  }
  print "#endif  /* defined(_AUTO_HH_) */";
  # print "#endif  /* __CINT || __GNUC_MINOR__ */" >"specialize.h";
  # print "#endif  /* __CINT__ */" >"LinkDef.h";
}

