//##
//      /**Author:        Boris Fominykh
//
//      Date:             March, 10,  1999
//
//      Structure:        MuDigiHitb
//
//      Description:
//      Structure MuDigiHitb and the set of related routines are
//      for the digi->hitb transformation of muon detector hits.*/

#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include <math.h>
#include <fstream.h>
#include <iostream.h>
#include <iomanip.h>
#include <stdlib.h>

#include <sys/types.h>
#include <netinet/in.h>
#include "etb.h"

#include "arte/arte.hh"
#include "arte/ArteMessage.hh"

typedef unsigned int   LWord;
typedef unsigned short Word;

extern "C" {
void AddTube(int,int);
void AddPad();
void AddPix();
int MuDHini();
void MuPHini();
void MuDiHi();
int ArteIni();
void PixConn();
int ConnectIni();
void TubeLoops();
void PadLoops();
void PixLoops();
void LoopsPW(Word,Word,Word);
int CheckBit(Word,Word,LWord);
void MuHotListIni();
void MuHotListRead();
void MuHotListFree();
Word MuHotListCheck(char*);
}

struct MuDigiHitb {

    Word Ivalid;
    Word Endian;

// tube chambers  -  16226 bytes
    const Word TubeGeoNx     = 30;
    const Word TubeGeoUpDown = 2;
    const Word TubeGeoLay    = 3;
    const Word TubeGeoSl     = 2;
    const Word TubeGeoSd     = 2;

  //## number of tube chambers
    Word NchambT[TubeGeoSl][TubeGeoLay][TubeGeoUpDown];
  //## number of wires in the tube chamber
    Word TubeGeoWire;
  //## gede
    LWord TubeGede[TubeGeoSl][TubeGeoLay][TubeGeoUpDown][TubeGeoNx][TubeGeoSd];
  //## chamber names
    char ChamListT[TubeGeoSl][TubeGeoLay][TubeGeoUpDown][TubeGeoNx][9];
  //## number of half-connectors per chamber
    Word NconnT[TubeGeoSl][TubeGeoLay][TubeGeoUpDown][TubeGeoNx];
    const Word Nconn      = 2;
  //## number of connected wires by half-connector
    const Word  NumbT = 8;
  //## mask
    Word  MaskT[TubeGeoSl][TubeGeoLay][TubeGeoUpDown][TubeGeoNx][Nconn];
  //## first connected wire number
    Word  Wire1T[TubeGeoSl][TubeGeoLay][TubeGeoUpDown][TubeGeoNx][Nconn][TubeGeoSd];
  //## first bit inside crate
    Word  Nbit1T[TubeGeoSl][TubeGeoLay][TubeGeoUpDown][TubeGeoNx][Nconn][TubeGeoSd];
  //##  crate
    Word  CrateT[TubeGeoSl][TubeGeoLay][TubeGeoUpDown][TubeGeoNx];
  //## step (+2 or -2)
    int  StepT[TubeGeoSl][TubeGeoLay][TubeGeoUpDown][TubeGeoNx];

// pad chambers  -  11298 bytes
    const Word PadGeoSl     = 2;
    const Word PadGeoUpDown = 2;
    const Word PadGeoCh     = 34;
    const Word PadGeoSd     = 2;
    const Word PadGeoRow    = 2;

  //## number of pad chambers
    Word NchambP[PadGeoSl][PadGeoUpDown];
  //## number of wires in the pad chamber
    Word PadGeoWire;
  //## number of pads per row
    Word Npads[PadGeoSl][PadGeoUpDown][PadGeoCh];
  //## gede
    LWord PadGede[PadGeoSl][PadGeoUpDown][PadGeoCh][PadGeoSd];
  //## chamber names
    char ChamListP[PadGeoSl][PadGeoUpDown][PadGeoCh][9];

  //## number of tube readout half-connectors per chamber
    Word NconnP[PadGeoSl][PadGeoUpDown][PadGeoCh];
  //## tube readout mask
    Word  MaskP[PadGeoSl][PadGeoUpDown][PadGeoCh][Nconn];
  //## first connected wire
    Word  Wire1P[PadGeoSl][PadGeoUpDown][PadGeoCh][Nconn][PadGeoSd];
  //## first bit inside crate
    Word  Nbit1P[PadGeoSl][PadGeoUpDown][PadGeoCh][Nconn][PadGeoSd];
  //## crate for wire readout
    Word  CrateP[PadGeoSl][PadGeoUpDown][PadGeoCh];
  //## step for wire readout(+2 or -2)
    int  StepP[PadGeoSl][PadGeoUpDown][PadGeoCh];

  //## number of pad readout connectors per chamber
    Word NconPP[PadGeoSl][PadGeoUpDown][PadGeoCh];
    const Word NconP = 4;
  //## pad readout mask
    Word  MaskPP[PadGeoSl][PadGeoUpDown][PadGeoCh][NconP];
  //## first connected pad
    Word  Pad1[PadGeoSl][PadGeoUpDown][PadGeoCh][NconP];
  //## number of connected pads per connector
    Word  NumbP[PadGeoSl][PadGeoUpDown][PadGeoCh][NconP];
  //## first bit inside crate
    Word  Nbit1PP[PadGeoSl][PadGeoUpDown][PadGeoCh][NconP];
  //## crate number
    Word  CratePP[PadGeoSl][PadGeoUpDown][PadGeoCh];

//  pixel chambers  -  9456 bytes
    const Word PixGeoSl = 4;
    const Word PixGeoQu = 4;
    const Word PixGeoNx = 48;
    const Word PixGeoNy = 17;

    Word PixSlAdd;
    Word PixQuAdd;
    Word PixConAdd;
    Word PixBitAdd;

  // number of connected pixels
    Word PixGeoNxC[PixGeoSl];
    Word PixGeoNyC[PixGeoSl];
  // NxMax and NyMax
    Word PixGeoNxM[PixGeoSl];
    Word PixGeoNyM[PixGeoSl];
  // GEDE volumes
    LWord PixGede[PixGeoSl][PixGeoQu];
  // list of names
    char ChamListX[PixGeoSl][PixGeoQu][6];

  //## number of connectors per chamber
    const Word PixNcon = 24;
  //## number of bits in the connector
    const Word PixNbit = 16;
  //## connected pixel in x
    Word  Pnx1[PixGeoSl][PixNcon][PixNbit];
  //##  connected pixel in y
    Word  Pny1[PixGeoSl][PixNcon][PixNbit];

  //##  pixel readout mask
    Word MaskX[PixGeoSl][PixGeoQu][PixNcon];
  //## first bit inside crate, the sign indicates the direction
    int  Nbit1X[PixGeoSl][PixGeoQu][PixNcon];
  //## crate number
    Word  CrateX[PixGeoSl][PixGeoQu][PixNcon];
};

static MuDigiHitb  MDH;

extern "C" 
{

int MuDHini()
{
  MuHotListIni();
  LWord cc = 0x01020304;
// not change the bit order for Big endian   
  if(cc == htonl(cc)) MDH.Endian=0;
// change the bit order for Little endian
  else MDH.Endian=1;
// read the list of hot channels
  MuHotListRead();
// initialization from ARTE tables
  int st1 = ArteIni();
// set the pixel chamber mapping
  PixConn();
// read the configuration file
  int st2 = ConnectIni();
// free the space
  MuHotListFree();
  if(st1 == 1 && st2 == 1) return 1;
  else return 0;
}

void MuDiHi()
{
// set the pointers to hits
  MuPHini();
// loops
  TubeLoops();
  PadLoops();
  PixLoops();
}

void clear()
{
// tube chambers
  for(Word sl=0; sl<MDH.TubeGeoSl; sl++) {
    for(Word lay=0; lay<MDH.TubeGeoLay; lay++) {
      for(Word ud=0; ud<MDH.TubeGeoUpDown; ud++) {
        MDH.NchambT[sl][lay][ud] = 0;
        for(Word nx=0; nx<MDH.TubeGeoNx; nx++) {
          char* name = "        ";
          strncpy(&MDH.ChamListT[sl][lay][ud][nx][0],name,9);
          MDH.TubeGede[sl][lay][ud][nx][0] = 0;
          MDH.TubeGede[sl][lay][ud][nx][1] = 0;
          MDH.NconnT[sl][lay][ud][nx]=0;
          MDH.CrateT[sl][lay][ud][nx]=0;
          MDH.StepT[sl][lay][ud][nx] =0;
          for(Word nc=0; nc<MDH.Nconn; nc++) {
            for(Word sd=0; sd<MDH.TubeGeoSd; sd++) {
              MDH.Wire1T[sl][lay][ud][nx][nc][sd]=0;
              MDH.Nbit1T[sl][lay][ud][nx][nc][sd]=0;
            }
          }
        }
      }
    }
  }

// pad chambers
  for(Word sl=0; sl<MDH.PadGeoSl; sl++) {
    for(Word ud=0; ud<MDH.PadGeoUpDown; ud++) {
      MDH.NchambP[sl][ud] = 0;
      for(Word nx=0; nx<MDH.PadGeoCh; nx++) {
        char* name = "        ";
        strncpy(&MDH.ChamListP[sl][ud][nx][0],name,9);
        MDH.Npads[sl][ud][nx]    = 0;
        MDH.NconPP[sl][ud][nx]    = 0;
        MDH.PadGede[sl][ud][nx][0] = 0;
        MDH.PadGede[sl][ud][nx][1] = 0;
        MDH.NconnP[sl][ud][nx]=0;
        MDH.NconPP[sl][ud][nx]=0;
        MDH.CrateP[sl][ud][nx]=0;
        MDH.CratePP[sl][ud][nx]=0;
        MDH.StepP[sl][ud][nx] =0;
        for(Word nc=0; nc<MDH.Nconn; nc++) {
          for(Word sd=0; sd<MDH.PadGeoSd; sd++) {
            MDH.MaskP[sl][ud][nx][nc]=0;
            MDH.Wire1P[sl][ud][nx][nc][sd]=0;
            MDH.Nbit1P[sl][ud][nx][nc][sd]=0;
          }
        }
        for(Word nc=0; nc<MDH.NconP; nc++) {
          MDH.MaskPP[sl][ud][nx][nc]=0;
          MDH.Pad1[sl][ud][nx][nc]=0;
          MDH.Nbit1PP[sl][ud][nx][nc]=0;
          MDH.NumbP[sl][ud][nx][nc]=0;
        }
      }
    }
  }

// pixel chambers
  for(Word sl=0; sl<MDH.PixGeoSl; sl++) {
    MDH.PixGeoNxC[sl]=0;
    MDH.PixGeoNxM[sl]=0;
    MDH.PixGeoNyC[sl]=0;
    MDH.PixGeoNyM[sl]=0;
    for(Word nc=0; nc<MDH.PixNcon; nc++) {
      for(Word nb=0; nb<MDH.PixNbit; nb++) {
        MDH.Pnx1[sl][nc][nb] = 0;
        MDH.Pny1[sl][nc][nb] = 0;
      }
    }
    for(Word qu=0; qu<MDH.PixGeoQu; qu++) {
      MDH.PixGede[sl][qu] = 0;
      char* name = "     ";
      strncpy(&MDH.ChamListX[sl][qu][0],name,6);
      for(Word nc=0; nc<MDH.PixNcon; nc++) {
        MDH.MaskX[sl][qu][nc]  = 0;
        MDH.Nbit1X[sl][qu][nc] = 0;
        MDH.CrateX[sl][qu][nc] = 0;
      }
    }
  }

  MDH.Ivalid = 0;
  return;
}

void PixConn()
{
// MU1,2 low density option
  int nc = 0;
  for(int sl=0; sl<2; sl++) {
// connector 1
    nc = 0;
    for(int i=0; i<8; i++) {
      MDH.Pnx1[sl][nc][2*i] = 1;
      MDH.Pny1[sl][nc][2*i] = 17-i;
    }
// connector 4
    nc = 3;
    MDH.Pnx1[sl][nc][0] = 5;
    MDH.Pnx1[sl][nc][1] = 4;

    MDH.Pny1[sl][nc][0] = 9;
    MDH.Pny1[sl][nc][1] = 9;
// connector 6
    nc = 5;
    MDH.Pnx1[sl][nc][0] = 7;
    MDH.Pnx1[sl][nc][1] = 6;
    MDH.Pnx1[sl][nc][2] = 7;
    MDH.Pnx1[sl][nc][3] = 6;

    MDH.Pny1[sl][nc][0] = 9;
    MDH.Pny1[sl][nc][1] = 9;
    MDH.Pny1[sl][nc][2] = 8;
    MDH.Pny1[sl][nc][3] = 8;
// connector 8
    nc = 7;
    for(int i=0; i<7; i++) {
      MDH.Pnx1[sl][nc][i] = 9-i%2;
      MDH.Pny1[sl][nc][i] = 9-i/2;
    }
    MDH.Pnx1[sl][nc][8] = 9;
    MDH.Pny1[sl][nc][8] = 5;
// connector 18
    nc = 17;
    for(int i=0; i<14; i++) {
      MDH.Pnx1[sl][nc][i] = 10+i;
      MDH.Pny1[sl][nc][i] = 1;
    }
// connectors 3,5,7,9,11,13,15,17,19,21,23
    Word r1=17;
    Word c1=0;      
    for(int nc=2; nc<23; nc+=2) {
      c1=nc+1;      
      for(int i=0; i<16; i++) {
        MDH.Pnx1[sl][nc][i] = c1-i%2;
        MDH.Pny1[sl][nc][i] = r1-i/2;
      }
    }
// connectors 10,12,14,16,20,22,24
    r1=9;
    for(int nc=9; nc<24; nc+=2) {
      if(nc != 17) { 
        if(nc < 17) c1=nc+2;
        else c1=nc;
        for(int i=0; i<16; i++) {
          MDH.Pnx1[sl][nc][i] = c1-i%2;
          MDH.Pny1[sl][nc][i] = r1-i/2;
        }
      }
    }
  }
 
// MU3
  int sl=2;
// connector 2
  nc = 1;
  for(int i=0; i<4; i++) {
    MDH.Pnx1[sl][nc][2*i+1] = 1;
    MDH.Pny1[sl][nc][2*i+1] = 8-i;
  }
// connectors 3 - 24
  Word r1=8;
  Word c1=0;      
  int chm = 0;
  for(int nc=2; nc<24; nc++) {
    c1=2*nc-1;      
    if(nc > 1 && nc < 4)  chm=8;
    if(nc > 3 && nc < 8)  chm=10;
    if(nc > 7 && nc < 10) chm=12;
    if(nc == 10)          chm=15;
    if(nc > 10)           chm=16;
    for(int i=0; i<chm; i++) {
      MDH.Pnx1[sl][nc][i] = c1-i%2;
      MDH.Pny1[sl][nc][i] = r1-i/2;
    }
  }
 
// MU4
  sl=3;
// connectors 1 - 24
  r1=9;
  for(int nc=0; nc<24; nc++) {
    c1=2*nc+2;      
    if(nc < 5)            chm=10;
    if(nc > 4 && nc < 7)  chm=12;
    if(nc == 7)           chm=13;
    if(nc == 8)           chm=15;
    if(nc > 8)            chm=16;
    for(int i=0; i<chm; i++) {
      MDH.Pnx1[sl][nc][i] = c1-i%2;
      MDH.Pny1[sl][nc][i] = r1-i/2;
    }
  }

  for(int nc=0; nc<6; nc++) {
    if(nc < 3) {
      MDH.Pnx1[sl][nc][10] = 20+nc*2;
      MDH.Pnx1[sl][nc][11] = 19+nc*2;
    }
    MDH.Pnx1[sl][nc][12] = 38+nc*2;
    MDH.Pnx1[sl][nc][13] = 37+nc*2;
    MDH.Pnx1[sl][nc][14] = 26+nc*2;
    MDH.Pnx1[sl][nc][15] = 25+nc*2;

    for(int i=10; i<16; i++) {
      if(nc < 3 || (nc > 2 && i > 11)) { 
        MDH.Pny1[sl][nc][i] = 1;
      }
    }
  }
}

void MuDHprint()
{
// create the log-file and pipe the programm output
  ofstream out("MuDigiHitb.print");
  if (!out) 
  {
    arteMsg << "MuDigiHitb - cannot open file for print" << endl;
    arteMsg.warn();
    return;
  }
  cout = out;
  cerr = out;

  int print=7;
  
  if(print == 1 || print == 3 || print == 5 || print == 7) {
  cout << "MuDigiHitb - Tube Chambers, Ivalid= " << MDH.Ivalid << endl;
  for(Word sl=0; sl<MDH.TubeGeoSl; sl++) {
    for(Word lay=0; lay<MDH.TubeGeoLay; lay++) {
      for(Word ud=0; ud<MDH.TubeGeoUpDown; ud++) {
	cout << "tube chamber sl " << dec << sl << " lay " << lay;
        cout << " UpDown " << ud << " Nchamb " << MDH.NchambT[sl][lay][ud] << endl; 
        if(MDH.NchambT[sl][lay][ud] > 0) {
          for(Word nx=0; nx<MDH.NchambT[sl][lay][ud]; nx++) {
            char name[9];
            strncpy(name,&MDH.ChamListT[sl][lay][ud][nx][0],9);
	    cout << name << " nx " << nx;
            cout << " Ncable " << MDH.NconnT[sl][lay][ud][nx] << endl;
            if(MDH.NconnT[sl][lay][ud][nx] > 0) {
              for(Word nc=0; nc<MDH.NconnT[sl][lay][ud][nx]; nc++) {
                for(Word sd=0; sd<MDH.TubeGeoSd; sd++) {
        	  cout << "   #cable " << nc;
                  cout << " mask " << hex << MDH.MaskT[sl][lay][ud][nx][nc];
                  cout << " side " << dec << sd;
		  cout << " wire1 " << dec << MDH.Wire1T[sl][lay][ud][nx][nc][sd];
		  cout << " crate " << hex << MDH.CrateT[sl][lay][ud][nx];
		  cout << " Nbit1 " << dec << MDH.Nbit1T[sl][lay][ud][nx][nc][sd];
                  Word word = MDH.Nbit1T[sl][lay][ud][nx][nc][sd] / 32;
                  Word bit  = MDH.Nbit1T[sl][lay][ud][nx][nc][sd] - word*32;
                  cout << " (" << word << "-" << bit << ") ";
		  cout << " step " << dec << MDH.StepT[sl][lay][ud][nx];
		  cout << " gede " << dec << MDH.TubeGede[sl][lay][ud][nx][sd];
                  cout << endl;
                }
              }
            }
          }
        }
      }
    }
  }
  }

  if(print == 2 || print == 3 || print == 6 || print == 7) {
  cout << "MuDigiHitb - Pad Chambers, Ivalid= " << MDH.Ivalid << endl;
  for(Word sl=0; sl<MDH.PadGeoSl; sl++) {
    for(Word ud=0; ud<MDH.PadGeoUpDown; ud++) {
      cout << "pad chamber sl " << dec << sl+2;
      cout << " UpDown " << ud << " Nchamb " << MDH.NchambP[sl][ud] << endl; 
      if(MDH.NchambP[sl][ud] > 0) {
        for(Word nx=0; nx<MDH.NchambP[sl][ud]; nx++) {
          char name[9];
          strncpy(name,&MDH.ChamListP[sl][ud][nx][0],9);
 cout << name << " nx " << nx;
 cout << " wires " << MDH.PadGeoWire << " Ncab " << MDH.NconnP[sl][ud][nx];
 cout << " pads " << 2*MDH.Npads[sl][ud][nx] << " Ncab " << MDH.NconPP[sl][ud][nx] << endl;
          if(MDH.NconnP[sl][ud][nx] > 0) {
            for(Word nc=0; nc<MDH.NconnP[sl][ud][nx]; nc++) {
              for(Word sd=0; sd<MDH.PadGeoSd; sd++) {
                cout << "   #cableW " << nc;
                cout << " mask " << hex << MDH.MaskP[sl][ud][nx][nc];
                cout << " side " << dec << sd;
                cout << " wire1 " << dec << MDH.Wire1P[sl][ud][nx][nc][sd] ;
		cout << " crate " << hex << MDH.CrateP[sl][ud][nx];
		cout << " Nbit1 " << dec << MDH.Nbit1P[sl][ud][nx][nc][sd] ;
                Word word = MDH.Nbit1P[sl][ud][nx][nc][sd] / 32 ;
                Word bit  = MDH.Nbit1P[sl][ud][nx][nc][sd] - word*32 ;
                cout << " (" << word << "-" << bit << ") ";
		cout << " step " << dec << MDH.StepP[sl][ud][nx];
		cout << " gede " << dec << MDH.PadGede[sl][ud][nx][sd] ;
                cout << endl;
              }
            }
          }
          if(MDH.NconPP[sl][ud][nx] > 0) {
            for(Word nc=0; nc<MDH.NconPP[sl][ud][nx]; nc++) {
              cout << "   #cableP " << nc;
              cout << " mask " << hex << MDH.MaskPP[sl][ud][nx][nc];
              cout << " pad1 " << dec << MDH.Pad1[sl][ud][nx][nc];
              cout << " Npads " << dec << MDH.NumbP[sl][ud][nx][nc];
              cout << " crate " << hex << MDH.CratePP[sl][ud][nx];
              cout << " Nbit1 " << dec << MDH.Nbit1PP[sl][ud][nx][nc];
              Word word = MDH.Nbit1PP[sl][ud][nx][nc] / 32 ;
              Word bit  = MDH.Nbit1PP[sl][ud][nx][nc] - word*32 ;
              cout << " (" << word << "-" << bit << ") ";
              cout << " gede1 " << dec << MDH.PadGede[sl][ud][nx][0];
              cout << " gede2 " << dec << MDH.PadGede[sl][ud][nx][1];
              cout << endl;
            }
          }
        }
      }
    }
  }
  }


  if(print == 4 || print == 5 || print == 6 || print == 7) {
  cout << "MuDigiHitb - Pixel Chambers, Ivalid= " << MDH.Ivalid << endl;
  for(Word sl=0; sl<MDH.PixGeoSl; sl++) {
    cout << "pixel chamber sl " << dec << sl;
    cout << " Npix X " << MDH.PixGeoNxM[sl] << " con " << MDH.PixGeoNxC[sl];
    cout << " Npix Y " << MDH.PixGeoNyM[sl] << " con " << MDH.PixGeoNyC[sl];
    cout << endl; 
    for(Word nc=0; nc<MDH.PixNcon; nc++) {
      cout << "connector " << dec << nc << " :" << endl;
      for(Word nb=0; nb<MDH.PixNbit; nb++) {
        cout << "   bit " << nb << " pix ";
        cout << MDH.Pnx1[sl][nc][nb] << " - " << MDH.Pny1[sl][nc][nb];
        if((nb+1)%4 == 0) cout << endl;
      }
      cout << endl;
    }
    for(Word qu=0; qu<MDH.PixGeoQu; qu++) {
      cout << "pixel sl " << dec << sl << " quadrant " << qu;
      cout << " gede " << MDH.PixGede[sl][qu] << "  ";
      char name[9];
      strncpy(name,&MDH.ChamListX[sl][qu][0],9);
      cout << name << endl;
      for(Word nc=0; nc<MDH.PixNcon; nc++) {
        cout << "  conn " << dec << nc;
        cout << " msk " << hex << MDH.MaskX[sl][qu][nc];
        cout << " cr " << dec << MDH.CrateX[sl][qu][nc];
        cout << " bit1 " << MDH.Nbit1X[sl][qu][nc];
        if((nc+1)%2 == 0) cout << endl;
      }
      cout << endl;
    }
  }
  }
}

// initiate the structure MDH from the ARTE tables
int ArteIni() 
{
  clear();
// number of tube chambers per layer
//       ud  l  sl
// sl=0, UpDown=0
  MDH.NchambT[0][0][0] = 27;
  MDH.NchambT[0][1][0] = 28;
  MDH.NchambT[0][2][0] = 28;
// sl=0, UpDown=1
  MDH.NchambT[1][0][0] = 29;
  MDH.NchambT[1][1][0] = 28;
  MDH.NchambT[1][2][0] = 28;
// sl=1, UpDown=0
  MDH.NchambT[0][0][1] = 28;
  MDH.NchambT[0][1][1] = 29;
  MDH.NchambT[0][2][1] = 29;
// sl=1, UpDown=1
  MDH.NchambT[1][0][1] = 30;
  MDH.NchambT[1][1][1] = 29;
  MDH.NchambT[1][2][1] = 29;

// number of pad chambers per superlayer
  MDH.NchambP[0][0] = 33;
  MDH.NchambP[0][1] = 34;
  MDH.NchambP[1][0] = 33;
  MDH.NchambP[1][1] = 34;

// number of pixels in the chamber
  MDH.PixGeoNxM[0] = 45;
    MDH.PixGeoNyM[0] = 34;
      MDH.PixGeoNxC[0] = 2;
        MDH.PixGeoNyC[0] = 2;
  MDH.PixGeoNxM[1] = 45;
    MDH.PixGeoNyM[1] = 34;
      MDH.PixGeoNxC[1] = 2;
        MDH.PixGeoNyC[1] = 2;
  MDH.PixGeoNxM[2] = 45;
    MDH.PixGeoNyM[2] = 34;
      MDH.PixGeoNxC[2] = 1;
        MDH.PixGeoNyC[2] = 4;
  MDH.PixGeoNxM[3] = 48;
    MDH.PixGeoNyM[3] = 36;
      MDH.PixGeoNxC[3] = 1;
        MDH.PixGeoNyC[3] = 4;

// loops over GEDE
  LWord Ngede = 0;

  char memo[9] = "        ";
  char numb[11] = "0123456789";
  for ( ArteTableIterator<GEDE> iv = ArteTable<GEDE>::begin(); 
        iv != ArteTable<GEDE>::end();
        ++iv ) {
    Ngede++;
    int   cmp  = (*iv)->cmp;
// tube and pad chambers
    if(cmp == 8) {
      int  lay  = (*iv)->lay - 1;   // # layer
      ArtePointer<GESL> gesl = (*iv)->gesl;
      int msl  = gesl->nsl;            // # muon superlayer
      //      float xd1  = ArtePointer<GEDE>(iv)->xmin;  // Xmin of chamber
      //      float xd2  = ArtePointer<GEDE>(iv)->xmax;  // Xmax of chamber
      float yd1  = (*iv)->ymin;  // Ymin of chamber
      float yd2  = (*iv)->ymax;  // Ymax of chamber
      //      float zd1  = ArtePointer<GEDE>(iv)->zmin;  // Zmin of chamber
      //      float zd2  = ArtePointer<GEDE>(iv)->zmax;  // Zmax of chamber

      int iup=0;                                // UpDown
      float absYd1 = (yd1>=0. ? yd1 : -yd1);
      float absYd2 = (yd2>=0. ? yd2 : -yd2);
      if(absYd1 > absYd2) iup=1;

// number of rows
//      int Nrow = ArteTableIterator<GEDE>::nrow();
//  cout << " " << endl;
//  cout << "Ngede " << Ngede << " Nrow " << Nrow;
//  cout << " iv,sl,lay: " << iv << " " << msl << " " << lay << " iup " << iup;
//  cout << " xd1,xd2 " << xd1 << " " << xd2 << endl;
//  cout << "yd1,yd2 " << yd1 << " " << yd2;
//  cout << " zd1,zd2 " << zd1 << " " << zd2 << endl;

// chamber name
      char name[9] = "        ";
      strncpy(name,(*iv)->hwc,9);

// decode chamber number
      int n1 = 0;
      char ch2 = name[6];
      for(int i=0; i<11; i++) {
        char ch1 = numb[i]; 
        if(ch1 == ch2) {
          n1 = i;
          break;
        }
      }
      int n2 = 0;
      ch2 = name[7];
      for(int i=0; i<11; i++) {
        char ch1 = numb[i]; 
        if(ch1 == ch2) {
          n2 = i;
          break;
        }
      }
      int nx = 10*n1+n2-1;

// side
      int iside=0;
      if(!strcmp(name,memo)) iside = 1;
      strncpy(memo,name,9);

//    cout << name << " msl,lay " << msl << " " << lay << " nx " << nx;
//    cout << " side " << iside << " n1,n2 " << n1 << " " << n2;
//    cout << " Ngede " << Ngede << endl;

// the tube chambers
      if(msl < 3) {
        int sl=msl-1;
// the special case of chamber 99
        if(iup == 1 && nx == 98) {
          nx=MDH.NchambT[sl][lay][iup]-1;         
        }

        if(nx > MDH.NchambT[sl][lay][iup]-1 || nx < 0) {
          arteMsg  << "MuDigiHitb: wrong tube ch. nx = " << nx << endl;
          arteMsg.warn();
	  return 0;
        }

        MDH.TubeGeoWire = (*iv)->nwi[0];  // number of wires
        strncpy(&MDH.ChamListT[sl][lay][iup][nx][0],name,9);
        MDH.TubeGede[sl][lay][iup][nx][iside]    = Ngede;
      }

// the pad  chambers
      if(msl > 2) {
        int sl=msl-3;
// the special case of chamber 99
        if(iup == 1 && nx == 98) {
          nx=MDH.NchambP[sl][iup]-1;         
        }

        if(nx > MDH.NchambP[sl][iup]-1 || nx < 0) {
          arteMsg << "MuDigiHitb: wrong pad ch. nx = " << nx << endl;
          arteMsg.warn();
	  return 0;
        }

        MDH.PadGeoWire = (*iv)->nwi[0];         // number of wires
        MDH.Npads[sl][iup][nx] = (*iv)->nwi[2]; // number of pads
        strncpy(&MDH.ChamListP[sl][iup][nx][0],name,9);
        MDH.PadGede[sl][iup][nx][iside]    = Ngede;
      }
    }
// pixel chambers
    if(cmp == 9) {
      ArtePointer<GESL> gesl = (*iv)->gesl;
      int msl = gesl->nsl;            // # muon superlayer
      int sl  = msl-1;

//      float xd1  = ArtePointer<GEDE>(iv)->xmin;  // Xmin of chamber
//      float xd2  = ArtePointer<GEDE>(iv)->xmax;  // Xmax of chamber
//      float yd1  = ArtePointer<GEDE>(iv)->ymin;  // Ymin of chamber
//      float yd2  = ArtePointer<GEDE>(iv)->ymax;  // Ymax of chamber
//      float zd1  = ArtePointer<GEDE>(iv)->zmin;  // Zmin of chamber
//      float zd2  = ArtePointer<GEDE>(iv)->zmax;  // Zmax of chamber

// number of rows
//      int Nrow = ArteTableIterator<GEDE>::nrow();
// chamber name
      char name[6] = "     ";
      strncpy(name,(*iv)->hwc,5);
// decode the chamber number
      int Nch = 0;
      char ch2 = name[4];
      for(int i=0; i<11; i++) {
        char ch1 = numb[i]; 
        if(ch1 == ch2) {
          Nch = i;
          break;
        }
      }
      int qu = Nch-1;

//  cout << " " << endl;
//  cout << name << " Nch " << Nch << " gede " << Ngede << " Nrow " << Nrow;
//  cout << " iv,msl: " << iv << " " << msl;
//  cout << " xd1,xd2 " << xd1 << " " << xd2 << endl;
//  cout << "yd1,yd2 " << yd1 << " " << yd2;
//  cout << " zd1,zd2 " << zd1 << " " << zd2 << endl;

      strncpy(&MDH.ChamListX[sl][qu][0],name,6);
      MDH.PixGede[sl][qu] = Ngede;
    }
  }
  MDH.Ivalid = 1;
  return 1;
}

// initiate the cable connection from the info.muonconfig file
int ConnectIni() 
{

  char line[150];
  //  FILE *conf = fopen("info.muonconfig", "r");
  FILE *conf = fopen(getenv("MUON_CONFIG"), "r");

  if(!conf) {
    arteMsg << "MuDigiHitb: NO muon configuration file" << endl;
    arteMsg.error();
    return 0;
  }
  int cratr=0;
  int cablr =0;
  Word crate=0;
  Word cable =0;
  char cablname[10];
  int icb=0;

  while (fgets(line, sizeof(line), conf) != NULL) {
    if (sscanf(line, "FED %x   CABLE %d   NAME %s " ,
               &cratr,&cablr,cablname) == 3) {

      crate = Word(cratr);
      cable = Word(cablr);

// check the hot channels
      Word status=MuHotListCheck(cablname);

//      if(status != 0xffff) {
//        cout << cablname << " status " << hex << status << dec << endl;
//      }

// cable bit position inside crate
      Word nbit = 16*cable; 

// decode cablname
      char numb[11] = "0123456789";
      char ch;

      char chi = cablname[0];
      char* chM = "M";
      if(chi == *chM) {

      chi = cablname[1];
      char* cht = "t";
      char* chp = "p";
      char* chx = "x";

      int sl=-1;
      ch = cablname[2];
      for(int i=0; i<11; i++) {
        char chn = numb[i]; 
        if(chn == ch) {
          sl = i;
          break;
        }
      }
      sl = sl-1;

      int type = 0;
      if(chi == *cht && sl >= 0 && sl < 2) type=1; // tube wires
      if(chi == *cht && sl > 1) type=2;            // pad wires
      if(chi == *chp && sl > 1) type=3;            // pads
      if(chi == *chx)           type=4;            // pixels

      int ind=0;
      ch = cablname[4];
      for(int i=0; i<11; i++) {
        char chn = numb[i]; 
        if(chn == ch) {
          ind = i;
          break;
        }
      }

      if(type < 4) {
        int lay = (ind-1)/2;
        int iup = ind-lay*2-1;

        int ncab=-1;
        ch = cablname[8];
        char* chA = "A";
        char* chB = "B";
        char* chC = "C";
        char* chD = "D";
        if(ch == *chA) ncab=0;
        if(ch == *chB) ncab=1;
        if(ch == *chC) ncab=2;
        if(ch == *chD) ncab=3;

        int nx1 = 0;
        ch = cablname[6];
        for(int i=0; i<11; i++) {
          char chn = numb[i]; 
          if(chn == ch) {
            nx1 = i;
            break;
          }
        }
        int nx2 = 0;
        ch = cablname[7];
        for(int i=0; i<11; i++) {
          char chn = numb[i]; 
          if(chn == ch) {
            nx2 = i;
            break;
          }
        }
        int nx = 10*nx1+nx2-1;
// the special case of chamber 99
        if(iup == 1 && nx == 98) {
          if(type == 1) nx=MDH.NchambT[sl][lay][iup]-1;         
          if(type > 1 && type < 4)  nx=MDH.NchambP[sl-2][iup]-1;         
        }

//          if(type == 2) {
//            cout << "  sl " << sl << " ind " << ind << " lay " << lay;
//            cout << " iup " << iup << " nx " << nx << " type " << type;
//            cout << " ncab " << ncab << endl; 
//          }

// tube wires
        if(type == 1) {
          if(nx > MDH.NchambT[sl][lay][iup]-1 || nx < 0) {
            arteMsg << "MuDigiHitb - tube wires: wrong nx = " << nx << endl;
            arteMsg.warn();
//            cout << "  sl " << sl << " ind " << ind << " lay " << lay;
//            cout << " iup " << iup << " nx " << nx << " type " << type;
//            cout << " ncab " << ncab << endl; 
//            cout << line << endl;
            return 0;
          }
          int nc = MDH.NconnT[sl][lay][iup][nx];
          MDH.NconnT[sl][lay][iup][nx] += 1;
          MDH.MaskT[sl][lay][iup][nx][nc] = status;
          if(iup == 0) {
            MDH.StepT[sl][lay][iup][nx] =-2;
            MDH.StepT[sl][lay][iup][nx] =-2;

            MDH.CrateT[sl][lay][iup][nx]       =crate;
            MDH.Nbit1T[sl][lay][iup][nx][nc][0]=nbit+1;
            MDH.Nbit1T[sl][lay][iup][nx][nc][1]=nbit;

            if(ncab == 0) {
              MDH.Wire1T[sl][lay][iup][nx][nc][0]=15;
              MDH.Wire1T[sl][lay][iup][nx][nc][1]=15;
            }
            if(ncab == 1) {
              MDH.Wire1T[sl][lay][iup][nx][nc][0]=7;
              MDH.Wire1T[sl][lay][iup][nx][nc][1]=7;
            }
          }
          if(iup == 1) {
            MDH.StepT[sl][lay][iup][nx] =2;
            MDH.StepT[sl][lay][iup][nx] =2;

            MDH.CrateT[sl][lay][iup][nx]       =crate;
            MDH.Nbit1T[sl][lay][iup][nx][nc][0]=nbit;
            MDH.Nbit1T[sl][lay][iup][nx][nc][1]=nbit+1;

            if(ncab == 0) {
              MDH.Wire1T[sl][lay][iup][nx][nc][0]=0;
              MDH.Wire1T[sl][lay][iup][nx][nc][1]=0;
            }
            if(ncab == 1) {
              MDH.Wire1T[sl][lay][iup][nx][nc][0]=8;
              MDH.Wire1T[sl][lay][iup][nx][nc][1]=8;
            }
          }
        }

// pad chamber wires
        if(type == 2) {
          Word slp = sl-2;
          if(nx > MDH.NchambP[slp][iup]-1 || nx < 0) {
             arteMsg << "MuDigiHitb-pad chamber wires: wrong nx = " << nx << endl;
            arteMsg.warn();
//            cout << "  sl " << sl << " ind " << ind << " lay " << lay;
//            cout << " iup " << iup << " nx " << nx << " type " << type;
//            cout << " ncab " << ncab << endl; 
//            cout << line << endl;
            return 0;
          }
          int nc = MDH.NconnP[slp][iup][nx];
          MDH.NconnP[slp][iup][nx] += 1;
          MDH.MaskP[slp][iup][nx][nc] = status;
          if(iup == 0) {
            MDH.StepP[slp][iup][nx]=-2;
            MDH.StepP[slp][iup][nx]=-2;

            MDH.CrateP[slp][iup][nx]       =crate;
            MDH.Nbit1P[slp][iup][nx][nc][0]=nbit+1;
            MDH.Nbit1P[slp][iup][nx][nc][1]=nbit;

            if(ncab == 0) {
              MDH.Wire1P[slp][iup][nx][nc][0]=15;
              MDH.Wire1P[slp][iup][nx][nc][1]=15;
            }
            if(ncab == 1) {
              MDH.Wire1P[slp][iup][nx][nc][0]=7;
              MDH.Wire1P[slp][iup][nx][nc][1]=7;
            }
          }
          if(iup == 1) {
            MDH.StepP[slp][iup][nx]=2;
            MDH.StepP[slp][iup][nx]=2;

            MDH.CrateP[slp][iup][nx]       =crate;
            MDH.Nbit1P[slp][iup][nx][nc][0]=nbit;
            MDH.Nbit1P[slp][iup][nx][nc][1]=nbit+1;

            if(ncab == 0) {
              MDH.Wire1P[slp][iup][nx][nc][0]=0;
              MDH.Wire1P[slp][iup][nx][nc][1]=0;
            }
            if(ncab == 1) {
              MDH.Wire1P[slp][iup][nx][nc][0]=8;
              MDH.Wire1P[slp][iup][nx][nc][1]=8;
            }
          }
        }

// pads
        if(type == 3) {
          Word slp = sl-2;
          if(nx > MDH.NchambP[slp][iup]-1 || nx < 0) {
            arteMsg << "MuDigiHitb-pad chamber pads: wrong nx = " << nx << endl;
            arteMsg.warn();
//            cout << "  sl " << sl << " ind " << ind << " lay " << lay;
//            cout << " iup " << iup << " nx " << nx << " type " << type;
//            cout << " ncab " << ncab << endl; 
//            cout << line << endl;
            return 0;
          }
          int nc = MDH.NconPP[slp][iup][nx];
          MDH.NconPP[slp][iup][nx] += 1;
          MDH.MaskPP[slp][iup][nx][nc] = status;
          MDH.CratePP[slp][iup][nx]    = crate;
          MDH.Nbit1PP[slp][iup][nx][nc]=nbit;
          MDH.NumbP[slp][iup][nx][nc]=15;
          if(ncab == 0 || ncab == 2) {
            if(slp == 1) {
              MDH.NumbP[slp][iup][nx][nc]=14;
              MDH.Nbit1PP[slp][iup][nx][nc]=nbit+1;
            }
          }
          Word padsh = 15;

// chambers Mp 15, 16, 17
          if(nx > 14 && nx < 18) {
            if(ncab == 1 || ncab == 3) {
              MDH.NumbP[slp][iup][nx][nc]=12;
              if(slp == 1) MDH.Nbit1PP[slp][iup][nx][nc]=nbit;
            }
            padsh=12;
          }
// chamber 18 bottom 
          if(iup == 1 && nx == 18) {
            if(ncab == 1 || ncab == 3) {
              MDH.Nbit1PP[slp][iup][nx][nc]=nbit;
              MDH.NumbP[slp][iup][nx][nc]=4;
              if(slp == 1) MDH.NumbP[slp][iup][nx][nc]=5;
            }
            if(ncab == 0 || ncab == 2) {
             MDH.Nbit1PP[slp][iup][nx][nc]=nbit+1;
            }
            padsh=4;
            if(slp == 1) padsh=5;
          }

          if(iup == 0) {
            if(ncab == 1) MDH.Pad1[slp][iup][nx][nc]=MDH.Npads[slp][iup][nx];
            if(ncab == 0) MDH.Pad1[slp][iup][nx][nc]=MDH.Npads[slp][iup][nx]+padsh;
            if(ncab == 3) MDH.Pad1[slp][iup][nx][nc]=0;
            if(ncab == 2) MDH.Pad1[slp][iup][nx][nc]=padsh;
          }
          if(iup == 1) {
            if(ncab == 1) MDH.Pad1[slp][iup][nx][nc]=0;
            if(ncab == 0) MDH.Pad1[slp][iup][nx][nc]=padsh;
            if(ncab == 3) MDH.Pad1[slp][iup][nx][nc]=MDH.Npads[slp][iup][nx];
            if(ncab == 2) MDH.Pad1[slp][iup][nx][nc]=MDH.Npads[slp][iup][nx]+padsh;
// add the link to chamber 99
            if(nx == 18) {
              int nx99=int(MDH.NchambP[slp][iup])-1;         
              if(ncab == 1 || ncab == 3) {
                int nc99 = int(MDH.NconPP[slp][iup][nx99]);
                MDH.NconPP[slp][iup][nx99] += 1;
                MDH.MaskPP[slp][iup][nx99][nc99] = status;
                MDH.CratePP[slp][iup][nx99]    =crate;
                MDH.Nbit1PP[slp][iup][nx99][nc99]=nbit+9;
                MDH.NumbP[slp][iup][nx99][nc99]=MDH.Npads[slp][iup][nx99];
                if(ncab == 1) MDH.Pad1[slp][iup][nx99][nc99]=0;
                if(ncab == 3) MDH.Pad1[slp][iup][nx99][nc99]=MDH.Npads[slp][iup][nx99];
              }
            }
          }
        }
      }
// pixels
      if(type == 4) {
// pixel chamber number
        int nch = ind;
        if(nch < 1 || nch > MDH.PixGeoQu) {
	  arteMsg << "MuDigiHitb-pix: wrong chamber number = " << dec << nch << endl;
          arteMsg.warn();
          return 0;
        }
// quadrant number
        int qu = nch-1;
     
// readout connector number
        int nc1 = 0;
        ch = cablname[6];
        for(int i=0; i<11; i++) {
          char chn = numb[i]; 
          if(chn == ch) {
            nc1 = i;
            break;
          }
        }
        int nc2 = 0;
        ch = cablname[7];
        for(int i=0; i<11; i++) {
          char chn = numb[i]; 
          if(chn == ch) {
            nc2 = i;
            break;
          }
        }
        int nc = 10*nc1+nc2-1;
        if(nc < 0 || nc > MDH.PixNcon-1) {
	  arteMsg << "MuDigiHitb-pix: wrong connector number = " << dec << nc << endl;
          arteMsg.warn();
          return 0;
        }

// code the connection status into 16-bit word
        Word stcon = 0;
        Word one    = 0x1;
        for(int i=0; i<16; i++) {
          if(MDH.Pnx1[sl][nc][i] > 0 &&
             MDH.Pny1[sl][nc][i] > 0) stcon = stcon | (one << i);
        }

// swapping
        Word nbb=nbit/16;
        Word nba=nbit-16*nbb;
        if(nbb%2 == 0) nbb += 1;
        else nbb -= 1;
        nbit = 16*nbb+nba;

        MDH.CrateX[sl][qu][nc]= crate;
        MDH.Nbit1X[sl][qu][nc]= nbit;
        MDH.MaskX[sl][qu][nc] = status & stcon;
      }
      }
      icb++;
    }
  }
  fclose(conf);
  return 1;
}

//*** loops over tube chambers
void TubeLoops() 
{
  if(MDH.Ivalid != 1) {
    arteMsg << "MuDigiHitb.TubeLoops: Ivalid = " << MDH.Ivalid << endl;
    arteMsg.error();
    return;
  }

  for(Word sl=0; sl<MDH.TubeGeoSl; sl++) {
    for(Word lay=0; lay<MDH.TubeGeoLay; lay++) {
      for(Word ud=0; ud<MDH.TubeGeoUpDown; ud++) {
        if(MDH.NchambT[sl][lay][ud] > 0) {
          for(Word sd=0; sd<MDH.TubeGeoSd; sd++) {
            for(Word nx=0; nx<MDH.NchambT[sl][lay][ud]; nx++) {
              Word ncon = MDH.NconnT[sl][lay][ud][nx];
              if(ncon > 0) {
                for(Word nc=0; nc<ncon; nc++) {
                  Word crate = MDH.CrateT[sl][lay][ud][nx];
                  Word nbit1 = MDH.Nbit1T[sl][lay][ud][nx][nc][sd];
                  Word wir1   = MDH.Wire1T[sl][lay][ud][nx][nc][sd];
                  int  step   = MDH.StepT[sl][lay][ud][nx];
                  int astp   = (step<0)? -step : step;
                  int gede = int(MDH.TubeGede[sl][lay][ud][nx][sd]);
                  for(Word wr=0; wr<MDH.NumbT; wr++) {
                    int wir=0;
                    if(step > 0) wir = int(wir1+wr);
                    if(step < 0) wir = int(wir1-wr);
                    LWord nbit = nbit1 + astp*wr;
// swapping
                    LWord nwd=nbit/32;
                    LWord nbt = nbit - 32*nwd;
                    nbit = 32*nwd + 31-nbt;
// check mask
                    Word mskb = astp*wr;
                    if(MDH.Nbit1T[sl][lay][ud][nx][nc][sd] > 
                       MDH.Nbit1T[sl][lay][ud][nx][nc][1-sd]) mskb += 1;
                    Word msk = 0;
                    Word one = 0x8000;
                    msk = msk + (one >> mskb);
                    Word mask = MDH.MaskT[sl][lay][ud][nx][nc] & msk;
                    if(mask != 0) {
// check DMUT
                      int check = CheckBit(1,crate,nbit);
                      if(check == 1) {
// add the tube wire into HITB
                        AddTube(gede,wir);
                      }
                    }
                  }
                }
              }
            }
          }
        }
      }
    }
  }
}

// loops over pad chambers
void PadLoops()
{
// create the log-file and pipe the programm output
  //  ofstream out("MuDigiHitb.loops");
  //  if (!out) 
  //  {
  //    cout << "MuDigiHitb - cannot open file for print" << endl;
  //    return;
  //  }
  //  cout = out;
  //  cerr = out;

  if(MDH.Ivalid != 1) {
    arteMsg << "PadWireLoops: Ivalid = " << MDH.Ivalid << endl;
    arteMsg.error();
    return;
  }

  for(Word sl=0; sl<MDH.PadGeoSl; sl++) {
    for(Word udt=0; udt<MDH.PadGeoUpDown; udt++) {
      Word ud = MDH.PadGeoUpDown-1-udt;

// loops over wire connectors of side 0
      LoopsPW(sl,ud,0);

// loops over pad connectors
      if(MDH.NchambP[sl][ud] > 0) {
        for(Word nx=0; nx<MDH.NchambP[sl][ud]; nx++) {
          Word ncon = MDH.NconPP[sl][ud][nx];
          if(ncon > 0) {
            Word crate= MDH.CratePP[sl][ud][nx];
            int gede1 = int(MDH.PadGede[sl][ud][nx][0]);
            int gede2 = int(MDH.PadGede[sl][ud][nx][1]);
            for(Word nc=0; nc<ncon; nc++) {
              Word nbit1 = MDH.Nbit1PP[sl][ud][nx][nc];
              Word pad1   = MDH.Pad1[sl][ud][nx][nc];
              int  npd    = MDH.NumbP[sl][ud][nx][nc];
              if(npd > 0) {
                for(Word pd=0; pd<npd; pd++) {
                  int pad = int(pad1+npd-pd-1);
                  LWord nbit = nbit1+pd;
// check mask
                  Word one = 0x8000;
                  Word msk = 0;
                  if(npd >= 14 || npd < 12) msk = one >> pd+15-npd;
                  if(npd == 12) msk = one >> pd;
                  Word mask = MDH.MaskPP[sl][ud][nx][nc] & msk;
                  if(mask != 0) {
// check DMUT
                    int check = CheckBit(2,crate,nbit);
                    if(check == 1) {
// add the pad into HITB
                        AddPad(gede1,pad);
                        AddPad(gede2,pad);
                    }
                  }
                }
              }
            }
          }
        }
      }

// loops over wire connectors of side 1
      LoopsPW(sl,ud,1);

    }
  }
}


// loops over pad chamber wire connectors of side sd
void LoopsPW(Word sl,Word ud,Word sd) 
{

  if(MDH.NchambP[sl][ud] > 0) {
    for(Word nx=0; nx<MDH.NchambP[sl][ud]; nx++) {
      Word ncon = MDH.NconnP[sl][ud][nx];
      if(ncon > 0) {
        Word crate = MDH.CrateP[sl][ud][nx];
        int gede = int(MDH.PadGede[sl][ud][nx][sd]);
        int  step   = MDH.StepP[sl][ud][nx];
        Word astp   = (step<0)? -step : step;
        for(Word nc=0; nc<ncon; nc++) {
          Word nbit1 = MDH.Nbit1P[sl][ud][nx][nc][sd];
          Word wir1   = MDH.Wire1P[sl][ud][nx][nc][sd];
          for(Word wr=0; wr<MDH.NumbT; wr++) {
            int wir=0;
            if(step > 0) wir = int(wir1+wr);
            if(step < 0) wir = int(wir1-wr);
//            wir=15-wir;
            LWord nbit = nbit1 + astp*wr;
// swap the bytes
            LWord nwd=nbit/32;
            LWord nbt = nbit - 32*nwd;
            nbit = 32*nwd + 31-nbt;
// check mask
            Word mskb = astp*wr;
            if(MDH.Nbit1P[sl][ud][nx][nc][sd] > 
              MDH.Nbit1P[sl][ud][nx][nc][1-sd]) mskb += 1;
            Word msk = 0;
            Word one = 0x8000;
            msk = msk + (one >> mskb);
            Word mask = MDH.MaskP[sl][ud][nx][nc] & msk;
            if(mask != 0) {
// check DMUT
              int check = CheckBit(1,crate,nbit);
              if(check == 1) {
// add the pad wire into HITB
                AddTube(gede,wir);
              }
            }
          }
        }
      }
    }
  }
}

//*** loops over pixel chambers
void PixLoops() 
{
  if(MDH.Ivalid != 1) {
    arteMsg << "MuDigiHitb.PixLoops: Ivalid = " << MDH.Ivalid << endl;
    arteMsg.error();
    return;
  }

  Word one = 0x8000;
  for(Word sl=0; sl<MDH.PixGeoSl; sl++) {
    for(Word qu=0; qu<MDH.PixGeoQu; qu++) {
//      int gede = int(MDH.PixGede[sl][qu]);
      for(Word nc=0; nc<MDH.PixNcon; nc++) {
        if(MDH.MaskX[sl][qu][nc] > 0) {
          Word crate = MDH.CrateX[sl][qu][nc];
          LWord  nbit1 = LWord(abs(MDH.Nbit1X[sl][qu][nc]));
          int step=1;
          if(MDH.Nbit1X[sl][qu][nc] < 0) step=-1;
          for(Word bt=0; bt<16; bt++) {
            LWord nbit = 0;
            if(step == 1)  nbit = nbit1 + bt;
            if(step == -1) nbit = nbit1 - bt;
// check mask
            Word msk = one >> bt;
            Word mask = MDH.MaskX[sl][qu][nc] & msk;
            if(mask != 0) {
// check DMUX
              int check = CheckBit(3,crate,nbit);
              if(check == 1) {
                MDH.PixSlAdd = sl;
                MDH.PixQuAdd = qu;
                MDH.PixConAdd= nc;
                MDH.PixBitAdd= 15-bt;
                AddPix();

		//     char nam[6];
		//     strncpy(nam,&MDH.ChamListX[sl][qu][0],5);
		//     cout << "pixel of " << nam;
		//     cout << " added in HITB, sl " << dec << sl << " qu " << qu;
		//     cout << " nx " << MDH.Pnx1[sl][nc][bt] << " ny " << MDH.Pny1[sl][nc][bt];
		//     cout << " gede " << dec << gede << " nc " << nc;
		//     cout << " crate " << hex << crate << dec << " nbit " << nbit;
		//     Word ww = nbit / 32 ;
		//     Word bb  = nbit - ww*32 ;
		//     cout << " (" << ww << "-" << bb << ") " << endl;

              }
            }
          }
        }
      }
    }
  }
}

// check the bit nbit in DMUT(table=1), DMUP(table=2), DMUX(table=3)
int CheckBit(Word table,Word crate,LWord nbit) 
{
  int* len   = 0;
  if(table == 1) len=(int*)(ArtePointer<DMUT>(0)->bits);
  if(table == 2) len=(int*)(ArtePointer<DMUP>(0)->bits);
  if(table == 3) len=(int*)(ArtePointer<DMUX>(0)->bits);
  if(*len <= 0) return 0;
  LWord nmax = LWord(*len);

// crate offset
  LWord nword = 0;
  LWord ncrate = nmax/34;
  LWord* Lbuf = 0;
  LWord word = 0;
  LWord first=0;
// loops over crate headers
  for (LWord ncr=0; ncr<ncrate; ncr++) {
    first=2+34*ncr;
    if(table == 1) Lbuf=(LWord*)(ArtePointer<DMUT>(first)->bits);
    if(table == 2) Lbuf=(LWord*)(ArtePointer<DMUP>(first)->bits);
    if(table == 3) Lbuf=(LWord*)(ArtePointer<DMUX>(first)->bits);
// not change the bit order for Big endian   
    if(MDH.Endian == 0) word = *Lbuf;
// change the bit order for Little endian
    else word = ESP_SWAP_INT(*Lbuf);
    LWord tmp1 = word << 16;
    Word  tmp2=Word(tmp1 >> 16);
    if(tmp2 == crate) {
      nword=first+nbit/32+1;
      break;
    }
  }
  if(nword == 0) {
    //  arteMsg << "CheckBit: no crate " << hex << crate << " in table " << table << endl;
    //    arteMsg.error();
    //  arteMsg.warn();
    return 0;
  }
  if(nword > nmax) {
    //    arteMsg << "CheckBit: nword= " << nword << " nmax= " << nmax << " table " << table  << endl;
    //    arteMsg.error();
    //    arteMsg.warn();
    return 0;
  } 

// take the word
  if(table == 1) Lbuf=(LWord*)(ArtePointer<DMUT>(nword+1)->bits);
  if(table == 2) Lbuf=(LWord*)(ArtePointer<DMUP>(nword+1)->bits);
  if(table == 3) Lbuf=(LWord*)(ArtePointer<DMUX>(nword+1)->bits);
// not change the bit order for Big endian   
  if(MDH.Endian == 0) word = *Lbuf;
// change the bit order for Little endian
  else word = ESP_SWAP_INT(*Lbuf);
// check the bit
  if(word == 0) return 0;
  Word nw = Word(nbit/32);
  Word nbtt = nbit-32*nw;
  Word nbyte = nbtt/8;
  Word nbb = nbtt-8*nbyte;
  Word nb = 7-nbb;  // change the order of bits inside byte
  Word nbt = 8*nbyte+nb;
  const LWord one = 0x1;
  LWord mask = one << nbt;
  LWord res = word & mask;
  //  cout << "crate " << hex << crate << " nword " << dec << nword << " nbit " << nbit;
  //  cout << " word " << hex << *Lbuf << "  " << word << " res " << res << endl;
  if(res == 0) return 0;
  return 1;
}

// initialization of the pointers to hits 
void MuPHini() 
{
// loops over GESL
  for ( ArteTableIterator<GESL> isl = ArteTable<GESL>::begin(); 
        isl != ArteTable<GESL>::end();
        ++isl ) {
    int   cmp  = (*isl)->cmp;
    if(cmp == 8 || cmp == 9) {
      int iv1 = (*isl)->fpl;
      int iv2 = iv1 + (*isl)->npl - 1;
      if(iv2 >= iv1) {
// loops over muon gede
        for ( int iv=iv1; iv<iv2; iv++) { 
          ArtePointer<GEDE> gede(iv);
          gede->hit1 = 0;
          gede->hitl = -1;
        }
      }
    }
  }
}

// add tube wire to HITB
void AddTube(int iv, int iw)
{
  ArtePointer<GEDE> gede(iv);
  float dist = gede->pit;    // distance between wires
  float xfw  = gede->xfw[0]; // X-position of first wire
  float xd1  = gede->xmin;   // Xmin of chamber
  float x = xd1+xfw+dist*float(iw);  // x-position of wire
  float z = gede->zfw[0];    // z-position of wire

// add the hit to HITB
  struct HITB hitb;
  hitb.x    = x;
  hitb.y    = 0;
  hitb.z    = z;
  hitb.q    = 0;
  hitb.r    = 0;
  hitb.gede = iv;   
  ArtePointer<HITB>::append(hitb);

// append the pointers from GEDE to HITB
  int nhitb = ArtePointer<HITB>::nrow();
  if(*gede->hit1 == 0) gede->hit1=nhitb;
  gede->hitl=nhitb;

  return;  
}

// add pad to HITB
void AddPad(int iv, int pad)
{
  ArtePointer<GEDE> gede(iv);
  int npmx = gede->nwi[2];   // number of pads in row
  if(pad < 0 || pad > 2*npmx-1) return;
  int nr = pad/npmx;         // raw number
  int np = pad-nr*npmx;      // pad number in the raw

  float xd1  = gede->xmin;   // Xmin of chamber
  float xd2  = gede->xmax;   // Xmax of chamber
  float yd1  = gede->ymin;   // Ymin of chamber
  float yd2  = gede->ymax;   // Ymax of chamber

// up - down 
  int ud = 0;
  if(fabsf(yd1) > fabsf(yd2)) ud=1;

  float yl2=0.01*float(gede->nwi[3]); // y-dist between pads
  float dxpad=gede->xfw[1];           // x-size of pad
  float dypd1=gede->xfw[2];           // y-size of first pad
  float dypd2=gede->xfw[3];           // y-size of standard pad
  float zpad =gede->zfw[1];           // z-position of pad
  float alsx =gede->sxmx - gede->sxmn;
  float xl1  =(xd2-xd1-alsx)/2.;      // x-dist from edge to pad
  //  float xl2  =alsx - 2.*dxpad;        // x-dist between pads
  float alsy =gede->symx - gede->symn;
  float yl1  =(yd2-yd1-alsy)/2.;       // y-dist from edge to pad
// y-size of last pad
  float dypd3=alsy-dypd1-yl2-(dypd2+yl2)*float(npmx-2);

// the pad position and size
  float xpad = 0.;
  if(nr == 0) xpad=xd1+xl1+dxpad*0.5;
  if(nr == 1) xpad=xd2-xl1-dxpad*0.5;
  float ypad = 0.;
  float dypad= 0.;
  if(np == 0) {
    if(ud == 0) ypad=yd1+yl1+0.5*dypd1;
    if(ud == 1) ypad=yd2-yl1-0.5*dypd1;
    dypad=dypd1;
  }
  if(np > 0 && np < npmx-1) {
    if(ud == 0) ypad=yd1+yl1+dypd1+yl2+(dypd2+yl2)*float(np-1)+0.5*dypd2;
    if(ud == 1) ypad=yd2-yl1-dypd1-yl2-(dypd2+yl2)*float(np-1)-0.5*dypd2;
    dypad=dypd2;
  }
  if(np == npmx-1) {
    if(ud == 0) ypad=yd2-yl1-0.5*dypd3;
    if(ud == 1) ypad=yd1+yl1+0.5*dypd3;
    dypad=dypd3;
  }

// add the hit to HITB
  struct HITB hitb;
  hitb.x    = xpad;
  hitb.y    = ypad;
  hitb.z    = zpad;
  hitb.q    = dxpad;
  hitb.r    = dypad;
  hitb.gede = iv;   
  ArtePointer<HITB>::append(hitb);

// append the pointers from GEDE to HITB
  int nhitb = ArtePointer<HITB>::nrow();
  if(*gede->hit1 == 0) gede->hit1=nhitb;
  gede->hitl=nhitb;

  return;  
}

// add pixel to HITB
void AddPix() 
{
  Word sl  = MDH.PixSlAdd;
  Word qu  = MDH.PixQuAdd;
  Word con = MDH.PixConAdd;
  Word bit = MDH.PixBitAdd;

  int  iv  = int(MDH.PixGede[sl][qu]);
  int  nx  = int(MDH.Pnx1[sl][con][bit]-1);
  int  nxc = int(MDH.PixGeoNxC[sl]);
  if(nxc < 1) return;
  int  ny = int(MDH.Pny1[sl][con][bit]-1);
  int  nyc = int(MDH.PixGeoNyC[sl]);
  if(nyc < 1) return;

  ArtePointer<GEDE> gede(iv);
  float xd1  = gede->xmin;   // Xmin of chamber
  float xd2  = gede->xmax;   // Xmax of chamber
  float yd1  = gede->ymin;   // Ymin of chamber
  float yd2  = gede->ymax;   // Ymax of chamber
  float zd1  = gede->zmin;   // Zmin of chamber
  float zd2  = gede->zmax;   // Zmax of chamber
  float xed  = gede->xfw[0]; // x-dist from edge to pixels
  float yed  = gede->xfw[1]; // y-dist from edge to pixels
  float xsz  = gede->xfw[2]; // x size of elemetary pixel
  float ysz  = gede->xfw[3]; // y size of elemetary pixel

// the relative hit position and size
  float dx = xsz*float(nxc);
  float x = 0.;
  if(sl > 1) x = xed+dx*(float(nx)-0.5);
  if(sl < 2 && nx == 1) {
    dx=xsz;  
     x=xed+dx*0.5;
  }
  if(sl < 2 && nx > 1) x=xed+xsz+dx*(float(nx)-1.5);

  float dy = ysz*float(nyc);
  float y = 0.;
  if(sl != 2) y = yed+dy*(float(ny)-0.5);
  if(sl == 2 && ny == 1) {
    dy=1.5*dy;  
     y=yed+dy*0.5;
  }
  if(sl == 2 && ny > 1) y=yed+dy*float(ny);

// hit coordinates
  if(qu == 0 || qu == 3) x=xd1+x; 
  else                   x=xd2-x; 
  if(qu == 0 || qu == 2) y=yd1+y; 
  else                   y=yd2-y; 
  float z = 0.5*(zd1+zd2);

// add the hit to HITB
  struct HITB hitb;
  hitb.x    = x;
  hitb.y    = y;
  hitb.z    = z;
  hitb.q    = dx;
  hitb.r    = dy;
  hitb.gede = iv;   
  ArtePointer<HITB>::append(hitb);

// append the pointers from GEDE to HITB
  int nhitb = ArtePointer<HITB>::nrow();
  if(*gede->hit1 == 0) gede->hit1=nhitb;
  gede->hitl=nhitb;

  return;  
}
}

