#ifndef __POIROT_SUPERCLUSTER_HH__
#define __POIROT_SUPERCLUSTER_HH__

#ifndef __POIROT_SECTORCLUSTER_HH__
#include "SectorCluster.hh"
#endif
#ifndef __POIROT_RECONSTRUCTEDTRACKINTERFACE_HH__
#include "ReconstructedTrackInterface.hh"
#endif
#ifndef __SGI_STL_VECTOR
#include <vector>
#endif

class TPolyMarker3D ;
class poirotSectorCluster ;

class poirotCluster : public poirotReconstructedTrackInterface,
			private poirotTrackDescription2D
{
public:
  // typedefs, enumerates
  typedef vector<poirotSectorCluster*> ClusterContainer ;
  
  // constructors, destructor
  static poirotCluster* Create(poirotSectorCluster* plusClus, 
			    poirotSectorCluster* minusClus) ;
  //poirotCluster(poirotSectorCluster* plusClus, poirotSectorCluster* minusClus) ;
  poirotCluster() ;
  poirotCluster(const poirotCluster&) ;
  virtual ~poirotCluster() ;

  // setters and getters
  const poirotSectorCluster* GetPlusCluster() const  { return fSectorClusters[0] ; }
  const poirotSectorCluster* GetZeroCluster() const  { return fSectorClusters[1] ; }
  const poirotSectorCluster* GetMinusCluster() const { return fSectorClusters[2] ;}
  const ClusterContainer& GetSectorClusters() const { return fSectorClusters ; } ;

  float GetXDif() const { return fXDif ; }
  virtual float GetMinChi2Probability() const { return fgMinChi2Probability ; }
  virtual void SetMinChi2Probability(float cp) { fgMinChi2Probability=cp ; }
  using poirotTrackDescription2D::GetX ;  
  using poirotTrackDescription2D::GetY ;  
  using poirotTrackDescription2D::GetZ ;  
  using poirotTrackDescription2D::GetTx ;  
  using poirotTrackDescription2D::GetTy ;
  using poirotTrackDescription2D::GetCov ;  
  using poirotTrackDescription2D::GetQ ;

  // other inlines
  float GetCentralZ() const { return GetZeroCluster()->GetRawZ() ; } ;
  float GetCentralX() const { return GetTx()*(GetCentralZ()-GetZ())+GetX(); } 
  float GetCentralY() const { return GetTy()*(GetCentralZ()-GetZ())+GetY(); } 
  float GetXDif(const poirotSectorCluster* zeroClus) const { return zeroClus->GetXDif(GetTx()) ; }
  bool Valid() const ;

  // other methods
  bool SetZeroIfValid(poirotSectorCluster* zeroClus) ;
  float CalcXDif() ;
  bool Merge(poirotCluster& rhs) ;
  void PerformHistogramming() const ;
  void Dump() const ;
  bool CreateReconstructedHits() ;
  bool MixedPitch() const ;
#ifdef __POIROT_USE_ROOT__
  void Draw(int color) ;
#endif 
private:
  inline bool OneFitIteration(int fitMode=0) ;
  void SetPlusCluster(poirotSectorCluster* sc)  { fSectorClusters[0]=sc ; }
  void SetZeroCluster(poirotSectorCluster* sc)  { fSectorClusters[1]=sc ; }
  void SetMinusCluster(poirotSectorCluster* sc) { fSectorClusters[2]=sc ;}
  bool ValidNHits(const poirotReconstructedHit* withoutThisOne) const ;
  float fXDif ;
  HitContainer fBadHits ;
  ClusterContainer fSectorClusters ;
  static float fgMinChi2Probability ;
};

inline bool
poirotCluster::Valid() const { 
  return fSectorClusters[0] && fSectorClusters[1] && fSectorClusters[2]; 
} 

#endif








