#ifndef __POIROT_NODEPROJECTION_HH__
#define __POIROT_NODEPROJECTION_HH__

#ifndef _Vector_H_
#include "CLHEP/Matrix/Vector.h"
#endif
#ifndef _SYMMatrix_H_
#include "CLHEP/Matrix/SymMatrix.h"
#endif

class poirotHit ;

class poirotReconstructedHit ;
class poirotTrackDescription ;

class poirotNodeProjection
{
public:
  poirotNodeProjection(poirotReconstructedHit*) ;
  poirotNodeProjection(const poirotHit*) ;
  //poirotNodeProjection(const poirotWire*) ;
  virtual ~poirotNodeProjection() ;
  inline void Init(const poirotHit*) ;
  inline void InitGeometry(const poirotHit*) ;
  void Project(const poirotTrackDescription*, bool updateInternal=false, bool calcDerivatives=true);
  double GetRm() const { return fRm ; } ;
  double GetD() const { return fD ; } ;
  double GetSigmaR2() const { return fSigmaR2 ; } ;
  double GetResidualR() const { return fResidualR ; } ;
  double GetResidualD() const { return fResidualD ; } ;
  double GetChi2() const { return fChi2 ; } ;
  const HepVector GetNormdResdQ() const { return fdDdQ/sqrt(fabs(fSigmaR2)) ; } ;
  const HepVector& GetdChi2dQ() const { return fdChi2dQ ; } ;
  const HepSymMatrix Getd2Chi2dQ2() const { return fd2Chi2dQ2 ; } ;
  bool GetEtaError() const { return fEtaError; } ;
  void Dump() const ;
  const poirotReconstructedHit* GetReconstructedHit() const { return fReconstructedHit ; } ;
private:
  // These have to be initialised only once
  poirotReconstructedHit* fReconstructedHit ; // Pointer to mother hit
  float fCosTheta, fSinTheta, fZw, fXw ; 
  float fRm ;
  // These depend on projection
  ///
  HepDouble fD ;
  HepVector fdDdQ ;
  ///
  HepDouble fResidualR ;
  HepDouble fResidualD ;
  ///
  float fChi2 ;
  bool fEtaError ;
  ///
  HepVector fProjVector ;
  HepVector fdChi2dQ ; 
  HepSymMatrix fd2Chi2dQ2 ;
  ///
  float fSigmaR2 ;
  float fSigmaC2 ;
};

#endif
