#ifndef __POIROT_RECO_HH__
#define __POIROT_RECO_HH__

#ifndef __STRING__
#include <string>
#endif
#ifndef __SGI_STL_VECTOR
#include <vector>
#endif
#ifndef __SGI_STL_MAP
#include <map>
#endif
#ifndef __POIROT_SUPERLAYERRECO_HH__
#include "poirot/SuperLayerReco.hh"
#endif

class poirotTrackCandidate ;
class poirotSuperLayerReco ;

class poirotReco
{
public:
  // typedefs
  typedef vector<poirotTrackCandidate*> TrackContainer ;
  typedef map<string, poirotSuperLayerReco> SuperLayerRecoContainer ;

  // constructors, destructor, assignment
  static poirotReco* Create() ;
  static void Delete() ;
  virtual ~poirotReco() ;

  // setters and getters
  static poirotReco* GetMe() { return fgMe ; }
  const TrackContainer& GetTrackCandidates() const { return fTrackCandidates ; }
  const SuperLayerRecoContainer& GetSuperLayerRecos() const { return fSuperLayerRecos ; }
  SuperLayerRecoContainer& GetSuperLayerRecos() { return fSuperLayerRecos ; }
  const poirotSuperLayerReco* GetSuperLayerReco(const string& name) const ;
  

  // other methods
  void ProcessOneEvent() ;
  void DumpClusters(ostream& os=cout) const ;

  void AppendRSEG(); // append all track candidates to the RSEG bank
  void AppendClustersToRSEG() ;
  static void SetAppendClustersToRSEG(bool on=true) { fgAppendClustersToRSEG=on ; } ;
  void DumpHits(ostream&) const ;
  int GetNClusters() const ;

private:
  poirotReco() ;
  void Reset() ;                // Delete hits, tracks, etc
  void SortHits() ;
  int SearchClusters() ;        // search clusters
  int SearchTracks() ;          // search tracks
  TrackContainer fTrackCandidates ; // Tracks
  SuperLayerRecoContainer fSuperLayerRecos ; // Superlayer events
  static bool fgAppendClustersToRSEG ;
  static poirotReco* fgMe ;
} ;


/******************* inlines **********************/
inline const poirotSuperLayerReco*
poirotReco::GetSuperLayerReco(const string& name) const {
  const SuperLayerRecoContainer::const_iterator superLayerReco = 
    fSuperLayerRecos.find(name) ;
  if(superLayerReco==fSuperLayerRecos.end()) return 0;
  return &((*superLayerReco).second) ;
}

#endif
