#ifndef __POIROT_RECONSTRUCTEDHIT_HH__
#define __POIROT_RECONSTRUCTEDHIT_HH__

#ifndef __ARTE_HH__
#include "arte/arte.hh"
#endif

class poirotHit ;
class poirotTrackDescription ;
class poirotNodeProjection ;

class poirotReconstructedHit
{
public:
  // typedefs, enumerates
  typedef ArtePointer<RHIT> ArtePointerRHIT ;
  typedef signed char LRSign_t ; // The left-right sign

  // constructors, destructor, assignment
  poirotReconstructedHit(const poirotReconstructedHit& rhs) ;
  poirotReconstructedHit() ;
  poirotReconstructedHit(const poirotHit* hit, LRSign_t lrSign=0) ;
  const poirotReconstructedHit& operator=(const poirotReconstructedHit& rhs) ;
  virtual ~poirotReconstructedHit() ;

  // setters and getters
  LRSign_t GetLRSign() const { return fLRSign ; }
  bool GetLRSignIsFixed() const { return fLRSignIsFixed; }
  const poirotHit* GetMeasuredHit() const { return fHit ; }

  // other inlines
  void FixLRSign() { fLRSignIsFixed=true ; }
  void ReleaseLRSign() { fLRSignIsFixed=false; }
  void FlipLRSign() { fLRSign *= -1 ; }
  inline void SetLRSign(double s) ;
  inline void SetLRSign(LRSign_t s) ;
  inline poirotNodeProjection* GetNodeProjection() ;
  inline void DeleteNodeProjection() ;
  inline double GetChi2() const ;

  // other methods
  bool UpdateInternal() ;
  ArtePointerRHIT AppendRHIT(const poirotTrackDescription*) ;
  ArtePointerRHIT GetRHIT() const { return fRHIT ; } ;

private:
  const poirotHit* fHit ;  // Pointer to corresponding poirotHit
  LRSign_t fLRSign ;    // Left-right sign (ambiguity)
  bool fLRSignIsFixed ; // Status flag for left-right sign
  poirotNodeProjection* fProjection ; //! Corresponding projection to track
  ArtePointerRHIT fRHIT ; // ArtePointer to RHIT bank
} ;

inline void poirotReconstructedHit::SetLRSign(double s) { 
  if(!GetLRSignIsFixed()) fLRSign  = (s>0) ? 1 : -1; 
}

inline void poirotReconstructedHit::SetLRSign(LRSign_t s) { 
  if(!GetLRSignIsFixed()) {
    if(s<0) fLRSign = -1 ;
    else if(s>0) fLRSign = 1;
    else fLRSign =0 ;
    //fLRSign  = (s>0) ? 1 :-1;
    if(abs(s)>=2) FixLRSign() ;
  }
} 

#ifndef __POIROT_NODEPROJECTION_HH__
#include "NodeProjection.hh"
#endif

inline poirotNodeProjection* 
poirotReconstructedHit::GetNodeProjection()
{
  if(!fProjection) {
    fProjection = new poirotNodeProjection(this) ;
  };
  return fProjection ;
}

inline void 
poirotReconstructedHit::DeleteNodeProjection()
{
  if(fProjection) {
    delete fProjection ;
    fProjection=0;
  };
}

inline double
poirotReconstructedHit::GetChi2() const { return fProjection->GetChi2() ; } ;
#endif
