#ifndef __POIROT_RECONSTRUCTEDTRACKINTERFACE_HH__
#define __POIROT_RECONSTRUCTEDTRACKINTERFACE_HH__

#ifndef __SGI_STL_VECTOR
#include <vector>
#endif
#ifndef __POIROT_TRACKDESCRIPTION_HH__
#include "TrackDescription.hh"
#endif

#ifndef __ARTE_HH__
#include "arte/arte.hh"
#endif

class poirotReconstructedHit ;
class poirotHit ;

extern "C" float prob_(const float&, const int&) ;
extern "C" float chisin_(const float&, const int&) ;

class poirotReconstructedTrackInterface 
  : public virtual poirotTrackDescription
{
public:
  // typedefs, enumerates
  typedef ArtePointer<RSEG> ArtePointerRSEG ;
  //
  poirotReconstructedTrackInterface() ;
  poirotReconstructedTrackInterface(const poirotReconstructedTrackInterface&) ;
  virtual ~poirotReconstructedTrackInterface() ; 
  poirotReconstructedTrackInterface& operator=(const poirotReconstructedTrackInterface&) ;
  ///
  typedef vector<poirotReconstructedHit*> HitContainer ;
  typedef HitContainer::iterator HitIterator ;
  virtual HitIterator begin() { return fHits.begin() ; } ;
  virtual HitIterator end() { return fHits.end() ; } ;
  //
  virtual const HitContainer& GetHits() const { return fHits ; } ;
  virtual HitContainer& GetHits() { return fHits ; } ;
  virtual void EraseHits() ;
  //static float GetMinChi2Probability() { return fgMinChi2Probability ; }
  //static void  SetMinChi2Probability(float x) { fgMinChi2Probability=x ; }
  virtual float GetMinChi2Probability() const = 0 ;
  virtual float GetChi2() const { return fChi2 ; }
  virtual void  SetChi2(float chi2) { fChi2 = chi2 ; }
  virtual int   GetError() const { return fError ; }
  virtual void  SetError(int error) { fError = error ; }
  virtual ArtePointerRSEG GetRSEG() const { return fRSEG ; }
  virtual int   GetNDof() const { return fHits.size()-GetDim() ; }
  virtual float GetChi2OverNDof() const { return fChi2/GetNDof() ; }
  virtual float GetChi2Prob() const { return prob_(GetChi2(),GetNDof()) ; }
  virtual bool ValidChi2(float chi2 = -1 ) const ;
  virtual const poirotReconstructedHit* FindHit(const poirotHit* ) const ;

  ArtePointerRSEG AppendRSEG() ;
  void AppendRSEG(ArtePointerRSEG rseg) ;
private:
  HitContainer fHits ;
  float fChi2 ;
  int fError ;
  static float fgMinChi2Probability ;
  ArtePointerRSEG fRSEG ;
};

inline bool
poirotReconstructedTrackInterface::ValidChi2(float chi2) const {
  if(chi2<0) chi2=GetChi2() ; 
  return chi2 < chisin_(1-GetMinChi2Probability(),GetNDof()) ; 
}

// Oke. So we once need o write a base class here.

// inline bool operator<(const poirotReconstructedTrackInterface& lhs,
// 		      const poirotReconstructedTrackInterface& rhs)
// {
//   if(rhs.GetHits().size()==lhs.GetHits().size())
//     return lhs.GetChi2() < rhs.GetChi2() ;
//   return rhs.GetHits().size() < lhs.GetHits().size() ;
// }

#endif
