#ifndef __POIROT_SECTORCLUSTER_HH__
#define __POIROT_SECTORCLUSTER_HH__

#ifndef __POIROT_HIT_HH__
#include "Hit.hh"
#endif
#ifndef __POIROT_GEDE_HH__
#include "Gede.hh"
#endif
#ifndef __SGI_STL_VECTOR
#include <vector>
#endif

class poirotSectorCluster
{
public:
  //typedef poirotSuperLayerEvent::HitContainer::const_iterator HitIterator ;
  // There is a bug in rootcint (repaired in new version)
  typedef vector<const poirotHit*> HitContainer ;
  typedef HitContainer::const_iterator HitIterator ;
  poirotSectorCluster(const poirotSectorCluster&) ;
  poirotSectorCluster() ;
  ~poirotSectorCluster() ;
  bool WithinYAcceptance(float tx, float ty) const ;
  float GetXDif(float tx, float ty=0) const ;
  float GetRawX() const ;
  float GetRawZ() const ;
  float GetRawTx() const ;
  //float GetStereoAngle() const ;
  poirotGede::EStereoAngle GetEStereoAngle() const ;
  float GetCosStereoAngle() const ;
  float GetSinStereoAngle() const ;
  float GetWirePitch() const ;
  int GetSize() const { return size() ; } ;
  HitIterator begin() const { return fHits.begin() ; } ;
  HitIterator  end() const { return fHits.end() ; } ;
  const poirotHit* front() const { return fHits.front() ; } ;
  const poirotHit* back() const { return fHits.back() ; } ;
  int size() const { return fHits.size() ; } ;
  void push_back(const poirotHit* hit) ;
  signed char GetLRSign(const poirotHit* hit) const ;
#ifdef __POIROT_USE_ROOT__
  //ClassDef(poirotSectorCluster,0)
  void Draw(int color) ;
  void PerformHistogramming() const ;
#endif
  void Dump() const ;
  bool ValidNHits() const ;
  static bool SameSectorCluster(const poirotHit* lhsWire, const poirotHit* rhsWire) ;
private:
  HitContainer fHits ;
  float fSumX ;
  float fSumZ ;
  float fY1 ;
  float fY2 ;
} ;

inline
poirotSectorCluster::~poirotSectorCluster() {}

inline
poirotSectorCluster::poirotSectorCluster(const poirotSectorCluster& rhs) 
  : fHits(rhs.fHits),fSumX(rhs.fSumX),fSumZ(rhs.fSumZ),
    fY1(rhs.fY1),fY2(rhs.fY2) {}

inline
poirotSectorCluster::poirotSectorCluster() 
  : fHits(0),fSumX(0), fSumZ(0), fY1(0), fY2(0) {}

inline void
poirotSectorCluster::push_back(const poirotHit* hit) 
{
  fHits.reserve(fHits.size()+1) ; // without this one, a memory leak
                                  // might be due to egcs 1.0 !!!!
  fHits.push_back(hit) ;
  fSumX += back()->GetRawX() ;
  fSumZ += back()->GetRawZ() ;
  fY1 = min(back()->GetRawY1(),fY1) ;
  fY2 = max(back()->GetRawY2(),fY2) ;
}

inline float
poirotSectorCluster::GetRawX() const {
  //return 0.5*(front()->GetWire()->GetRawX() + back()->GetWire()->GetRawX()) ;
  return fSumX/size() ;
}

inline float
poirotSectorCluster::GetRawZ() const {
  //return 0.5*(front()->GetWire()->GetRawZ() + back()->GetWire()->GetRawZ()) ;
  return fSumZ/size() ;
}

inline float 
poirotSectorCluster::GetRawTx() const {
  return GetRawX()/GetRawZ() ;
}

inline poirotGede::EStereoAngle
poirotSectorCluster::GetEStereoAngle() const {
  return front()->GetGede()->GetEStereoAngle() ;
}

inline float
poirotSectorCluster::GetCosStereoAngle() const {
  return front()->GetGede()->GetCosStereoAngle() ;
}

inline float
poirotSectorCluster::GetSinStereoAngle() const {
  return front()->GetGede()->GetSinStereoAngle() ;
}

inline float
poirotSectorCluster::GetWirePitch() const {
  return front()->GetGede()->GetWirePitch() ;
}

inline bool
poirotSectorCluster::WithinYAcceptance(float tx, float ty) const 
{
  // Moved formula from poirotGede.hh.
  float yraw = GetRawZ() *
    (ty*GetCosStereoAngle()+tx*GetSinStereoAngle()) ;
  return fY1 <= yraw && yraw <= fY2 ;
}

inline float
poirotSectorCluster::GetXDif(float tx, float ty) const {
  //for zero degree only 
  return GetRawX()- GetRawZ() * 
    (tx*GetCosStereoAngle() - ty*GetSinStereoAngle()) ;
}


#endif
