#ifndef __POIROT_TRACKCANDIDATE_HH__
#define __POIROT_TRACKCANDIDATE_HH__

#ifndef __SGI_STL_VECTOR
#include <vector>
#endif
#ifndef __ARTE_HH__
#include "arte/arte.hh"
#endif
#ifndef __POIROT_TRACK_HH__
#include "Track.hh"
#endif

class poirotCluster ;
class rangerTrack ;
class poirotTrack ;

class poirotTrackCandidate 
{
public:
  // typedefs, enumerates
  typedef vector<const poirotCluster*> ClusterContainer ;
  typedef ArtePointer<RSEG> ArtePointerRSEG ;

  // constructors, destructor
  poirotTrackCandidate();
  poirotTrackCandidate(const poirotCluster*, const poirotCluster*) ;
  poirotTrackCandidate(const poirotCluster* cl) ;
  poirotTrackCandidate( const poirotTrackCandidate& );
  ~poirotTrackCandidate();

  // other methods
  bool Valid() const ;
  bool CleanUp(const poirotTrackCandidate&) ;
  bool AddIfValid(const poirotCluster*) ;
  void Add(const poirotCluster*) ;
  const ClusterContainer& GetClusters() const { return fClusters ; } ;
  double GetTargetChi2() const { return fTargetChi2 ; } ;
  rangerTrack* GetRangerTrack() const { return fRangerTrack ; } ; 
  poirotTrack* GetTrack() const { return fTrack ; } ;
  
  ArtePointerRSEG GetRSEG() const { return fRSEG ; } ;
  ArtePointerRSEG AppendRSEG();
  bool RangerFit() ;
  bool Fit(bool=true) ;
private:
  ClusterContainer fClusters ;
  float fTargetChi2 ;
  ArtePointerRSEG fRSEG ;
  rangerTrack* fRangerTrack ;
  poirotTrack* fTrack ;
  HepVector fQ ;
  HepSymMatrix fCov ;
  float fChi2 ;
} ;


#endif
