#ifndef __POIROT_TRACK_DESCRIPTION_HH__
#define __POIROT_TRACK_DESCRIPTION_HH__

#ifndef _Vector_H_
#include "CLHEP/Matrix/Vector.h"
#endif
#ifndef _SYMMatrix_H_
#include "CLHEP/Matrix/SymMatrix.h"
#endif

/** poirotTrackDescription is a base class for describing tracks. It constains a vector of parameters and a corresponding covariance matrix. **/

class poirotTrackDescription 
{
public:
  virtual const HepVector& GetQ() const = 0 ;
  virtual const HepSymMatrix& GetCov() const = 0 ;
  virtual void SetQ(const HepVector& q ) = 0 ;
  virtual void SetCov(const HepSymMatrix& cov) = 0 ;
  virtual const HepDouble GetTx() const = 0 ;
  virtual const HepDouble GetTy() const = 0 ;
  virtual const HepDouble GetX() const = 0 ;
  virtual const HepDouble GetY() const = 0 ;
  virtual const HepDouble GetZ() const = 0 ;
  virtual const int GetDim() const = 0 ;
  virtual ~poirotTrackDescription() {} ;
} ;

class poirotTrackDescriptionBase : public virtual poirotTrackDescription, public HepVector
{
private:
  ///
  HepSymMatrix fCov;
public:
  poirotTrackDescriptionBase(int dim=4) ;
  poirotTrackDescriptionBase(const HepVector&, const HepSymMatrix& = 0) ;
  virtual const HepVector& GetQ() const {return *this ; } ;
  virtual const HepSymMatrix& GetCov() const { return fCov; } ;
  virtual void SetCov(const HepSymMatrix& cov) { fCov=cov ; } ;
  virtual void SetQ(const HepVector& q ) { HepVector::operator=(q) ; } ; 
  virtual const int GetDim() const { return num_row() ; } ;
  virtual ~poirotTrackDescriptionBase() ;
} ;

class poirotTrackDescription2D : public virtual poirotTrackDescription, 
			      private poirotTrackDescriptionBase
{
public:
  poirotTrackDescription2D() : poirotTrackDescriptionBase(2) {} ;
  poirotTrackDescription2D(const HepVector& q, const HepSymMatrix& cov= 0) : 
    poirotTrackDescriptionBase(q,cov) {} ;
  virtual ~poirotTrackDescription2D() {} ;
  
  // Declared by interface class
  using poirotTrackDescriptionBase::GetQ   ;
  using poirotTrackDescriptionBase::GetCov ;
  using poirotTrackDescriptionBase::SetQ   ;
  using poirotTrackDescriptionBase::SetCov ;
  using poirotTrackDescriptionBase::GetDim ;
  virtual const HepDouble GetX() const { return 0 ; } ;
  virtual const HepDouble GetY() const { return 0 ; } ;
  virtual const HepDouble GetZ() const { return 0 ; } ;
  virtual const HepDouble GetTx() const { return HepVector::operator[](0) ; } ;
  virtual const HepDouble GetTy() const { return HepVector::operator[](1) ; } ; 

  // Extra methods
  void SetTx(const HepDouble tx) { HepVector::operator[](0) = tx ; } ;
  void SetTy(const HepDouble ty) { HepVector::operator[](1) = ty ; } ;
};

class poirotTrackDescription4D : private poirotTrackDescriptionBase,
			      public virtual poirotTrackDescription 
{
public:
  poirotTrackDescription4D() : poirotTrackDescriptionBase(4) {} ;  
  poirotTrackDescription4D(const HepVector& q, const HepSymMatrix& cov= 0) : 
    poirotTrackDescriptionBase(q,cov) {} ;
  virtual ~poirotTrackDescription4D() {} ;

  // Declared by interface class
  using poirotTrackDescriptionBase::GetQ   ;
  using poirotTrackDescriptionBase::GetCov ;
  using poirotTrackDescriptionBase::SetQ   ;
  using poirotTrackDescriptionBase::SetCov ;
  using poirotTrackDescriptionBase::GetDim ;
  virtual const HepDouble GetX() const  { return HepVector::operator[](0) ; } ;
  virtual const HepDouble GetY() const  { return HepVector::operator[](1) ; } ;
  virtual const HepDouble GetZ() const  { return 0 ; } ; 
  virtual const HepDouble GetTx() const { return HepVector::operator[](2) ; } ;
  virtual const HepDouble GetTy() const { return HepVector::operator[](3) ; } ; 

  // Extra methods
  void SetX(const HepDouble x)   { HepVector::operator[](0) = x  ; } ;
  void SetY(const HepDouble y)   { HepVector::operator[](1) = y  ; } ;
  void SetTx(const HepDouble tx) { HepVector::operator[](2) = tx ; } ;
  void SetTy(const HepDouble ty) { HepVector::operator[](3) = ty ; } ;
};

#endif




