#include "poirot/ReconstructedHit.hh"
#include "poirot/TrackDescription.hh"
#include "poirot/Hit.hh"

//__________________________________________________________________
// poirotReconstructedHit
//
// Container for:
// * left-right ambiguity
// Functionality:
//
// Author: Wouter Hulsbergen 
//__________________________________________________________________

//#ifdef __OTR_CLASSDEF__
//ClassImp(poirotReconstructedHit) ;
//#endif

poirotReconstructedHit::poirotReconstructedHit() 
  : fHit(0), fLRSign(0), fLRSignIsFixed(false), fProjection(0) {}

poirotReconstructedHit::poirotReconstructedHit(const poirotHit* hit, LRSign_t lrSign) 
  : fHit(hit), fLRSign(0), fLRSignIsFixed(false),
    fProjection(0)
{
  SetLRSign(lrSign) ;
}

poirotReconstructedHit::poirotReconstructedHit(const poirotReconstructedHit& rhs) :
  fHit(rhs.fHit), fLRSign(rhs.fLRSign), 
  fLRSignIsFixed(rhs.fLRSignIsFixed), fProjection(0) 
{
  if(rhs.fProjection) fProjection = new poirotNodeProjection(*(rhs.fProjection)) ;
}

const poirotReconstructedHit& 
poirotReconstructedHit::operator=(const poirotReconstructedHit& rhs)
{
  fHit = rhs.fHit; 
  fLRSign = rhs.fLRSign ; 
  fLRSignIsFixed = rhs.fLRSignIsFixed ;
  if(!rhs.fProjection) delete fProjection ;
  else {
    if(fProjection) *fProjection=*rhs.fProjection ;
    else fProjection = new poirotNodeProjection(*(rhs.fProjection)) ;
  }
  return *this ;
}

poirotReconstructedHit::~poirotReconstructedHit() 
{
  delete fProjection ;
}

bool
poirotReconstructedHit::UpdateInternal() 
{
  bool changed=false ;
  if(!fLRSignIsFixed && fProjection && GetLRSign()*fProjection->GetD()<=0) {
    FlipLRSign() ;
    changed = true ;
    //SetLRSign(fProjection->GetD()) ;
  }
  return changed ;
}

ArtePointer<RHIT>
poirotReconstructedHit::AppendRHIT(const poirotTrackDescription* track)
{
  RHIT rhit;
  fRHIT = ArteTable<RHIT>::push_back(rhit) ;
  ArtePointer<HITB> hitb = GetMeasuredHit()->GetHITB() ;
  float dz = GetMeasuredHit()->GetRawZ() -
    track->GetZ() ;
  fRHIT->x  = dz*track->GetTx() + track->GetX() ; 
  fRHIT->y  = dz*track->GetTy() + track->GetY() ; 
  fRHIT->tx = track->GetTx() ;
  fRHIT->ty = track->GetTy() ;
  fRHIT->q  = fabs(hitb->q)*GetLRSign() ; //GetNodeProjection()->GetD() ;
  fRHIT->p  = 1; //1E-6; // Set 1/p to very high value (no multiple scatt.)
  if(ArteTable<EVHD>::begin()->exp > 0) hitb->q = fabs(hitb->q)*GetLRSign() ;
  createRelation(hitb,fRHIT,&HITB::rhit) ;
  return fRHIT ;
}
