#include <algorithm>
#include <functional>
#include <arte/arte.hh>
#include <arte/artefunc.h>
#include "poirot/Hit.hh"
#include "poirot/SuperLayerReco.hh"
#include "poirot/Cluster.hh"
#ifndef __POIROT_SECTORCLUSTER_HH__
#include "poirot/SectorCluster.hh"
#endif

#ifdef __OTR_USE_ROOT__
#include <TH1.h>
#endif

//__________________________________________________________________
// SuperLayerReco
//
// Container for:
// * Hits
// * Sector clusters
// * Super clusters
// Functionality
// * Reconstruction of clusters
//
// Author: Wouter Hulsbergen && Ivar Siccama
//__________________________________________________________________

//#define __OTR_RECOINFO__

poirotSuperLayerReco::poirotSuperLayerReco() : fHits(), fClusters() 
{
  //cout << "poirotSuperLayerReco::poirotSuperLayerReco()" << endl ;

} ;

poirotSuperLayerReco::~poirotSuperLayerReco() 
{  
  for(ClusterContainer::const_iterator clIt=fClusters.begin() ;
      clIt!=fClusters.end(); clIt++)
    delete *clIt ;
  //fHits.erase( fHits.begin(), fHits.end() );
  //fClusters.erase( fClusters.begin(), fClusters.end() );
}

void
poirotSuperLayerReco::Reset()
{
  // Delete SuperClusters
  for(ClusterContainer::iterator clIt=fClusters.begin() ;
      clIt!=fClusters.end(); clIt++) delete *clIt ;
  fClusters.erase( fClusters.begin(), fClusters.end() );

  // Delete sector clusters
  fZeroClusters.erase(fZeroClusters.begin(),fZeroClusters.end()) ;
  fPlusClusters.erase(fPlusClusters.begin(),fPlusClusters.end()) ;
  fMinusClusters.erase(fMinusClusters.begin(),fMinusClusters.end()) ;

  // Delete hits
  for(HitContainer::iterator hitIt=fHits.begin() ;
      hitIt!=fHits.end() ; hitIt++) delete *hitIt ;
  fHits.erase( fHits.begin(), fHits.end() );
}

bool SLRcompareHit( const poirotHit* lhs, const poirotHit* rhs )
{
  // Do not forget to change poirotSectorCluster::SameCluster if you
  // change this one !!!
  int rhsvalue = rhs->GetXColumn() + 10000 *
    (rhs->GetGede()->GetSectorId() + 100 * rhs->GetGede()->GetEStereoAngle()) ;
  int lhsvalue = lhs->GetXColumn() + 10000 *
    (lhs->GetGede()->GetSectorId() + 100 * lhs->GetGede()->GetEStereoAngle()) ; 
  return lhsvalue < rhsvalue ;

//   return lhs->GetGede()->GetEStereoAngle()<rhs->GetGede()->GetEStereoAngle() ||
//     ((lhs->GetGede()->GetEStereoAngle()==rhs->GetGede()->GetEStereoAngle()) &&
//      ((lhs->GetGede()->GetSectorId() < rhs->GetGede()->GetSectorId()) ||
//       ((lhs->GetGede()->GetSectorId()== rhs->GetGede()->GetSectorId()) &&
//        (lhs->GetRawX()<rhs->GetRawX())))) ;
}
 
void
poirotSuperLayerReco::SortHits()
{
  //cout << "poirotSuperLayerEvent::SortHits()" << endl ;
  sort( fHits.begin(), fHits.end(), SLRcompareHit );
  //fHits.sort( compareHit );
}

void 
poirotSuperLayerReco::DumpHits(ostream& os) const
{
  os << "poirotSuperLayerReco::DumpHits(...) dumps " << fHits.size() 
     << " hits." << endl ;
  int i=0;
  for(HitContainer::const_iterator hitp=fHits.begin();
      hitp != fHits.end(); hitp++, i++)
    if ( *hitp ) os << *hitp << endl ;
  return ;
}

// #ifdef __OTR_USE_ROOT__

// void 
// poirotSuperLayerReco::DrawHits()
// {
//   cout << "poirotSuperLayerReco::DrawHits(), nr of hits = " << fHits.size()
//        << endl;
// //   for(HitContainer::const_iterator hitp=fHits.begin();
// //       hitp != fHits.end(); hitp++)
// //     (*hitp)->Draw(2) ;
// }

// void 
// poirotSuperLayerReco::DrawClusters()
// {
//   cout << "poirotSuperLayerReco::DrawClusters() " << fClusters.size() << endl ;
//   int color=3 ;
//   for(poirotSuperLayerReco::ClusterContainer::iterator clusterp= fClusters.begin() ;
//       clusterp != fClusters.end() ; clusterp++, color++)
//     (*clusterp)->Draw(color) ;
// };

// #endif

void 
poirotSuperLayerReco::DumpClusters(ostream& os) const
{
  os << "poirotSuperLayerReco::DumpClusters(...) dumps " << fClusters.size()
     << " clusters. " << endl ;
  for(poirotSuperLayerReco::ClusterContainer::const_iterator clusterp= fClusters.begin() ;
      clusterp != fClusters.end() ; clusterp++)
    (*clusterp)->Dump() ;
}

void 
poirotSuperLayerReco::AppendRSEG()
{
  // Append poirotSuperClusters to the RSEG bank. I added this routine
  // for FLT studies. They are not yet interested in tracks, only in
  // clusters.
  for(ClusterContainer::iterator clusIt = fClusters.begin() ; 
      clusIt != fClusters.end() ; clusIt++) (*clusIt)->AppendRSEG() ;
}

void
poirotSuperLayerReco::SearchSectorClusters()
{
  //cout << "poirotSuperLayerReco::SearchSectorClusters()" << endl ;
  //This is the cluster search algorithm for clusters in one poirotSector only.
  SortHits() ;
  // Cleanup current stuff
  if(fPlusClusters.size()>0 || fZeroClusters.size()>0 || fMinusClusters.size()>0) {
    cout << "poirotSuperLayerReco::CreateSectorClusters() has already been called !" << endl ;
    return ;
  }
  fPlusClusters.reserve(100) ;
  fZeroClusters.reserve(100) ;
  fMinusClusters.reserve(100) ;

  HitContainer::const_iterator hitIt(fHits.begin()), nextHitIt(hitIt) ;
  nextHitIt++;
  poirotSectorCluster* currentCluster = new poirotSectorCluster() ;
  for(hitIt=fHits.begin() ; hitIt!=fHits.end() ; hitIt++, nextHitIt++) {
    currentCluster->push_back(*hitIt) ;
    //     cout << (*hitIt)->GetGede()->GetSectorId() << "("
    // 	 << (int)((*hitIt)->GetGedeIndex()) << ","
    // 	 << (int)((*hitIt)->GetWireIndex()) << ","
    //      	 << (int)((*hitIt)->GetPRWireIndex())<< ")" ;
    bool clusterIsComplete = (nextHitIt==fHits.end()) || 
      !poirotSectorCluster::SameSectorCluster(*hitIt,*nextHitIt) ;
    if(clusterIsComplete) {
      //      cout << endl ;
      if(currentCluster->ValidNHits()) {
   	switch (currentCluster->GetEStereoAngle())
   	  {
   	  case poirotGede::kPlusFiveDegrees :
   	    fPlusClusters.push_back(*currentCluster) ; break ;
   	  case poirotGede::kMinusFiveDegrees :
   	    fMinusClusters.push_back(*currentCluster) ; break ;
   	  default :
   	    fZeroClusters.push_back(*currentCluster) ; break ;
   	  }
      }
      // cout << "|" ;
      delete currentCluster;
      currentCluster = new poirotSectorCluster() ;
    }
  }
  delete currentCluster;
  
#ifdef __OTR_RECOINFO__
  cout << "  NHits : " << fHits.size() 
       << "  NSectorClusters (+0-): "  << fPlusClusters.size() << " "
       << fZeroClusters.size() << " " << fMinusClusters.size() << endl ;
#endif
}

inline bool 
CompareXDif(const poirotCluster* lhs, const poirotCluster* rhs) {
  return fabs(lhs->GetXDif()) < fabs(rhs->GetXDif()) ;
} ;

inline bool 
CompareChi2(const poirotCluster* lhs, const poirotCluster* rhs) {
  return lhs->GetChi2Prob() > rhs->GetChi2Prob() ;
}

int
poirotSuperLayerReco::SearchClusters()
{
  //cout << "poirotSuperLayerReco::SearchClusters()" << endl ;
  // Ths routine implements the cluster finding algorithm in three
  // stereo layers. It creaes co called poirotSuperClusters, which are
  // made up of poirotSectorClusters, defined in the
  // SearchSectorCluster memberfunction.
#ifdef __OTR_USE_ROOT__
  static TH1F* xdifH1 = new TH1F("xdifH1","",500,-3,3) ;
  static TH1F* clusterMultH1 = new TH1F("clusterMultH1","",100,-0.5,99.5) ;
  static TH1F* clusterSizeH1 = new TH1F("clusterSizeH1","",20,-0.5,19.5) ;
#endif
  //SortHits() ;
  SearchSectorClusters() ;

  // Make all combinations of one plus and one minus cluster. Find a
  // zero cluster that is close enough in the Tx projection. If more
  // than one cluster is close enough, choose the best one. This is
  // not completely unbiased but pays a bit in speed.
  poirotCluster* currentCluster ;
  for(SectorClusterContainer::iterator minusIt=fMinusClusters.begin();
      minusIt!=fMinusClusters.end(); minusIt++) 
    for(vector<poirotSectorCluster>::iterator plusIt=fPlusClusters.begin();
	plusIt!=fPlusClusters.end(); plusIt++)
      {
	currentCluster = poirotCluster::Create(plusIt,minusIt) ;
	if(currentCluster) {
	  for(SectorClusterContainer::iterator zeroIt=fZeroClusters.begin();
	      zeroIt!=fZeroClusters.end(); zeroIt++)
	    currentCluster->SetZeroIfValid(zeroIt) ;
	  if(currentCluster->Valid()) {
	    fClusters.push_back(currentCluster) ;
	  } else {
	    delete currentCluster ;
	  }
	}
      }
  //#else
  // Make ALL valid combinations of one plus, one minus and one zero cluster.
  // This does not make a difference at all in MC !!!!
  //#endif

#ifdef __OTR_RECOINFO__
  cout << "Found " << fClusters.size() << " valid clusters, left " ;
#endif

  // Fit all clusters in the Tx-Ty plane. If the fit yields an invalid
  // chi2, remove the cluster.
  for(ClusterContainer::iterator clusterIt=fClusters.begin() ;
      clusterIt!=fClusters.end(); clusterIt++) {
    (*clusterIt)->CreateReconstructedHits() ;
    //     if(!(*clusterIt)->ValidChi2()) {
    //       delete *clusterIt ;
    //       *clusterIt = 0;
    //       fClusters.erase(clusterIt--) ;
    //     }
  } 
//   ClusterContainer::iterator new_end = 
//     remove_if(fClusters.begin(), fClusters.end(),
// 	      bind2nd(equal_to<poirotCluster*>(),(poirotCluster*)(0))) ;
//   fClusters.erase(new_end,fClusters.end()) ;
//   sort(fClusters.begin(),fClusters.end(),CompareChi2) ;
  
// #ifdef __OTR_RECOINFO__
//   cout << fClusters.size() << " after chi2 cut, left" ;
// #endif

// #define __OTR_SEARCHCLUSTERS_DOCLEANUP__
// #ifdef __OTR_SEARCHCLUSTERS_DOCLEANUP__
//   /** REMOVE OR MERGE ALL OVERLAPPING CLUSTERS **/
//   /** (Be careful with order: this is the only correct one ?!) **/
//   //   for(ClusterContainer::reverse_iterator tClusterIt=fClusters.rbegin();
//   //       tClusterIt!=fClusters.rend(); tClusterIt++) 
//   //     for(ClusterContainer::reverse_iterator tClusterIt2=fClusters.rbegin();
//   // 	tClusterIt2!=tClusterIt; tClusterIt2++) 
//   //       if(tClusterIt->Merge(*tClusterIt2) || !tClusterIt2->Valid()) {
//   // 	fClusters.erase(&(*tClusterIt2)) ;
//   //       }; 

//   /** This seems the optimal solution, but it is also slow ...! Is it finite ? **/

//   // WDH 02/08/99. Ivar found out last month that there is a bug in here ...
//   bool merged ;
//   for(ClusterContainer::iterator clusterIt=fClusters.begin() ;
//       clusterIt!=fClusters.end(); clusterIt++) 
//     if(*clusterIt) {
//       ClusterContainer::iterator clusterIt2= fClusters.end();
//       clusterIt2--;
//       for(;clusterIt2!= clusterIt; clusterIt2--)
// 	if(*clusterIt2 && 
// 	   ((merged=(*clusterIt)->Merge(**clusterIt2)) || !(*clusterIt2)->Valid())) {
// 	  delete *clusterIt2;
// 	  *clusterIt2 = 0;
// 	  //fClusters.erase(clusterIt2) ;
// 	  //#ifdef OTR_SEARCHCLUSTERS_SLOW
// 	  //if(merged) clusterIt2=fClusters.end();
// 	  //#endif
// 	}
//     }
//   // now remove all NULL-pointers
//   new_end = 
//     remove_if(fClusters.begin(), fClusters.end(),
// 	      bind2nd(equal_to<poirotCluster*>(),(poirotCluster*)(0))) ;
//   fClusters.erase(new_end,fClusters.end()) ;

// #endif
// #ifdef __OTR_RECOINFO__
//   cout << fClusters.size() << " after cleanup. " << endl ;
// #endif
//   for(ClusterContainer::iterator clusterIt=fClusters.begin();
//       clusterIt!=fClusters.end(); clusterIt++) {
//     //(*clusterIt)->CreateReconstructedHits() ;
// #ifdef __OTR_USE_ROOT__
//     clusterSizeH1->Fill(float((*clusterIt)->GetHits().size())) ;
//     xdifH1->Fill(float((*clusterIt)->GetXDif()/
// 		       (*clusterIt)->GetZeroCluster()->GetWirePitch())) ;
// #endif
//     if(!(*clusterIt)->ValidChi2()) {
//       delete *clusterIt ;
//       fClusters.erase(clusterIt--) ;
//     }
//   }
// #ifdef __OTR_USE_ROOT__
//   clusterMultH1->Fill(fClusters.size()) ;
// #endif
  return fClusters.size() ;
}
