/* Use these staff in the form call_via_istream(F f, (Arg1*)0, ..., (ArgN*)0);
   where F f - some function object which takes 0<=N<=5 arguments and
   expression f(a1, .., aN) 
   (a1, .., aN are of types Arg1, .., ArgN respectively) 
   is valid.
   call_via_istream returns an object of type CallViaIstream0..5<...>
   which has the operator()(istream* is) => it extracts N arguments from "*is"
   and calls f(a1, .., aN)               
*/

// Author V.Balagura (balagura@mail.desy.de)   (09.04.98).

#ifndef _call_via_istream_hh
#define _call_via_istream_hh 1

#include <iostream.h>

/* -------------------- 0 args -------------------- */
template <class F>
class CallViaIstream0 {
public:
  F function;

  CallViaIstream0(F f) : function(f) {}
  void operator()(istream* is) {
    function();
  }
};
template <class F>
inline CallViaIstream0<F> call_via_istream(F f) { 
  return CallViaIstream0<F>(f); 
}
/* -------------------- 1 arg -------------------- */
template <class F, class A1>
class CallViaIstream1 {
public:
  F function;

  CallViaIstream1(F f) : function(f) {}
  void operator()(istream* is) {
    A1 a1;
    *is >> a1; if ( !(*is) ) return;
    function(a1);
  }
};
template <class F, class A1>
inline CallViaIstream1<F, A1> call_via_istream(F f, A1*) { 
  return CallViaIstream1<F, A1>(f); 
}
/* -------------------- 2 args -------------------- */
template <class F, class A1, class A2>
class CallViaIstream2 {
public:
  F function;

  CallViaIstream2(F f) : function(f) {}
  void operator()(istream* is) {
    A1 a1;
    A2 a2;
    *is >> a1; if ( !(*is) ) return;
    *is >> a2; if ( !(*is) ) return;
    function(a1, a2);
  }
};
template <class F, class A1, class A2>
CallViaIstream2<F, A1, A2> call_via_istream(F f, A1*, A2*) { 
  return CallViaIstream2<F, A1, A2>(f); 
}
/* -------------------- 3 args -------------------- */
template <class F, class A1, class A2, class A3>
class CallViaIstream3 {
public:
  F function;

  CallViaIstream3(F f) : function(f) {}
  void operator()(istream* is) {
    A1 a1;
    A2 a2;
    A3 a3;
    *is >> a1; if ( !(*is) ) return;
    *is >> a2; if ( !(*is) ) return;
    *is >> a3; if ( !(*is) ) return;
    function(a1, a2, a3);
  }
};
template <class F, class A1, class A2, class A3>
inline CallViaIstream3<F, A1, A2, A3> call_via_istream(F f, A1*, A2*, A3*) { 
  return CallViaIstream3<F, A1, A2, A3>(f); 
}
/* -------------------- 4 args -------------------- */
template <class F, class A1, class A2, class A3, class A4>
class CallViaIstream4 {
public:
  F function;

  CallViaIstream4(F f) : function(f) {}
  void operator()(istream* is) {
    A1 a1;
    A2 a2;
    A3 a3;
    A4 a4;
    *is >> a1; if ( !(*is) ) return;
    *is >> a2; if ( !(*is) ) return;
    *is >> a3; if ( !(*is) ) return;
    *is >> a4; if ( !(*is) ) return;
    function(a1, a2, a3, a4);
  }
};
template <class F, class A1, class A2, class A3, class A4>
inline CallViaIstream4<F, A1, A2, A3, A4> call_via_istream
(F f, A1*, A2*, A3*, A4*) { 
  return CallViaIstream4<F, A1, A2, A3, A4>(f); 
}
/* -------------------- 5 args -------------------- */
template <class F, class A1, class A2, class A3, class A4, class A5>
class CallViaIstream5 {
public:
  F function;

  CallViaIstream5(F f) : function(f) {}
  void operator()(istream* is) {
    A1 a1;
    A2 a2;
    A3 a3;
    A4 a4;
    A5 a5;
    *is >> a1; if ( !(*is) ) return;
    *is >> a2; if ( !(*is) ) return;
    *is >> a3; if ( !(*is) ) return;
    *is >> a4; if ( !(*is) ) return;
    *is >> a5; if ( !(*is) ) return;
    function(a1, a2, a3, a4, a5);
  }
};
template <class F, class A1, class A2, class A3, class A4, class A5>
inline CallViaIstream5<F, A1, A2, A3, A4, A5> call_via_istream
(F f, A1*, A2*, A3*, A4*, A5*) { 
  return CallViaIstream5<F, A1, A2, A3, A4, A5>(f); 
}

#endif
