// 
//  Author:  Rainer Mankel                     Created:  29-Sep-98
//           Humboldt University Berlin        Modified:  
//  Email:   mankel@ifh.de                     HERA-B Collaboration
// 
#include <iostream.h>
#include "arte/ArteMessage.hh"
#include "rang/Volume.hh"
#include "rang/VolBox.hh"
#include "rang/VolEpi.hh"
#include "rang/VolTrd1.hh"
#include "rang/VolTrd2.hh"
#include "rang/VolTube.hh"
#include "rang/VolTubs.hh"
#include "rang/VolCone.hh"
#include "rang/VolDummy.hh"
#include "rang/Node.hh"
#include "rang/Arrow.hh"
#include "rang/FLAGRotFix.hh"

//-- Constructor
rangerVolume::rangerVolume(){
  Gewa = 0;
  for (int i=0; i<=2; i++) {         // initialize rot matrix
    for (int j=0; j<=2; j++) {
      rotTot[i][j] = (i==j)? 1 :0;
    }
  }
  z = 0;
  msValue = 0;
  rotated = 0;
  zTilted = 0;
  totRotated = 0;
  nextVolume = 0;
  previousVolume = 0;
};

//-- Constructor from gewa
rangerVolume::rangerVolume( rangerGeWa* gewa ){
  Gewa = gewa;                       // copy pointer to gewa
  rotTot[0][0] = gewa->rot(1) * gewa->cos() - gewa->rot(2) * gewa->sin();
  rotTot[0][1] = gewa->rot(4) * gewa->cos() - gewa->rot(5) * gewa->sin();
  rotTot[0][2] = gewa->rot(7) * gewa->cos() - gewa->rot(8) * gewa->sin();
  rotTot[1][0] = gewa->rot(1) * gewa->sin() + gewa->rot(2) * gewa->cos();
  rotTot[1][1] = gewa->rot(4) * gewa->sin() + gewa->rot(5) * gewa->cos();
  rotTot[1][2] = gewa->rot(7) * gewa->sin() + gewa->rot(8) * gewa->cos();
  rotTot[2][0] = gewa->rot(3);
  rotTot[2][1] = gewa->rot(6);
  rotTot[2][2] = gewa->rot(9);
  z = zMean();
  switch (gewa->cmp()) {
  case 0:
    msValue = 1.0 * gewa->ms();  // wall
    break;
  case 1:
    msValue = 1.1 * gewa->ms();  // SiVD
    break;
  case 2:
    msValue = 1.4 * gewa->ms();  // HDC
    break;
  case 3:
    msValue = 1.6 * gewa->ms();  // MSGC
    break;
  default:
    msValue = 1.1 * gewa->ms();
    break;
  }

  rotated = (gewa->rot(1) == 1 && gewa->rot(5) == 1 && gewa->rot(9) == 1)?
    0 : 1;
  zTilted = !(gewa->rot(3) == 0 && gewa->rot(6) == 0 && 
	      gewa->rot(7) == 0 && gewa->rot(8) == 0);
  totRotated = (rotTot[0][0] == 1 && rotTot[1][1] == 1 && rotTot[2][2] == 1)?
    0 : 1;
  
  //if (z>200. && z<650.)  msValue *= 1.8;  // extra MS for magnet
  if (z>200. && z<650.) {
    if (gewa->cmp() != 1 && gewa->cmp() != 3) {
      msValue *= 1.8;  // extra MS for magnet
    }

    if (z>400.) {
      msValue *= 1.3; // extra extra MS
    }
  }

  nextVolume = 0;
  previousVolume = 0;
};

//-- Destructor
rangerVolume::~rangerVolume(){
  delete Gewa;
};


void rangerVolume::rotateParam(double x,  // input parameters
			       double y,
			       double zz,
			       double tx,
			       double ty,
			       double& xP, // output parameters
			       double& yP,
			       double& zP,
			       double& txP,
			       double& tyP){
  if (rotated) {
    if (zTilted) {
      xP = Gewa->rot(1) * x + Gewa->rot(4) * y + Gewa->rot(7) * zz;
      yP = Gewa->rot(2) * x + Gewa->rot(5) * y + Gewa->rot(8) * zz;
      zP = Gewa->rot(3) * x + Gewa->rot(6) * y + Gewa->rot(9) * zz;

      double tzn = Gewa->rot(3) * tx + Gewa->rot(6) * ty + Gewa->rot(9);
      if (tzn == 0) {
	cout << "rangerVolume::rotateParam: tzn=" << tzn << endl;
	tzn = .00001;
      }
      txP = (Gewa->rot(1) * tx + Gewa->rot(4) * ty + Gewa->rot(7)) 
	/ tzn;
      tyP = (Gewa->rot(2) * tx + Gewa->rot(5) * ty + Gewa->rot(8)) 
	/ tzn;
    }
    else {
      xP = Gewa->rot(1) * x + Gewa->rot(4) * y;  // coordinates in PCS 
      yP = Gewa->rot(2) * x + Gewa->rot(5) * y;  // see Hera-B 97-090 p 68
      zP = zz;
      txP = tx;
      tyP = ty;
    }
  }
  else {
    xP = x;
    yP = y;
    zP = z;
    txP = tx;
    tyP = ty;
  }
}

void rangerVolume::rotateArrow(rangerArrow& a, int direction){

  if (rotated) {
    double xP, yP, zP, txP, tyP;
    if (direction == +1) {
      if (zTilted) {
#ifdef ROTFIX
	xP = Gewa->rot(1) * a.x + Gewa->rot(4) * a.y + Gewa->rot(7) * (a.z-1080);
	yP = Gewa->rot(2) * a.x + Gewa->rot(5) * a.y + Gewa->rot(8) * (a.z-1080);
	zP = Gewa->rot(3) * a.x + Gewa->rot(6) * a.y + Gewa->rot(9) * (a.z-1080)
	  + 1080;
#else	
	xP = Gewa->rot(1) * a.x + Gewa->rot(4) * a.y + Gewa->rot(7) * a.z;
	yP = Gewa->rot(2) * a.x + Gewa->rot(5) * a.y + Gewa->rot(8) * a.z;
	zP = Gewa->rot(3) * a.x + Gewa->rot(6) * a.y + Gewa->rot(9) * a.z;
#endif
	
	double tzn = Gewa->rot(3) * a.tx + Gewa->rot(6) * a.ty + Gewa->rot(9);
	if (tzn == 0) {
	  cout << "rangerVolume::rotateArrow: tzn=" << tzn << endl;
	  tzn = .00001;
	}
	txP = (Gewa->rot(1) * a.tx + Gewa->rot(4) * a.ty + Gewa->rot(7)) 
	  / tzn;
	tyP = (Gewa->rot(2) * a.tx + Gewa->rot(5) * a.ty + Gewa->rot(8)) 
	  / tzn;
      }
      else {
	xP = Gewa->rot(1) * a.x + Gewa->rot(4) * a.y;  // coordinates in PCS 
	yP = Gewa->rot(2) * a.x + Gewa->rot(5) * a.y;  // see Hera-B 97-090 p 68
	zP = a.z;
	txP = a.tx;
	tyP = a.ty;
      }
      a.z = zP;
      a.x = xP;
      a.y = yP;
      a.tx = txP;
      a.ty = tyP;
    }
    else {
      if (zTilted) {
#ifdef ROTFIX
	xP = Gewa->rot(1) * a.x + Gewa->rot(2) * a.y + Gewa->rot(3) * (a.z-1080);
	yP = Gewa->rot(4) * a.x + Gewa->rot(5) * a.y + Gewa->rot(6) * (a.z-1080);
	zP = Gewa->rot(7) * a.x + Gewa->rot(8) * a.y + Gewa->rot(9) * (a.z-1080)
	  + 1080;
#else
	xP = Gewa->rot(1) * a.x + Gewa->rot(2) * a.y + Gewa->rot(3) * a.z;
	yP = Gewa->rot(4) * a.x + Gewa->rot(5) * a.y + Gewa->rot(6) * a.z;
	zP = Gewa->rot(7) * a.x + Gewa->rot(8) * a.y + Gewa->rot(9) * a.z;
#endif

	double tzn = Gewa->rot(7) * a.tx + Gewa->rot(8) * a.ty + Gewa->rot(9);
	if (tzn == 0) {
	  cout << "rangerVolume::rotateArrow: tzn=" << tzn << endl;
	  tzn = .00001;
	}
	txP = (Gewa->rot(1) * a.tx + Gewa->rot(2) * a.ty + Gewa->rot(3)) 
	  / tzn;
	tyP = (Gewa->rot(4) * a.tx + Gewa->rot(5) * a.ty + Gewa->rot(6)) 
	  / tzn;
      }
      else {
	xP = Gewa->rot(1) * a.x + Gewa->rot(2) * a.y;  // coordinates in PCS 
	yP = Gewa->rot(4) * a.x + Gewa->rot(5) * a.y;  // see Hera-B 97-090 p 68
	zP = a.z;
	txP = a.tx;
	tyP = a.ty;
      }
      a.z = zP;
      a.x = xP;
      a.y = yP;
      a.tx = txP;
      a.ty = tyP;
    }
  }
}



int rangerVolume::passedBody(rangerNode* n){
  return passed(n->getState().z(),n->getState().x(),n->getState().y(),
		n->getState().tx(),n->getState().ty());
}



float rangerVolume::sLengNode(rangerNode* n){
  return sLength(n->getState().z(),n->getState().x(),n->getState().y(),
		n->getState().tx(),n->getState().ty());
}



rangerVolume* rangerVolume::makeVolume( rangerGeWa* gewa ) {
  rangerVolume* vb = 0;
  switch ( gewa->shp() ) {
  case 1:
    vb = new rangerVolBox( gewa );
    break;
  case 2:
    vb = new rangerVolTrd1( gewa );
    break;
  case 3:
    vb = new rangerVolTrd2( gewa );
    break;
  case 5:
    vb = new rangerVolTube( gewa );
    break;
  case 6:
    vb = new rangerVolTubs( gewa );
//     if (vb) 
//       cout << "rangerVolume::makeVolume: new Tubs created, z=" << vb->z 
// 	   << " Gede/Gwal=" << vb->getGede() << "/" << vb->getGwal() 
// 	   << "/" << vb->plNumber() << endl;
    break;
  case 7:
    vb = new rangerVolCone( gewa );
    break;
  case 10:
    vb = new rangerVolEpi( gewa );
    break;
  case 34:
    vb = new rangerVolBox( gewa );   // Lanyov trapezoid, treat like a box
    break;
  case 42:
    vb = new rangerVolBox( gewa );   // Lanyov trapezoid, treat like a box
    break;
  default:
    arteMsg << "rangerVolume::makeVolume: use rangerVolDummy for shp="
	    << gewa->shp() << " pl=" << gewa->plNumber() << endl;
    static int id;
    arteMsg.warn(id);
    vb = new rangerVolDummy( gewa );
    break;
  };

  return vb;
};

