// 
//  Author:  Rainer Mankel                     Created:  25-Apr-99
//           Humboldt University Berlin        Modified:  
//  Email:   mankel@ifh.de                     HERA-B Collaboration
// 

#include <algo.h>
#include "rang/Domain.hh"
#include "rang/DomWin.hh"
#include "rang/DomObj.hh"
#include "rang/NavItem.hh"
#include "rang/XWin.hh"
#include "rang/XPro.hh"


int rangerXPro::xWinFill(rangerDomain* dom,rangerDomWin* win,
			 float z,double x,double tx) { 
  if (dom->xwin()) {
    dom->xwin()->setx(z,x,tx);
    if (win) {
      dom->xwin()->setWin(win);
//       cout << "rangerXPro::xWinFill: win=" << win << endl;
     if (win->getNI()) navSet.insert(win->getNI());
     return 1;
    }
    else {
      cout << "rangerXPro::xWinFill: window undefined" << endl;
      return 0;
    }
  } 
  return 0;
}


void rangerXPro::xProInt(vector<rangerDomain*>& domains) {

  // find the first filled xWin from left side
  vector<rangerXWin*>::iterator xwit1 = xWs.end();
  vector<rangerXWin*>::iterator xwit2 = xWs.end();
  for (vector<rangerXWin*>::iterator xwit=xWs.begin();
       xwit !=xWs.end(); ++xwit) {
    if ((*xwit)->filled()) {
      if (xwit1 == xWs.end()) {
	xwit1 = xwit;
      }
      xwit2 = xwit;      // xwit2 is the last valid iterator
    }
  }

  // do interpolations if necessary
  for (vector<rangerXWin*>::iterator xwit=xwit1;
       xwit != (xwit2+1); ++xwit) {
    if ((*xwit)->filled()) continue;
    // try fixup
    (*xwit)->setx((*xwit)->dom()->zref(),
		  (*(xwit-1))->x() 
		  + ((*xwit)->dom()->zref() - ((*(xwit-1))->z())) 
		  *  (*(xwit-1))->tx(),(*(xwit-1))->tx());
    // find plausible navigation window

    int direction =  (find(domains.begin(),domains.end(),(*xwit2)->dom()) 
		      > find(domains.begin(),domains.end(),(*xwit1)->dom()))? 
      1: -1;
    if ((*(xwit-1))->win()) {
      if ((*(xwit-1))->win()->getNI()) {
	if ((*(xwit-1))->win()->getNI()->getTargets(direction).begin()
	    != ((*(xwit-1))->win()->getNI()->getTargets(direction).end())) {
	  (*xwit)->setWin((*((*(xwit-1))->win()->
			     getNI()->getTargets(direction).begin()))->win());
	  navSet.insert(*((*(xwit-1))->win()->
			  getNI()->getTargets(direction).begin()));
	}
      }
    }
  }
}


int rangerXPro::navig(const rangerDomWin* dw) {
  if (dw->getNI()) {
    return ((find(navSet.begin(),navSet.end(),dw->getNI()) != navSet.end())?
	    1:0);
  }
  else {
    cout << " rangerXPro::navig: invalid dw->getNI" << endl;
    return 0;
  }
}
 
      

void rangerXPro::dump(const int level) {
  cout << "......................................" << endl;
  cout << " XPro dump: ft=" << ftLink << " qptEst=" << qptEst << endl;
  if (level >= 2) {
    cout << " ... dump of navSets follows" << endl;
    for (set<rangerNavItem*>::iterator nsi=navSet.begin();
	 nsi != navSet.end(); ++nsi) {
      (*nsi)->dump();
    }
  }
  if (level >= 3) {
    cout << " ... dump of xwins follows" << endl;
    for (vector<rangerXWin*>::iterator xwi=xWs.begin();
	 xwi != xWs.end(); ++xwi) {
      (*xwi)->dump();
    }
  }
}
    
    
