// 
//  Author:  Rainer Mankel                     Created:  29-Sep-98
//           Humboldt University Berlin        Modified:  
//  Email:   mankel@ifh.de                     HERA-B Collaboration
// 

// class rangerHitBase
// abstract node base class for all measurements

#ifndef rangerHitBase_HH
#define rangerHitBase_HH

#include "arte/arte.hh"
#include "CLHEP/Matrix/Matrix.h"
#include "CLHEP/Matrix/SymMatrix.h"
#include "CLHEP/Matrix/Vector.h"
#include "rang/Node.hh"
#include "rang/Volume.hh"
#include "rang/FitMode.hh"

class rangerHitBase : public rangerNode {

public:

  rangerHitBase();
  rangerHitBase(ArtePointer<RHIT> rhit,rangerFitMode fm);
  rangerHitBase(ArtePointer<HITB> hitb,rangerFitMode fm);
  rangerHitBase(ArtePointer<RCCL> rccl,int charge,rangerFitMode fm);
  virtual ~rangerHitBase();   // virtual destructor
  
  virtual float msPlain() const { return 0; }

  void processNoise(rangerParticleType pType);  
                             // apply process noise to state of this node
  void predProcessNoise(rangerParticleType pType);  
                             // apply process noise to prediction
  virtual void projectP(HepVector& resid,HepMatrix& HH,rangerStateP* s)=0;
  virtual void projectPI(HepVector& resid,HepMatrix& HH,rangerStatePI* s)=0;
  virtual void projectL(HepVector& resid,HepMatrix& HH,rangerStateL* s)=0;
  virtual void projectLI(HepVector& resid,HepMatrix& HH,rangerStateLI* s)=0;
  int filter();              // filter a node from scratch
  int filter(rangerNode* n,
	     rangerParticleType pType); // filter a node 
                                        //using state from other mode
  int absorb();              // update a node by absorbing predicted state
  void residUpdate();        // update residual and chiSq
  void residUpdSmooth(rangerNode* n,HepMatrix& A);

  virtual int dim() const=0;       // dimension of measurement
  inline const double getChiSq() const { return chiSq; };
  inline HepVector* getMeasurement() { return m ; } ;
  inline HepVector* getResidual() { return r ; } ;

protected:
  HepVector* m;       // measurement
  HepSymMatrix* V;    // cov matrix of measurement
  HepMatrix* H;       // projection matrix
  HepVector* r;       // residual
  HepSymMatrix* R;    // cov matrix of residual
  double chiSq;       // chiSquare contribution

private:

};

#endif
