// 
//  Author:  Rainer Mankel                     Created:  17-Apr-99
//           Humboldt University Berlin        Modified: 16-Jun-99 
//  Email:   mankel@ifh.de                     HERA-B Collaboration
// 

// rcZevnt.C
// reconstruction main program for full detector setup
// (in statu nascendi)


#include "arte/arte.hh"
#include "rang/zfpar.hh"
#include "rang/rgideal.hh"
#include "reco/recoOptions.hh"
#include "reco/rcsetp0.hh"
#include "care/care.hh"
#include "care/ECALDetector.hh"
#include "cats/cats.h"

#include <set.h>
#include <string>

// Prototypes
extern "C" {
  void careinit_();
  void marpleinit_();
  void rgmaintr_();
  void tfunidm_();
  void careevnt1_();
  void careevnt2_();
  void mtchrscr_();
  void tfbuild_();
  void rcelid_();
  void rcvert_();
  void Riter_init_();
  void Riter_evnt_();
  void murecb_();
  void rcmuid_();
  void rcrings_();
  void matchsvdmt_();
}
int SvdInit();
int RICHrec(int level=10);
void HolmesEvent();

extern recoOptions RecoOpt;

extern "C" {
  void rcZevnt_() {

    // first-event initialization
    static int init = 0;
    if (! init) {
      careinit_();
      marpleinit_();
      if (RecoOpt.CheckActiveAlg("RITER")) {
	Riter_init_();
      }
      init = 1;
    }

    zfpar_.fit = 0;

    rgmaintr_();   // main tracker reconstruction

    if (RecoOpt.CheckActiveAlg("CATS")) {
      // here cats should be called
      CatsEvent();
    }
    else if (RecoOpt.CheckActiveAlg("HOLMES")) {
      // here holmes should be called
      HolmesEvent();
    }
    else {
      // ideal SVD reco
      ArteTable<RSEG>::iterator mainTrEnd = ArteTable<RSEG>::end();
      zfpar_.segstor = 1;
      rgideal_(1.,8,0.,190.,0);
      rcsetp0(mainTrEnd,ArteTable<RSEG>::end());  // clear momentum info
    }

    //cout << " call tfunidm" << endl;

    // matching VDS/main tracker
    if (RecoOpt.CheckActiveAlg("MARPLE")) {
      matchsvdmt_(); // match with marple
    }
    else {
      tfunidm_();    // ideal match SVD-main tracker
    }

    //cout << " call careevnt1" << endl;
    careevnt1_();  // ECAL cluster reconstruction

    //cout << " call mtchrscr" << endl;
    mtchrscr_();

    //cout << " call careevnt2" << endl;
    careevnt2_();  // set relations

    //cout << "call murecb" << endl;
    murecb_();     // continuation into muon system

    zfpar_.fit = 2;
    //cout << " call tfbuild" << endl;
    tfbuild_();    // refit and book RTRA    

    //cout << " call tfelid" << endl;
    rcelid_();     // electron identification

    // locate and attach bremsstrahlung photons 
    // rcbrems_();

    //cout << " call rcmuid" << endl;
    rcmuid_();     // muon identification

    // store neutral particles
    ArteTable<RTRA>::iterator beforeNeutrals = ArteTable<RTRA>::end();
    ECALDetector* ecal = ECALDetector::instance();
    ecal->storeNeutrals();  // store neutral particles
    //    cout << ArteTable<RTRA>::end() - beforeNeutrals
    //	 << " neutrals stored" << endl;

    if (RecoOpt.CheckActiveAlg("RIRE")) {
      RICHrec();    // RICH reconstruction
    }
    else if (RecoOpt.CheckActiveAlg("RITER")) {
      // here RITER should be called
      Riter_evnt_();
    }

    if (RecoOpt.CheckActiveAlg("VERTEX")) {
      rcvert_();     // reconstruct vertices
    }

    //rcrings_();

  }
}
