// -*-C++-*-
#ifndef __VESSEL_H__
#define __VESSEL_H__

#include "rich/ricutil.h"
#include "rich/SphericalMirror.h"
#include "rich/PlaneMirror.h"
#include "rich/PhotonDetectorEntryWindow.h"

 
// ---------------------------------------------------------------------------

class RICHVessel {
Tube eshroud,pshroud;
DiElectric* radiator;
static int classCounter;

public:
/// photon tracing through the radiator over a the distance 'l'
  int traceonly(Photon* p,double l);   
/// photon tracing through the radiator over a the distance 'l'
  int trace(Photon* p,double l);   
/// return the Tube describing the electron beam shroud (See class Tube).
  inline const Tube& ElectronBeamShroud() const { return eshroud; }
/// return the Tube describing the proton beam shroud (See class Tube).
  inline const Tube& ProtonBeamShroud()   const { return pshroud; }
/// set the Tube describing the electron beam shroud (See class Tube).
  inline       Tube& ElectronBeamShroud()       { return eshroud; }
/// set the Tube describing the proton beam shroud (See class Tube).
  inline       Tube& ProtonBeamShroud()         { return pshroud; }
//  inline const DiElectric& Radiator()     const { return *radiator; }
/// returns the dielectric material describing the radiator
  inline       DiElectric& Radiator()           { return *radiator; }
/// set the dielectric material describing the radiator
  inline       void  Radiator(DiElectric* r)    { radiator=r; }


SphericalMirror*           spherical_mirror;
PlaneMirror*              plane_mirror;
PhotonDetectorEntryWindow* photondetector_window;

/// return 1, if the photon could be successfully trace until it leaves the photon detector entry window, else a zero is returned
int raytrace(Photon *p);
/// transformation of v, which removes the planar mirror and the rotation of the spherical mirror.
void track_upper(StraightLine* t,double *l);
void track_lower(StraightLine* t,double *l);
int  geometry_trace(Photon* p);
void MaxEfficiency(Table& refl,int numpoint,int mode);
/// print the configuration of the vessel to an ASCII file (vessel.dat) .
void print();
/// constructor reading from an ASCII-FILE produced using 'print'.
RICHVessel(char* path=0);
/// destructor
~RICHVessel();
};

// ---------------------------------------------------------------------------

extern RICHVessel   *RICH_Vessel;

  
// ---------------------------------------------------------------------------

#endif
