*CMZ :          01/10/96  19.12.58  by  T. Oest
*-- Author :    T. Oest   11/04/96

      SUBROUTINE V0ST1(TRK1, TRK2, ZINT, SZINT, CH2, 
     +                 IRET)

************************************************************************
*
* Description: This routine calculates the point of intersection for two
*              straight tracks in (x,z) and (y,z) and combines it
*
* Called from:
*
* External references:
*
* Argument in/out description:
*    input
*       TRK1    : first track in MREC
*       TRK2    : first track in MREC
*
*    output
*       Z1      : first (second) point of intersection
*       SZ1     : errors on Z1, Z2
*       CH2     : chi2
*       IRET    : < 0 ERROR
*               : = 1 NO SECOND SOLUTION
*
************************************************************************

      IMPLICIT NONE

#include "arte/Arte.fi"
#include "auto/EV.fi"

      INTEGER           IRET
      INTEGER           TRK1, TRK2, TR(2)
      DOUBLE PRECISION  ZINT, SZINT, CH2

      INTEGER           INDEX(5,5), I, J, IERROR
      REAL*4            ZMAG
      DOUBLE PRECISION  TX(2), TY(2), X(2), Y(2), Z(2), ZOUT
      DOUBLE PRECISION  POUT(5),VTRK1(5,5),VTRK2(5,5)
      DOUBLE PRECISION  ZX, ZY, SX, SY, DDTX, DDTY
      DOUBLE PRECISION  P(2), PX(2), PY(2), PZ(2)

#include "arte/Arte.ff"
#include "auto/EV.ff"

      DATA ZMAG /200./
      DATA INDEX /  1,  2,  4,  7, 11,
     +              2,  3,  5,  8, 12,
     +              4,  5,  6,  9, 13,
     +              7,  8,  9, 10, 14,
     +             11, 12, 13, 14, 15 /
************************************************************************
      IRET   = 1
C-----------------------------------------------------------------------
C  fill in track parameter
C-----------------------------------------------------------------------

      IF (FRTRA_ZF(TRK1).LE. FRTRA_ZF(TRK2)) THEN
       TR(1)  = TRK1
       TR(2)  = TRK2
      ELSE
       TR(1)  = TRK2
       TR(2)  = TRK1
      ENDIF

      DO I=1,2
       J       = TR(I)
       TX(I)   = DBLE( FRTRA_TXF(J) )
       TY(I)   = DBLE( FRTRA_TYF(J) )
       X(I)    = DBLE( FRTRA_XF(J) )
       Y(I)    = DBLE( FRTRA_YF(J) )
       Z(I)    = DBLE( FRTRA_ZF(J) )
       P(I)    = DBLE(1./ABS(FRTRA_PF(J)))
       PZ(I)   = P(I)/SQRT(1.+FRTRA_TXF(J)**2+FRTRA_TYF(J)**2)
       PX(I)   = PZ(I) * FRTRA_TXF(J)
       PY(I)   = PZ(I) * FRTRA_TYF(J)
      ENDDO

      IF (FRTRA_ZF(TRK1).NE.FRTRA_ZF(TRK2) .AND. Z(2).GT.ZMAG) THEN
       ZOUT = Z(1)
       CALL V0SWIM (.FALSE., TR(2), ZOUT, POUT, VTRK2, IERROR)
       IF (IERROR.NE.0) THEN
        IRET = -1
        GOTO 9000
       ENDIF
       Z(2)  = POUT(1)
       X(2)  = POUT(2)
       Y(2)  = POUT(3)
       TX(2) = POUT(4)
       TY(2) = POUT(5)
       PZ(2)   = P(2)/SQRT(1.+POUT(4)**2+POUT(5)**2)
       PX(2)   = PZ(2) * POUT(4)
       PY(2)   = PZ(2) * POUT(5)
      ELSE
       DO I=1,5
       DO J=1,5
        VTRK2(I,J) = FRTRA_CVF(INDEX(I,J),TR(2))
       ENDDO
       ENDDO
      ENDIF

       DO I=1,5
       DO J=1,5
        VTRK1(I,J) = FRTRA_CVF(INDEX(I,J),TR(1))
       ENDDO
       ENDDO
C-----------------------------------------------------------------------
C  LOOP
C-----------------------------------------------------------------------
      DDTX = 1./(TX(1)-TX(2))
      DDTY = 1./(TY(1)-TY(2))
      ZX  = (X(2)-X(1)+Z(1)*TX(1)-Z(2)*TX(2))*DDTX
      ZY  = (Y(2)-Y(1)+Z(1)*TY(1)-Z(2)*TY(2))*DDTY

      SX  =   DDTX**2 * VTRK1(1,1) + 2.*Z(1)*TX(2)/DDTX**3 * VTRK1(1,3)
     +      + (Z(1)*TX(2)*DDTX)**2 * VTRK1(3,3)
     +      + DDTX**2 * VTRK2(1,1) - 2.*Z(2)*TX(1)/DDTX**3 * VTRK2(1,3)
     +      + (Z(2)*TX(1)*DDTX)**2 * VTRK2(3,3)

      SY  =   DDTY**2 * VTRK1(2,2) + 2.*Z(1)*TY(2)/DDTY**3 * VTRK1(2,4)
     +      + (Z(1)*TY(2)*DDTY)**2 * VTRK1(4,4)
     +      + DDTY**2 * VTRK2(2,2) - 2.*Z(2)*TY(1)/DDTY**3 * VTRK2(2,4)
     +      + (Z(2)*TY(1)*DDTY)**2 * VTRK2(4,4)




      SZINT = 1./(1./SX + 1./SY)
      ZINT  = (ZX/SX + ZY/SY)*SZINT
      SZINT = SQRT(SZINT)

      CH2   = (ZINT-ZX)**2/SX + (ZINT-ZY)**2/SY


c      PRINT*,' FIRST  SOLUTION ',ZX, SX
c      PRINT*,' SECOND SOLUTION ',ZY, SY
c      PRINT*,'        SOLUTION ',ZINT, SZINT, CH2

 9000 END

************************************************************************
* End of V0ST1
************************************************************************
