#ifndef __WAX_CHANNEL_HH__
#define __WAX_CHANNEL_HH__

#ifndef __WAX_BASICTYPES_HH__
#include "BasicTypes.hh"
#endif

#ifndef __OTR_DECODE_H__
#include "OTrDecode.h"
#endif

class waxChannel
{
public:
  // typedefs, enumerates
  // constructors, destructors, assignment
  waxChannel() {}
  waxChannel(OTrWire& wire) : fWire(wire) {}
  ~waxChannel() {}

  // setters and getters
  
  // decoding the wire structure
  //  const OTrWire&    GetWire() const { return fWire ; }
  TdcCount_t     GetTdcCount()     const { return fWire.dtime; }
  TdcCount_t     GetTZeroCount()   const { return fWire.tzero; }
  Status_t       GetStatus()       const { return fWire.status; }
  GedeIndex_t    GetGedeIndex()    const { return fWire.GEDE; }
  WireIndex_t    GetWireIndex()    const { return fWire.cell; }
  AsdIndex_t     GetAsdIndex()     const { return fWire.ASD; }
  ChannelIndex_t GetChannelIndex() const { return fWire.wire; }
  void SetTZeroCount(TdcCount_t tzc) { fWire.tzero=tzc; }
  void SetTZero(DriftTime_t tz) { fWire.tzero=int(128+4*tz+0.5) ; }

  // decoding the tdccount
  DriftTime_t GetTZero()      const { return GetTZero(GetTZeroCount()); }
  DriftTime_t GetDriftTime()  const {  
    return GetDriftTime(GetTdcCount(),GetTZeroCount()) ; }

  static DriftTime_t GetTZero(TdcCount_t tzcount) {  
    return 0.25*(tzcount-128) ; }
  static DriftTime_t GetDriftTime(TdcCount_t tdc,TdcCount_t tzcount) {
    return tdc+GetTZero(tzcount) ; }
  static const waxChannel* GetChannel(AsdIndex_t asdid, ChannelIndex_t chid) {
    return (const waxChannel*)(fgDecodeTable->ASD8s[asdid]->wire+chid) ; }
  static void SetDecodeTable(OTR_Decode* d) { fgDecodeTable=d ; }
private:
  OTrWire fWire ;
  static OTR_Decode* fgDecodeTable ;
} ;


#endif
