#ifndef __WAX_EVENTBUILDER_HH__
#define __WAX_EVENTBUILDER_HH__

#ifndef __STRING__
#include <string>
#endif
#ifndef ROOT_Rtypes
 #define BIT(n)       (1 << (n))
#endif
#ifndef __OTR_DECODE_H__
#include "OTrDecode.h"
#endif

class waxEventBuilder;
class waxEvent ;
class waxGedeManager ;
class waxDSRManager ;

class waxEventBuilder
{
public:
  // typedefs, enumerates
  enum EDataStream { kDS_DOTR      =BIT(1),    // data from DOTR bank
		     kDS_FARM      =BIT(2),    // datafile in Farm (l4) format 
		     kDS_STANDALONE=BIT(3),    // datafile in StandAlone format
		     kDS_MC        =BIT(4),    // monte carlo data (from HITB bank)
		     kDS_DST       =BIT(5)} ;  // dst data (from HITB bank)
  enum EDataFormat { kDF_DAQ       =BIT(6),    // DOTR, FARM or STANDALONE
		     kDF_HITB      =BIT(7) } ; // MC or DST

  // constructors, destructor, assignment
  waxEventBuilder() ;
  virtual ~waxEventBuilder() ;
  
  // setters and getters
  waxEvent* GetEvent() {return fEvent; }
  const OTR_Decode* GetDecodeTable() const { return fDecodeTable ; }
  const waxGedeManager* GetGedeManager() const { return fGedeManager ; }
  const waxDSRManager* GetDSRManager() const { return fDSRManager ; }
  int GetDataType() const { return fDataType ; }
  int GetNProcessedEvents() { return fNProcessedEvents ; }

  static waxEventBuilder* GetMe() { return fgMe ; }
  static const std::string& GetConfigDirName() { return fgConfigDirName ; }
 
  // other methods
  waxEvent* ProcessOneEvent() ;
  int GetRunNumber() ;
  int GetEventNumber() ;
private:
  // private methods
  void EraseCurrentEvent() ;
  waxEvent* DAQCreateEvent() ;
  waxEvent* HITBCreateEvent() ;
  static int GetOuterTrackerCMP() { return 2 ; }
  int GetDataFormat() const { return GetDataType() & (kDF_DAQ+kDF_HITB) ; }
  int GetDataStream() const { 
    return GetDataType() & (kDS_DOTR+kDS_FARM+kDS_STANDALONE+kDS_MC+kDS_DST) ; }

  // data members
  OTR_Decode* fDecodeTable ;
  waxGedeManager* fGedeManager ;  // owner of the gedes
  waxDSRManager* fDSRManager ;    // owner of the rt relations
  waxEvent* fEvent ;                  // contains the hits
  int fDataType ;                // DAQ/HITB + DOTR/FARM/STANDALONE/MC/DST
  int fNProcessedEvents ;        // number of processed events
  static waxEventBuilder* fgMe ;
  static std::string fgConfigDirName ;
} ;

#endif
