#ifndef __WAX_HITB_HH__
#define __WAX_HITB_HH__

#ifndef __WAX_HIT_HH__
#include "Hit.hh"
#endif

class waxHITB
{
public:
  // constrcutors, destructor
  waxHITB(ArtePointer<HITB> hitb) : fHITB(hitb) {}
  static waxHITB* Create(ArtePointer<HITB> hitb) ;
  waxHITB() {}

  // data from hitb bank
  ArtePointer<HITB> GetHITB() const { return fHITB ; }
  GedeIndex_t GetGedeIndex() const { return fHITB->gede.index() ; }
  WireIndex_t GetWireIndex() const { 
    return waxHit::GetWireIndex(fHITB->chid) ; }
  AsdIndex_t GetAsdIndex() const { 
    return waxHit::GetAsdIndex(fHITB->chid) ; }
  ChannelIndex_t GetChannelIndex() const {
    return waxHit::GetChannelIndex(fHITB->chid) ; }
      
  const waxGede* GetGede() const {
    return waxHit::GetGedeManager()->GetGede(GetGedeIndex()) ; }
  float GetRawX() const { return fHITB->x ; }
  float GetRawZ() const { return fHITB->z ; }
  float GetRawY1() const { return GetGede()->GetRawY1(GetWireIndex()) ; }
  float GetRawY2() const { return GetGede()->GetRawY2(GetWireIndex()) ; }
  int GetMonoLayer() const { 
    return GetWireIndex()%(GetGede()->GetNMonoLayers()) ; }
  float GetDriftRadius() const { return fabs(fHITB->q) ; }
  float GetDriftRadiusError() const { return fHITB->r ; }
  TdcCount_t GetTdcCount() const { return int(fHITB->y+0.5) ; }

  // data from wire structure (if availlable)
  const waxChannel* GetChannel() const {
    return waxChannel::GetChannel(GetAsdIndex(),GetChannelIndex()) ; }
  DriftTime_t GetDriftTime() const { 
    return GetChannel()->GetDriftTime() ; }
  //   DriftTime_t GetTZero() const { 
  //     return GetChannel()->GetTZero() ; }
  
  // other methods
#ifdef __NEW_ARTE__
  ArtePointer<TAHB> AppendTAHB() ;
#endif
private:
  ArtePointer<HITB> fHITB ;
} ;

#endif
