#ifndef __OTR_DECODE_H__
#define __OTR_DECODE_H__

#ifdef __cplusplus
extern "C" 
{
#endif
#include <stdio.h>
#include "component.h"
#include "grr.h"
#include "etb.h"

#define MAXASD 7300
#define MAXFED 65535
#define COMPID 0x400
#define MAXTDC 913

typedef struct{
  unsigned char  dtime;   
  unsigned char  tzero;
  unsigned char  status;
  unsigned short GEDE;     /* GEDE id                        */
  unsigned char  cell;     /* SubLayer + 4*(Cell in GEDE)    */  
  unsigned short ASD;      /* ASD8 index                     */
  unsigned char  wire;     /* Wire on ASD                    */
} OTrWire;

typedef struct{
  unsigned char   SuperLayer;
  unsigned char   Layer;
  unsigned char   Sector;
  unsigned char   Module;
  unsigned char   ftb_ASD;
  unsigned char   TDC_Connector;
  unsigned short  TDC_FED;
  char   hwc[16];
  OTrWire   wire[16];
} OTrASD8;

typedef struct{
  unsigned short ASD[8];   /*Pointer to ASD8s               */
  unsigned char  VME_num;
  unsigned char  Sharc_Board;
  unsigned char  Sharc_Connector;
  unsigned char  status;
  unsigned short FED;
}OTrTDC;

typedef struct{
  unsigned short fAsdIndex ;
  unsigned char  fChannelIndex ;
}OTrGedeWireStruct ;

typedef struct{
  unsigned char fNWires ;
  OTrGedeWireStruct*  fWires ;
}OTrGedeStruct ;

typedef struct{
  unsigned char  NLayer;
  unsigned char  Nasd[7][13];
  unsigned char  FirstASD[7][13];
  unsigned short pos[7][13];           /*pointer to 1-st ASD in sector  */
  unsigned short *Line;                /*buffer with pointers to ASD    */
}OTrSuperLayer;

typedef struct{
  OTrASD8           *ASD8s[MAXASD];       /*ASD8 information               */
  OTrTDC            *TDCs[MAXTDC];        /*TDC information                */  
  unsigned short    FEDs[MAXFED];         /*Pointer FED->TDCs              */
  OTrSuperLayer     Slayers[32];          /*Pointer SL,Lay,Sect,ASD->FED,ch*/
                                          /*0-15 '-',16-31 '+'side         */  
 // GEDE         *GEDE_info[MAXGEDE];     /*GEDE information               */
  unsigned short fMinGedeIndex ;  
  unsigned short fNGedes ;
  OTrGedeStruct* fGedes ;
  unsigned short* fGedeMap ;
  unsigned short* fGedeMapEnd ;
}OTR_Decode;

/************************************************************************/
/* OTR_Decode *OTr_Decode_init(                                         */
/*     mode               -- 0 for ascii,1 for db data sourse           */
/*     Tables_filename    -- File with list of cable  tables,           */
/*     TDC_table_filename -- "crate+board" to FED correspondence table  */
/*     );                                                               */
/* Return:      Pointer to beginning of table or NULL if error.         */
/* Description: Fills table for fast decoding.                          */
/************************************************************************/
OTR_Decode *OTr_Decode_init(int mode,const char *Tables_filename,
                                   const char *TDC_table_filename);

/************************************************************************/
/* int OTr_decode_TDC_to_ASD(                                           */
/*         OTR_Decode *OTR -- Pointer returned by OTr_Decode_init,      */
/*         unsigned short FED     -- TDC board ID,                      */
/*         unsigned char channel  -- Channel,                           */
/*         char *SL               -- SuperLayer,                        */
/*         char *Layer            -- Layer,                             */
/*         char *Sector           -- Sector,                            */
/*         char *ASD              -- ASD,                               */
/*     );                                                               */
/* Return:      1 or 0 if error.                                        */
/* Description: Convert (FED,Channel)->(SL,Layer,Sector,ASD,Wire).      */
/************************************************************************/
int OTr_Decode_TDC_to_ASD(OTR_Decode *OTR,unsigned short FED,
			  unsigned char channel,char *SL,char *Layer,
			  char *Sector,char *ASD);

/************************************************************************/
/* int OTr_decode_ASD_to_TDC(                                           */
/*         OTR_Decode *Table -- Pointer returned by OTr_decode_TDC_init,*/
/*         unsigned char SL     -- SuperLayer,                          */
/*         unsigned char Layer  -- Layer,                               */
/*         unsigned char Sector -- Sector namber,                       */
/*         unsigned char ASD    -- ASD in sector,                       */
/*         unsigned short *FED   -- FED                                 */
/*         unsigned char *Conn  -- Connector,                           */
/*     );                                                               */
/*                                                                      */
/* Return:      1 or 0 if error.                                        */
/* Description: Convert (SL,Layer,Sector,ASD) ot (FED,Connector).       */
/************************************************************************/
int OTr_decode_ASD_to_TDC(OTR_Decode *OTR,unsigned char SL,
			  unsigned char Layer,unsigned char Sector,
			  unsigned char ASD,unsigned short *FED,
			  unsigned char *Conn);

/************************************************************************/
/*  int OTr_get_event(                                                  */
/*      OTR_Decode *OTR -- Pointer to OTR_Decode struct.                */
/*      int *Nhits      -- Number of hits in event.                     */
/*      Wire *wires[]   -- Array of pointers to Wires.                  */
/*  );                                                                  */
/*  return: 1 or 0 in case of error.                                    */
/************************************************************************/
int OTr_get_event(OTR_Decode *OTR,int *Nhits,OTrWire **wires[]);

int OTr_fill_event(OTR_Decode *OTR,ETB_fed feds[],
		   int *Nhits,OTrWire **wires[],int Nfeds);
/************************************************************************/
/*  int OTr_set_tzero(                                                  */
/*      OTR_Decode *OTR -- Pointer to OTR_Decode struct.                */
/*      unsigned short GEDE  -- GEDE-id                                 */
/*      unsigned char cell   -- cell in GEDE                            */
/*      unsigned char tzero  -- tzero                                   */
/*  );                                                                  */
/*  return: 1 or 0 in case of error.                                    */
/************************************************************************/
int OTr_set_tzero(OTR_Decode *OTR,unsigned short GEDE,unsigned char cell,
		  unsigned char tzero);


/* some internal functions */
int OTr_Decode_TDC_to_ASDindex(OTR_Decode *OTR,unsigned short FED,
			      unsigned char channel,unsigned short *Line);
int OTr_Decode_ASD_to_ASDindex(OTR_Decode *OTR,unsigned char SL,
			       unsigned char Layer,unsigned char Sector,
			       unsigned char ASD,unsigned short *Line);
int OTr_Decode_TDCLine(OTR_Decode *OTR,unsigned short FED,
		       unsigned short *Line);
int OTr_Convert_GEDE_to_ASD(unsigned short GEDE, unsigned char cell,char *SL,
			   char *Layer,char *Sector,char *ASD,char *Wire);
int OTr_Convert_ASD_to_GEDE(int dbl, int w1, int ch, int plus, int *wires, 
             int *mono, int *wire);
void OTr_create_gedes(OTR_Decode *decodeTable) ;
/***************************/

#ifdef __cplusplus
}
#endif

#endif

