#ifndef __WAX_RTRELATION_HH__
#define __WAX_RTRELATION_HH__

#ifndef __SGI_STL_VECTOR
#include <vector>
#endif
#ifndef __STRING__
#include <string>
#endif
#ifndef __OTR_DB_H__
#include "otr_db.h"
#endif

class waxParization ;
class TH1 ;

class waxRTRelation
{
public:
  // typedefs, embedded classes
  enum EPitch { kPitchFive=0, kPitchTen=1 } ;
  typedef float DriftRadius_t ;
  typedef float DriftTime_t ;
  class RTResult {
  public:
    RTResult() : fT(0),fR(0),fdTdR(0),fSigmaR(0) {}
    DriftTime_t   fT ;
    DriftRadius_t fR ;
    float         fdTdR ;
    DriftRadius_t fSigmaR ;
  } ;
  
  // constructors, destructor, assignment
  waxRTRelation() ;
  waxRTRelation(const string& filename) ;
  waxRTRelation(LedaTableIterator<class OTR_r_t>&) ;
#ifdef __OTR_USE_ROOT__
  waxRTRelation(int ledakey, EPitch, int, int, TH1 *, float, float, float) ;
#endif
  waxRTRelation(EPitch pitch) ;
  virtual ~waxRTRelation() ;

  // setters and getters
  DriftRadius_t GetRMin() const { return fgRMin[fEPitch] ; } 
  DriftRadius_t GetRMax() const { return fgRMax[fEPitch] ; } 
  int GetLedaKey() const { return fLedaKey ; }

  // other inline methods
  inline RTResult GetRTResultFromR(const DriftRadius_t r) const ;
  inline RTResult GetRTResultFromT(const DriftTime_t t) const ;
  RTResult CalcRTResultFromR(const DriftRadius_t r) const ;
  RTResult CalcRTResultFromT(const DriftTime_t t) const ;

  // IO
  void Read(const string& filename) ;
  void Write(const string& filename) const ;
  void ReadDB() ;
  void WriteDB() const ;
private:
  inline int GetBinFromR(const DriftRadius_t r) const ;
  inline int GetBinFromT(const DriftTime_t r) const ;
  static int GetNTableBins() { return fgNTableBins ; }
  void FillRTTable() ;

private:
  static DriftRadius_t fgRMin[2] ; // the minimum distance to wire 
  static DriftRadius_t fgRMax[2] ; // the maximum distance to wire 
  int fLedaKey ;                   // key of this rt-relation in leda table
  unsigned char fEPitch ;          // layer pitch (5 or 10 mm)
  waxParization* fTParization ;    // parametrization of t(r) in tdc 
  waxParization* fSParization ;    //         "          V(r) in cm^2

  // temporary data (tables for fast retrieval)
  static int fgNTableBins ; // number of bins in tables
  DriftTime_t ftTRMax ;  // time belonging to the minimum distance to wire
  DriftTime_t ftTRMin ;  // time belonging to the maximum distance to wire
  vector<RTResult> ftRTTable ;
  vector<int> ftMapTable ;
} ;

/*************** inlines ********************/

inline int waxRTRelation::GetBinFromT(const DriftTime_t t) const
{
  int bin=int((t-ftTRMin)/(ftTRMax-ftTRMin)+0.5) ;
  if(bin>=fgNTableBins) bin=fgNTableBins-1 ;
  else if(bin<0) bin=0;
  return ftMapTable[bin] ;
}

inline int waxRTRelation::GetBinFromR(const DriftRadius_t r) const 
{
  int bin = int((r-GetRMin())/(GetRMax()-GetRMin())+0.5) ;
  if(bin<0) bin=0;
  else if(bin>=fgNTableBins) bin=fgNTableBins-1 ;
  return bin ;
}

inline waxRTRelation::RTResult 
waxRTRelation::GetRTResultFromR(const DriftRadius_t r) const 
{
  RTResult rt = ftRTTable[GetBinFromR(r)] ;
  rt.fT += rt.fdTdR*(r-rt.fR) ;
  return rt;
}

inline waxRTRelation::RTResult 
waxRTRelation::GetRTResultFromT(const DriftTime_t t) const 
{
  RTResult rt = ftRTTable[GetBinFromT(t)] ;
  rt.fR += (t-rt.fT)/rt.fdTdR ;
  return rt;
}

#endif
