#include <string>
#include "wax/RTRelation.hh"
#include "wax/DSRManager.hh"
#include "wax/GedeManager.hh"
#include "wax/Gede.hh"
#include "wax/otr_db.h"

#ifdef USE_ARTE
#ifndef __ARTE_HH__
#include "arte/arte.hh"
#endif
#endif

waxDSRManager*
waxDSRManager::fgMe = 0;

void
waxDSRManager::ReadDB()
{
  // first find max index
  int max_irt(0) ;
  int min_irt(1000) ;
  for(LedaTableIterator<class OTR_r_t> 
	ledartIt = LedaTableIterator<class OTR_r_t>();
      ledartIt.valid(); ledartIt.next()) {
    max_irt=max(max_irt,ledartIt->irt) ;
    min_irt=min(min_irt,ledartIt->irt) ;
  }

  // now read rt-relation
  fRTRelations.resize(max_irt-min_irt,0) ;
  fRTIndexMin = min_irt ;
  for(LedaTableIterator<class OTR_r_t> 
	ledartIt = LedaTableIterator<class OTR_r_t>();
      ledartIt.valid(); ledartIt.next()) {
    DSRContainer::iterator rtIt = fRTRelations.begin()
      +(ledartIt->irt-fRTIndexMin) ;
    if(!(*rtIt)) *rtIt = new waxRTRelation(ledartIt) ;
    (*rtIt)->ReadDB() ;
  }

  // last but not least: set pointer of gede to rtrelation
  for(waxGedeManager::GedeContainer::iterator
	gedeIt=fGedeManager->GetGedes().begin() ;
      gedeIt!=fGedeManager->GetGedes().end() ; gedeIt++)
    if(*gedeIt) {
      LedaPointer<class OTR_nede> ledaNedeIt(OTR_nede->get->key((*gedeIt)->GetHWC())) ;
      LedaPointer<class OTR_r_t>  ledaRtIt(ledaNedeIt->tor_t) ;
      waxRTRelation* rtr = GetRTRelationFromRTIndex(ledaRtIt->irt) ;
      (*gedeIt)->SetRTRelation(rtr) ;
      if(!rtr) cout << "Could not find rt relation in db !" << endl ;
    }
  
}

void
waxDSRManager::WriteDB()
{
  // Write dsrs
  for(DSRContainer::const_iterator dsrIt = fRTRelations.begin();
      dsrIt != fRTRelations.end(); dsrIt++) 
    (*dsrIt)->WriteDB() ;
  // Write relation to nede
  for(waxGedeManager::GedeContainer::const_iterator
	gedeIt=fGedeManager->GetGedes().begin() ;
      gedeIt!=fGedeManager->GetGedes().end() ; gedeIt++)
    if(*gedeIt) {
      RelateRows(OTR_nede,tor_t,  key((*gedeIt)->GetGEDE()->hwc),
		 OTR_r_t, tonede, key((*gedeIt)->GetRTRelation()->GetLedaKey()) );
    }
}

// void
// waxDSRManager::FillOTTS()
// {
// }

waxDSRManager::waxDSRManager(const string& configdirname,
			     waxGedeManager* gedemanager) 
  : fRTRelations(2)
{
  
  string dir(configdirname) ;
#ifdef USE_ARTE
  if(ArteTable<EVHD>::begin()->exp <= 0) { // MC run
    cout << "Initializing rtrelations for MC." << endl ;
    string rtfilename = dir + "/caldata/rtrelation/mcrtr05mm.dat" ;
    SetRTRelation05mm(new waxRTRelation(rtfilename)) ;
    rtfilename = dir + "/caldata/rtrelation/mcrtr10mm.dat" ;
    SetRTRelation10mm(new waxRTRelation(rtfilename)) ;
    fMCRTRelation05mm = GetRTRelation05mm() ;
    fMCRTRelation10mm = GetRTRelation10mm() ;
  } else {
#endif //USE_ARTE
    string rtfilename = dir + "/caldata/rtrelation/rtr05mm.dat" ;
    SetRTRelation05mm(new waxRTRelation(rtfilename)) ;
    rtfilename = dir + "/caldata/rtrelation/rtr10mm.dat" ;
    SetRTRelation10mm(new waxRTRelation(rtfilename)) ;
#ifdef USE_ARTE
  }
#endif

  for(waxGedeManager::GedeContainer::iterator 
	gedeIt = gedemanager->GetGedes().begin() ;
      gedeIt != gedemanager->GetGedes().end() ; gedeIt++) 
    if(*gedeIt) {
      switch((*gedeIt)->GetEPitch())
	{
	case waxGede::kFiveMM:
	  (*gedeIt)->SetRTRelation(GetRTRelation05mm()) ;
	  break ;
	case waxGede::kTenMM:
	  (*gedeIt)->SetRTRelation(GetRTRelation10mm()) ;
	  break ;
	}
    }
}
