#include <algorithm>
#include "wax/Event.hh"
#include "wax/Hit.hh"

#ifdef USE_ARTE
 #include <arte/arte.hh>
 #include <arte/artefunc.h>
#endif

//__________________________________________________________________
// waxEvent
//
// Container for:
// * Hits
// Functionality:
// * Read new hits
// * Append to HITB
//
// Author: Wouter Hulsbergen && Ivar Siccama
//__________________________________________________________________

waxEvent::waxEvent() : fHits() {} ;

waxEvent::~waxEvent() { 
  //for(HitContainer::const_iterator hitp=fHits.begin(); 
  //    hitp != fHits.end(); hitp++) delete (*hitp);
}

void
waxEvent::Next(OTR_Decode* decodeTable)
{
  //cout << "waxEvent::Next(...) decodeTable=" << decodeTable << endl ;
  //for(HitContainer::const_iterator hitp=fHits.begin();
  //    hitp != fHits.end(); hitp++) delete (*hitp);
  fHits.erase( fHits.begin(), fHits.end() );
  int nwires(0) ;
  OTrWire **wires(0) ;
  
  if(OTr_get_event(decodeTable,&nwires,&wires)) {
    fHits.reserve(nwires) ;
    for(int i=0; i<nwires; i++)
      fHits.push_back(waxHit((waxChannel*)(wires[i]))) ;
  } else cout << "wax_get_event(...) returns 0 ..." << endl ;
  //SortHits() ;
}

//void
//waxEvent::Add(waxHit* hit) {
//  fHits.push_back(hit) ;
//}

void 
waxEvent::DumpHits(ostream& os) const
{
  os << "waxEvent::DumpHits(...) dumps " << fHits.size() 
     << " hits." << endl ;
  for(HitContainer::const_iterator hitIt=fHits.begin();
      hitIt != fHits.end(); hitIt++)
    os << *hitIt << endl ;
  return ;
}

inline bool compareHit( const waxHit& lhs, const waxHit& rhs )
{
  return (lhs.GetGedeIndex() < rhs.GetGedeIndex()) ||
    ((lhs.GetGedeIndex()==rhs.GetGedeIndex()) 
     && (lhs.GetWireIndex() < rhs.GetWireIndex()));
}

void
waxEvent::SortHits()
{
  sort( fHits.begin(), fHits.end(), compareHit );
}

#ifdef USE_ARTE
void
waxEvent::AppendHITB()  
{
  // Append all hits in this superlayer to the HITB bank. Before
  // calling this function make sure that the hits are sorted with
  // SortHits() and that the GEDE::hit1 and hitl entries are reset.

  HitContainer::iterator hitIt = GetHits().begin() ;
  if(hitIt != GetHits().end()) {
    //ArtePointer<HITB> hitbIt = (*hitIt).AppendHITB();  //old
    ArtePointer<HITB> hitbIt = 
      ArteTable<HITB>::allocate(GetHits().size()) ;
    (*hitIt).AppendHITB(hitbIt) ; //new 
    ArtePointer<GEDE> currentGEDE = hitbIt->gede ;
    currentGEDE->hit1 = hitbIt ;
    ArtePointer<HITB> prevhitbIt=hitbIt ;
    hitIt++ ;
    hitbIt++ ;
    for(; hitIt != GetHits().end() ; hitIt++,hitbIt++) {
      //hitbIt = (*hitIt).AppendHITB();
      (*hitIt).AppendHITB(hitbIt) ;
      if(hitbIt->gede != currentGEDE) {
	currentGEDE->hitl = prevhitbIt ;
	currentGEDE = hitbIt->gede ;
	currentGEDE->hit1 = hitbIt ;
      }
      prevhitbIt = hitbIt ;
    }
    currentGEDE->hitl = prevhitbIt ;
  }
}
#endif
