#include <iostream>
#include <algorithm>
#include "wax/EventBuilder.hh"
#include "wax/GedeManager.hh"
#include "wax/DSRManager.hh"
#include "wax/Event.hh"
#include "wax/Channel.hh"
#include "wax/Gede.hh"

//__________________________________________________________________
// waxEventBuilder
//
// Container for: 
// * waxEvent 
// * waxDecode
// * waxGedeManager
//
// Functionality:
// Building of the waxEvent from the arte HITB bank (dst and monte
// carlo), from the DOTR bank or from the DAQ data files.
//
// Author: Wouter Hulsbergen
//__________________________________________________________________

waxEventBuilder* waxEventBuilder::fgMe = 0;

std::string waxEventBuilder::fgConfigDirName = "/afs/desy.de/group/hera-b/OTR/db" ;

bool waxCompareGede(const unsigned short lhsgedeId, const unsigned short rhsgedeId)
{
  static const waxGedeManager* gedemanager =
    waxEventBuilder::GetMe()->GetGedeManager() ;
  static const int mingedeid =
    waxEventBuilder::GetMe()->GetDecodeTable()->fMinGedeIndex ;
  const waxGede* lhsGede = gedemanager->GetGede(lhsgedeId+mingedeid) ;
  const waxGede* rhsGede = gedemanager->GetGede(rhsgedeId+mingedeid) ;
  int lhsindex = 
    lhsGede->GetModuleId() + 100*
    (lhsGede->GetSectorId()+ 100*
     (lhsGede->GetLayerId() + 10*lhsGede->GetSuperLayerId())) ;
  int rhsindex = 
    rhsGede->GetModuleId() + 100*
    (rhsGede->GetSectorId()+ 100*
     (rhsGede->GetLayerId() + 10*rhsGede->GetSuperLayerId())) ;
  return lhsindex<rhsindex ;
}

waxEventBuilder::waxEventBuilder()
  : fDecodeTable(0), fGedeManager(0), fEvent(0),
    fDataType(0), fNProcessedEvents(0) 
{
  cout << "waxEventBuilder::waxEventBuilder()" << endl ;
  if(fgMe) {
    cout << "Already an waxEventBuilder object!" << endl ;
    return ;
  }
  fgMe = this ;
  int mode = 0;
  fDecodeTable =  
    OTr_Decode_init(mode,
		    "/afs/desy.de/group/hera-b/OTR/db/tables/list_of_tables_hallw",
		    "/afs/desy.de/group/hera-b/OTR/db/tables/tdc_fed_hallw") ;
		    //"/afs/desy.de/group/hera-b/OTR/db/tables/list_of_tables_sep99",
		    //"/afs/desy.de/group/hera-b/OTR/db/tables/tdc_fed_sep99") ;
  cout << "waxEventBuilder::waxEventBuilder() fDecodeTable = " 
       << fDecodeTable << endl ;
  waxChannel::SetDecodeTable(fDecodeTable) ;
  
#ifdef USE_ARTE
  fGedeManager = new waxGedeManager() ;
#else
  fGedeManager = new waxGedeManager(GetConfigDirName()) ;
#endif

  // sort gedemap in decode table
  sort(fDecodeTable->fGedeMap,fDecodeTable->fGedeMapEnd,waxCompareGede) ;
  
  fDSRManager = new waxDSRManager(GetConfigDirName(),fGedeManager) ;
  fEvent = new waxEvent() ;
  cout << "End of waxEventBuilder::waxEventBuilder()" << endl ;
}

waxEventBuilder::~waxEventBuilder()
{
  //delete fDecodeTable ;
  delete fGedeManager ;
  delete fEvent ;
  if(fgMe==this) fgMe = 0;
}

waxEvent*
waxEventBuilder::ProcessOneEvent()
{ 
  //cout << "waxEventBuilder::ProcessOneEvent()" << endl ;
  fEvent->Next(fDecodeTable) ;
#ifdef USE_ARTE
  fGedeManager->ResetHITBPointers() ;
  fEvent->AppendHITB();
#endif //USE_ARTE
  fNProcessedEvents++ ;
  return fEvent ;
}

// #ifndef USE_ARTE
// int 
// waxEventBuilder::GetRunNumber() {
//   int runnr=0;
//   if(GetEventBuffer()) runnr = GetEventBuffer()->GetLogHdr()->run ;
//   return runnr ;
// }

// int waxEventBuilder::GetEventNumber() {
//   int evtnr=0;
//   if(GetEventBuffer()) evtnr = GetEventBuffer()->GetLogHdr()->evt ;
//   return evtnr ;
// }
// #endif
  // Temporary for monte carlo ...
//   if(ArteTable<EVHD>::begin()->exp <= 0) { // MC
//    for(ArteTable<HITB>::iterator hitbIt = ArteTable<HITB>::begin() ;
//        hitbIt != ArteTable<HITB>::end() ; hitbIt++) 
//      if(hitbIt->gede->cmp==2) {
//        //hitbIt->q = fabs(hitbIt->q) ;
//        if(hitbIt->chid==0) {
// 	 ArtePointer<DIGB> digb = hitbIt->digb.begin() ;
// 	 int part = digb->id/10000;
// 	 int c = (part-1)/4;  // wire (0..31)
// 	 int m = (part-1)%4;    // monolayer (0..3), or `tube'
// 	 WireIndex_t wireId = c + m * 32 ; 
// 	 int gedeindex = hitbIt->gede.index() - fDecodeTable->fMinGedeIndex ;
// 	 if(gedeindex >=0 && gedeindex<fDecodeTable->fNGedes ) {
// 	   OTrGedeStruct* gede = 
// 	     fDecodeTable->fGedes + gedeindex ;
// 	   if(wireId<gede->fNWires) {
// 	     OTrGedeWireStruct* wire = gede->fWires + wireId ;
// 	     hitbIt->chid = waxHit::GetCHID(wireId, wire->fAsdIndex, wire->fChannelIndex) ;
// 	   } else {
// 	     hitbIt->chid = waxHit::GetCHID(wireId,0,0) ;
// 	   }
// 	 } else {
// 	   hitbIt->chid = waxHit::GetCHID(wireId,0,0) ;
// 	 }
// 	 // 	 cout << "eventbuilder " << hitbIt << " " << hitbIt->chid << " "
// 	 // 	      << int(wireId) << " " << m << " " << c << endl ;
//        }
//      }
//   }
