#include <iostream>
#include <string.h>
#include "wax/Gede.hh"

//__________________________________________________________________
// waxGede
//
// Container for:
// * pointer to arte GEDE
//
// Functionality:
// * geometry decoding
//
// In the near future this class wil be `reduced' to a pointer to a
// GEDE/NEDE structure (ARTE or standalone) and maybe some alignment
// constants. I would also like to rename it :-)
//
// Author: Wouter Hulsbergen && Ivar Siccama
//__________________________________________________________________

//const float waxGede::fgLayerPitch[2] = {0.500, 1.000} ;
//const float waxGede::fgWirePitch[2]  = {0.870, 1.740} ;
//const float waxGede::fgStereoAngle[3] = {-0.08000,0.00000,0.08000 } ;

int waxGede::fgOTrComponentId = 2;

waxGede::~waxGede()
{
#ifndef USE_ARTE
  if(fGEDE) delete fGEDE ;
#endif
}

waxGede::waxGede() 
  : fGEDE(0), fGEDEIndex(0), fSuperLayerId(0) {}

const waxGede& waxGede::operator=(const waxGede& rhs)
{
#ifdef USE_ARTE
  fGEDE = rhs.fGEDE ;
#else
  if(rhs.fGEDE) {
    if(!fGEDE) fGEDE = new GEDE ;
    *fGEDE = *rhs.fGEDE ;
  } else if(fGEDE) {
    delete fGEDE ;
    fGEDE = 0;
  };
#endif
  fGEDEIndex = rhs.fGEDEIndex ;
  fSuperLayerId = rhs.fSuperLayerId ;
  fSuperLayerName = rhs.fSuperLayerName ;
  fEStereoAngle = rhs.fEStereoAngle ;
  fEPitch = rhs.fEPitch ;
  fEXSide = rhs.fEXSide ;
  return *this ;
}

waxGede::waxGede( const waxGede& rhs ) :
  fGEDE(0)
{
  operator=(rhs) ;
}


#ifdef USE_ARTE
waxGede::waxGede( ArtePointer<GEDE> g ) :
  fGEDE(g), fGEDEIndex(g.index()), fSuperLayerId(0)
{
  fSuperLayerName = ArtePointer<GESL>(GetGEDE()->gesl)->name;
  //  fSuperLayerId = gede()->gesl.index();
  Initialize() ;
}
#endif  //USE_ARTE

waxGede::waxGede( istream& is ) :
  fGEDE(0), fGEDEIndex(0), fSuperLayerId(0)
{ 
  is >> fGEDEIndex;    
  if ( fGEDEIndex < 0 ) {
    cout << "ERROR in waxGede: fGEDEIndex = " << fGEDEIndex << endl;
    exit(1);
  }
#ifdef USE_ARTE
  fGEDE = ArtePointer<GEDE>( fGEDEIndex) ;
#else
  fGEDE = new GEDE ;
#endif
  is >> fGEDE->cmp;
  is >> fGEDE->lay;
  is >> fGEDE->shp;
  is >> fGEDE->xmin >> fGEDE->xmax ;
  is >> fGEDE->ymin >> fGEDE->ymax ;
  is >> fGEDE->zmin >> fGEDE->zmax ;
  is >> fGEDE->sxmn >> fGEDE->sxmx ;
  is >> fGEDE->symn >> fGEDE->symx ;
  is >> fGEDE->cos  >>  fGEDE->sin ;
  for ( int i = 0; i < 9; i++ ) is >> fGEDE->rot[i];
  is >> fGEDE->acc >>  fGEDE->eff ;
  is >> fGEDE->dod ;
  is >> fGEDE->pit ;
  is >> fGEDE->ms ;
  for ( int i = 0; i < 4; i++ ) is >> fGEDE->xfw[i];
  for ( int i = 0; i < 4; i++ ) is >> fGEDE->zfw[i];
  for ( int i = 0; i < 4; i++ ) is >> fGEDE->nwi[i];
  is >> fGEDE->nsl ;
#ifdef USE_ARTE
  int gesl, gmat ;
  is >> gesl >> gmat ;
  fGEDE->gesl= ArtePointer<GESL>(gesl) ;
  fGEDE->gmat= ArtePointer<GMAT>(gmat) ;
#else
  is >> fGEDE->ogesl >> fGEDE->ogmat ;
#endif
  is >> fGEDE->nwl ;
  is >> fGEDE->fwl ;
  is >> fGEDE->xmad >> fGEDE->ymad >> fGEDE->zmad;

  for(  int i = 0; i < 4; i++ ) is >> fGEDE->y1w[i];
  for(  int i = 0; i < 4; i++ ) is >> fGEDE->y2w[i];
 
  // is >> gvol;
  // is >> hwc;
  is >> fGEDE->dedx ;
  is >> fGEDE->gcop ;

  is >> fGEDE->sec ;
  is >> fSuperLayerName ;
#ifdef NEW_GEOMETRY_FILE
  is >> fGEDE->hwc ;
#endif

  Initialize() ;

  if ( !is ) { cout << "waxGede: input stream is at end" << endl; }
}

string waxGede::GetSuperLayerName(SuperLayerId_t slId) 
{
  string name ;
  switch(slId)
    {
    case  1: name = "MC01" ; break ;
    case  2: name = "MC02" ; break ;
    case  3: name = "MC03" ; break ;
    case  4: name = "MC04" ; break ;
    case  5: name = "MC05" ; break ;
    case  6: name = "MC06" ; break ;
    case  8: name = "MC08" ; break ;
    case  9: name = "PC01" ; break ;
    case 10: name = "PC02" ; break ;
    case 11: name = "PC03" ; break ;
    case 12: name = "PC04" ; break ;
    case 13: name = "TC01" ; break ;
    case 14: name = "TC02" ; break ;
    case 17: name = "MC01" ; break ;
    case 18: name = "MC02" ; break ;
    case 19: name = "MC03" ; break ;
    case 20: name = "MC04" ; break ;
    case 21: name = "MC05" ; break ;
    case 22: name = "MC06" ; break ;
    case 24: name = "MC08" ; break ;
    case 25: name = "PC01" ; break ;
    case 26: name = "PC02" ; break ;
    case 27: name = "PC03" ; break ;
    case 28: name = "PC04" ; break ;
    case 29: name = "TC01" ; break ;
    case 30: name = "TC02" ; break ;
    default: name = "Unkown superlayer" ;
    }
  return name ;
}

SuperLayerId_t waxGede::GetSuperLayerId(const char* namechar)
{
  SuperLayerId_t slid=99;
  if     (!strncmp(namechar,"MC01",4)) slid = 1; 
  else if(!strncmp(namechar,"MC02",4)) slid = 2; 
  else if(!strncmp(namechar,"MC03",4)) slid = 3; 
  else if(!strncmp(namechar,"MC04",4)) slid = 4; 
  else if(!strncmp(namechar,"MC05",4)) slid = 5; 
  else if(!strncmp(namechar,"MC06",4)) slid = 6; 
  else if(!strncmp(namechar,"MC08",4)) slid = 8; 
  else if(!strncmp(namechar,"PC01",4)) slid = 11;
  else if(!strncmp(namechar,"PC02",4)) slid = 12;
  else if(!strncmp(namechar,"PC03",4)) slid = 13;
  else if(!strncmp(namechar,"PC04",4)) slid = 14;
  else if(!strncmp(namechar,"TC01",4)) slid = 21; 
  else if(!strncmp(namechar,"TC02",4)) slid = 22;
  //else if(!strncmp(namechar,"PC04",4)) id = 31; // MSL1
  //else if(!strncmp(namechar,"TC01",4)) id = 32; // MSL2
  //else if(!strncmp(namechar,"TC02",4)) id = 33; // mini TC
  else {
    cout << "** ERROR in waxGede::GetSuperLayerId(const char* name = " 
	 << namechar << ")" << endl; 
  }
  return slid ;
}

void waxGede::Initialize()
{
  // if(GetNMonoLayers()>0) {

    // Set EStereoAngle
  if(GetSinStereoAngle() > 0.050 /*rad*/) fEStereoAngle = kPlusFiveDegrees ;
  else if(GetSinStereoAngle() < -0.050) fEStereoAngle = kMinusFiveDegrees ;
  else fEStereoAngle = kZeroDegrees ;
  
  // Set EPitch
  if(GetLayerPitch() < 1.0/*cm*/) fEPitch = kFiveMM ;
  else fEPitch = kTenMM ;

  // Set ESide
  if(GetGEDE()->xfw[0] < fGEDE->xfw[1]) fEXSide = kXSideMinus ;
  else fEXSide = kXSidePlus ;
  
  // determine superlayerid
  fSuperLayerId = GetSuperLayerId(fSuperLayerName.c_str()) ; 
  //}
}  

int
waxGede::GetNWires() const
{
  int n = 0;
  for ( int i = 0; i < GetGEDE()->nsl ; i++ ) n += GetGEDE()->nwi[i];
  return n;
}


// int waxGede::DetermineSection() const
// {
//   // some ARTE GEDE sections are "unphysical", they are used in GEANT
//   // they can easily recognized, since they are very short: 
//   // ... and they have no monolayers !! (wouter)
//   if ( GetNMonoLayers()<1 || GetPMax().y() - GetPMin().y()< 10 ) { return -2; }
//   char* sulName = "0000";
//   int layid = atoi(GetGEDE()->hwc+4) ;
//   int modid = atoi(GetGEDE()->hwc+6) ;
//   int secid = atoi(GetGEDE()->hwc+9) ;
//   //cout << GetGEDE()->hwc << " " << layid << " " 
//   //     << secid << " " << modid << " |  " << layerId() << " " << endl ;
//   if(secid != GetSectorNumber())
//     cout << "Error in geometry file: sector numbers not equal ! " << endl ;
//   int s = secid*100 + modid ;
//   return s;
// }

// void waxGede::Translate( const ThreeVector& p )
// {
//   GetGEDE()->xmin += p.x() ; 
//   GetGEDE()->xmax += p.x() ; 
//   GetGEDE()->ymin += p.y() ;
//   GetGEDE()->ymax += p.y() ; 
//   GetGEDE()->zmin += p.z() ;
//   GetGEDE()->zmax += p.z() ; 

//   for ( int i=0; i < 4; i++ ) {
//     GetGEDE()->xfw[i] += GetCosStereoAngle()*p.x() - GetSinStereoAngle()*p.y() ;
//     GetGEDE()->y1w[i] += GetSinStereoAngle()*p.x() + GetCosStereoAngle()*p.y() ;
//     GetGEDE()->y2w[i] += GetSinStereoAngle()*p.x() + GetCosStereoAngle()*p.y() ;
//     GetGEDE()->zfw[i] += p.z();
//   };
// }

// void waxGede::LongDump() const
// {
//   cout << "GEDE index = " << GetGEDEIndex() << " " 
//        << GetSuperLayerName() << endl;
//   cout << "sl = " << superLayerId() 
//        << " lay = " << layerId() 
//        << " section = " << determineSection() << endl; 
//   cout << "pmin = " << GetPMin() << " pmax = " << GetPMax() << endl;
//   cout << "wire pitch = " << GetEPitch() << " " << GetWirePitch() << endl;
//   cout << "stereo angle = " << GetEStereoAngle() << endl;
//   cout << "nr of mono layers = " << GetNMonoLayers() << endl;
//   cout << " cos = " << GetGEDE()->cos << " sin = " << GetGEDE()->sin << endl;
// }

ostream & operator << (ostream& os, const waxGede& g)
{ 
  os << " gede = " << g.GetGEDEIndex() 
     << " " << g.GetSuperLayerId() 
     << " " << g.GetLayerId() 
     << " " << g.GetSectorId() << endl;
  return os;
}

